/*
* ============================================================================
*  Name     : CSTTConnector
*  Created  : 29.08.2006 by Zsolt Pszota 
*  Description:
*     
*  Version  :
*  Copyright: 
* ============================================================================
*/

//#define PROXYCONNECTOR_ENABLED

#ifndef STTCONNECTOR_H
#define STTCONNECTOR_H

// INCLUDES
#include <e32base.h>
#include <es_sock.h>
#include <in_sock.h>
#include "STTConnectorEvents.h"
#ifdef PROXYCONNECTOR_ENABLED
 #include "STTTransferSocketInitializator.h"
#endif
#include "STTSocketCollector.h"
#include "STTBaseData.h"
#include "STTTimeOutHandler.h"
//#include "STDefs.h"

#ifdef EKA2
#include "STTImeiInfoSupplier.h"
#endif


// CONSTANTS

// FORWARD DECLARATIONS

// CLASS DECLARATION

enum TConnectorStates 
{
	EInitializing = 1,
	EInitialized,
	EConnecting,
	EReceivingInitialization,
	ESendingInitialization,
	EWaitingForConnections,
	EProcessingIncomingMessage
};

enum TConnectorListenerStates 
{
	EOffline = 1000,
	EWaitingForConnection,
	EProcessingConnection
};

enum TConnectorSimpleStates 
{
	EConOffline = 2000,
	EConConnecting,
	EConInitializing,
	EConConnected,
	EConnectedAndInitialized
};

enum TConnectorType
{
	EConProxy = 3000,
	EConListener
};

/**
*  CSTTConnectorProxy class.
* 
*/
class CSTTConnector : public CActive, public MTimeOutObserver
#ifdef EKA2
,public MImeiInfoObserver
#endif
    {
    public: // Constructors and destructor

		static CSTTConnector* NewL(RSocketServ* aSocketServ, MSTTSocketCollector* aSocketCollector, CSTTBaseData* aBaseData, const TDesC& aIMEIPostFix, RConnection* aConnection=NULL, MSTTConnectorEvents* aConnectorEventHandler=NULL);

        /**
        * Destructor.
        */
        ~CSTTConnector();

	protected:
		// -- active object --
		void DoCancel();
        /**
        * EPOC default constructor.
        */
        void ConstructL(const TDesC& aIMEIPostFix);
		CSTTConnector(RSocketServ* aSocketServ, MSTTSocketCollector* aSocketCollector, CSTTBaseData* aBaseData, RConnection* aConnection, MSTTConnectorEvents* aConnectorEventHandler);

#ifdef EKA2
	private:
		void ImeiInfoReceivedL(CTelephony::TPhoneIdV1Pckg& aImeiInfo);
		void GettingImeiFailedL();
#endif

#ifdef PROXYCONNECTOR_ENABLED
		void CheckAndAddTransferSocket(TPtrC8 aConnectionId);
		void ReleaseAllTransferSocket();
#endif

		void MyAppend8_16(TDes8 &aTarget, TDesC16 &aSource);
		void MyAppend16_8(TDes16 &aTarget,TDesC8 &aSource);

    public: // New functions
    
    	/**
    	 * @return KErrNone if no errors occured
    	 */
		TInt StartL(TConnectorType aConnectorType, RConnection* aConnection = NULL);
		
		/**
		 * @param aLocalAddress contains the address of the device on which it accepts incoming connections
		 *        if getting the address fails it will be set to 0 
		 * 
    	 * @return KErrNone if no errors occured
    	 */
		TInt StartL(TConnectorType aConnectorType, RConnection* aConnection, TInetAddr& aLocalAddress);

		void Stop();
		void ProcessConnection();
		void SetConnectionTimeOut(TInt aConnectionTimeOut);
		TInt GetConnectionTimeOut();

	public: // from MSTTConnectorBase
		TConnectorSimpleStates GetConnectorState();
		HBufC* GetIMEI();
		TInt GetLastErrorCode();

	public: // from MTimeOutObserver
		void TimeOutExpired();

	private:
		TConnectorStates GetState();
		
		TInt GetLocalIPAddressL(TInetAddr& aAddr, RConnection* aConnection = NULL);


#ifdef PROXYCONNECTOR_ENABLED
		void ReceiveInitialization();
		void ContinueReading();
		void ReadIncoming();
		void SendInitialization();
		void WaitForFirstConnection();
		void WaitForNextConnection();
		void ReadIncomingMessage();
		void ProcessIncoming();
#endif

    private:
		// -- active object --
		void RunL();

    private: // Data
		TConnectorSimpleStates				iSimpleState;
		HBufC*								iIMEI;
		HBufC* 								iIMEIPostFix;
		RConnection*						iConnection;
		TInetAddr 							iProxyAddress;
		
		#ifdef EKA2
		CImeiInfoSupplier*					iImeiInfoSupplier;
		#endif
		
		TBool								iGettingImei;
		TBool								iConnectIssued;
		TInt								iLastErrorCode;
		TInt								iConnectionTimeOut;
		RSocketServ*						iSocketServ;
		RSocket								iSocket;
		CSTTBaseData*						iBaseData;
		MSTTConnectorEvents*				iConnectorEventHandler;
		
#ifdef PROXYCONNECTOR_ENABLED
		CSTTTransferSocketInitializator*	iTransferSocket;
#endif
		MSTTSocketCollector*				iSocketCollector;

		CTimeOutHandler*					iTimeOutHandler;
		TConnectorStates					iState;
		TBuf8<1024>							iTransferingBuffer;
		TBuf8<1024>							iTemporaryBuffer;
		TSockXfrLength						iRecievedLength;

		TConnectorListenerStates			iListenerState;
		RSocket								iSocketListener;
		TConnectorType						iConnectorType;

		RSocket*							iBlankSocket;
    };

#endif

// End of File
