/*
* ============================================================================
*  Name     : STTBaseData from STTBaseData.h
*  Part of  : SymTorrentTracker
*  Created  : 06.07.2006 by Zsolt Pszota
*  Version  :
*  Copyright: 
* ============================================================================
*/

// INCLUDE FILES
#include  "STTBaseData.h"

// Constructors and destructor ------------------------------------------------
CSTTBaseData::CSTTBaseData()
{
}

void CSTTBaseData::ConstructL()
{
	iPartnerPort	= -1;
	iServicePort	= -1;
	iConnectionPort = -1;
	iProxyAddress  = NULL;

	iItemListHead = CSTTTorrentListItem::NewL(_L(""),_L8(""));
}

EXPORT_C CSTTBaseData::~CSTTBaseData()
{
	if (iProxyAddress!=NULL)
		delete iProxyAddress;
	DeleteAllListItem();
	delete iItemListHead;
}

EXPORT_C CSTTBaseData* CSTTBaseData::NewL()
{
	CSTTBaseData * self=new(ELeave) CSTTBaseData();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}

// New functions --------------------------------------------------------------

EXPORT_C CSTTTorrentListItem* CSTTBaseData::InsertListItemL(TPtrC aTorrentFileName, TPtrC8 aInfoHash)
{
	TInt compare=-1;
	CSTTTorrentListItem* lastitem = iItemListHead;
	CSTTTorrentListItem* tempitem = iItemListHead->GetNextSibling();

	while (tempitem!=NULL && (compare=tempitem->GetTorrentFileName().Compare(aTorrentFileName))<0)
	{
		lastitem = tempitem;
		tempitem = tempitem->GetNextSibling();
	}
	if (compare>0 || tempitem==NULL)
		return lastitem->InsertAfterL(aTorrentFileName,aInfoHash);
	else
		return NULL;
}

EXPORT_C TBool CSTTBaseData::DeleteListItem(TPtrC aTorrentFileName)
{
	TBool res = EFalse;
	CSTTTorrentListItem* lastitem = iItemListHead;
	CSTTTorrentListItem* tempitem = iItemListHead->GetNextSibling();
	while (tempitem!=NULL && !res)
	{
		if (tempitem->GetTorrentFileName().Compare(aTorrentFileName)==0)
		{
			lastitem->DeleteNext();
			res = ETrue;
		}
		else
		{
			lastitem = tempitem;
			tempitem = tempitem->GetNextSibling();
		}
	}
	return res;
}

EXPORT_C void CSTTBaseData::DeleteAllListItem()
{
	while (iItemListHead->GetNextSibling()!=NULL)
		iItemListHead->DeleteNext();
}

EXPORT_C CSTTTorrentListItem* CSTTBaseData::GetTorrentListFirstItem()
{
	return iItemListHead->GetNextSibling();
}

// Getter and setter methods --------------------------------------------------

EXPORT_C TInt CSTTBaseData::GetPartnerPort()
{
	return iPartnerPort;
}

EXPORT_C void CSTTBaseData::SetPartnerPort(TInt aPartnerPort)
{
	iPartnerPort = aPartnerPort;
}

EXPORT_C TInt CSTTBaseData::GetServicePort()
{
	return iServicePort;
}

EXPORT_C void CSTTBaseData::SetServicePort(TInt aServicePort)
{
	iServicePort = aServicePort;
}

EXPORT_C TInt CSTTBaseData::GetConnectionPort()
{
	return iConnectionPort;
}

EXPORT_C void CSTTBaseData::SetConnectionPort(TInt aConnectionPort)
{
	iConnectionPort = aConnectionPort;
}

EXPORT_C TPtr CSTTBaseData::GetProxyAddress()
{
	return iProxyAddress->Des();
}

EXPORT_C void CSTTBaseData::SetProxyAddress(TPtrC aServerName)
{
	if (iProxyAddress!=NULL)
		delete iProxyAddress;
	iProxyAddress = aServerName.AllocL();
}
