/*
 * ============================================================================
 *  Name     : STTConnector from STTConnector.h
 *  Part of  : SymTorrentTest
 *  Created  : 30.05.2006 by Zsolt Pszota
 *  Version  :
 *  Copyright: 
 * ============================================================================
 */

// INCLUDE FILES
#include "STTConnector.h"
//#include "STLogger.h"
#include <es_sock.h>

#ifndef __WINS__
	#ifndef EKA2
		#include <plpvariant.h>
	#endif
#endif


// -- CSTTConnector -----------------------------------------------------------
// Constructors and destructor ------------------------------------------------
CSTTConnector::CSTTConnector(RSocketServ* aSocketServ, MSTTSocketCollector* aSocketCollector, CSTTBaseData* aBaseData, RConnection* aConnection, MSTTConnectorEvents* aConnectorEventHandler)
:CActive(EPriorityNormal),iConnectorEventHandler(aConnectorEventHandler),iSocketCollector(aSocketCollector),iConnection(aConnection)
{ 
	iSocketServ				= aSocketServ;
	iBaseData				= aBaseData;
	iSimpleState			= EConOffline;
	iLastErrorCode			= 0;
	iConnectionTimeOut		= 30*1000*1000;
	iTimeOutHandler			= NULL;
	iBlankSocket			= NULL;
}

void CSTTConnector::ConstructL(const TDesC& aIMEIPostFix)
{
	CActiveScheduler::Add(this);

    // get imei
	iIMEI = NULL;
	
	iIMEIPostFix = aIMEIPostFix.AllocL(); 
	
	#ifdef __WINS__
		iIMEI = _L("1234567890").AllocL();
	#else
		#ifdef EKA2
			iIMEI = KNullDesC().AllocL();
			iGettingImei = ETrue;
			iImeiInfoSupplier = CImeiInfoSupplier::NewL();
			iImeiInfoSupplier->SetObserver(this);
			iImeiInfoSupplier->GetImeiInfoL();
		#else
			TPlpVariantMachineId imei;
			PlpVariant::GetMachineIdL(imei);
			iIMEI = imei.AllocL();
		#endif
	#endif
		
    // initial values
	iState = EInitialized;
#ifdef PROXYCONNECTOR_ENABLED
	iTransferSocket = NULL;
#endif
}

CSTTConnector::~CSTTConnector()
{
	delete iTimeOutHandler;
	
	Cancel();
	if (iSimpleState!=EConOffline)
	
#ifdef PROXYCONNECTOR_ENABLED
		if (iConnectorType==EConProxy)
		{
			iSocket.Close();
		}
		else
		{
#endif
			iSocketListener.Close();
			delete iBlankSocket;
			iBlankSocket = NULL;
#ifdef PROXYCONNECTOR_ENABLED
		}
	
	ReleaseAllTransferSocket();
#endif

	delete iIMEI;
	
#ifdef EKA2
	if (iImeiInfoSupplier)
		delete iImeiInfoSupplier;
#endif

	delete iIMEIPostFix;
}

CSTTConnector* CSTTConnector::NewL(RSocketServ* aSocketServ, MSTTSocketCollector* aSocketCollector, CSTTBaseData* aBaseData, const TDesC& aIMEIPostFix, RConnection* aConnection, MSTTConnectorEvents* aConnectorEventHandler)
{
	CSTTConnector * self=new(ELeave) CSTTConnector(aSocketServ, aSocketCollector, aBaseData, aConnection, aConnectorEventHandler);
	CleanupStack::PushL(self);
	self->ConstructL(aIMEIPostFix);
	CleanupStack::Pop();
	return self;
}

#ifdef EKA2
// MImeiInfoObserver methods --------------------------------------------------

void CSTTConnector::ImeiInfoReceivedL(CTelephony::TPhoneIdV1Pckg& aImeiInfo)
{
	delete iIMEI;
	iIMEI = NULL;

	TBuf<120> tempbuf;
	tempbuf.Copy(aImeiInfo().iSerialNumber);
	tempbuf.Append(iIMEIPostFix->Des());
	iIMEI = tempbuf.AllocL();

	iGettingImei = EFalse;
	
	if (iConnectIssued)
	{
		iConnectIssued = EFalse;
		StartL(iConnectorType,iConnection);
	}
}

void CSTTConnector::GettingImeiFailedL()
{
	iGettingImei = EFalse;
	
	if (iConnectIssued)
	{
		iConnectIssued = EFalse;
		StartL(iConnectorType,iConnection);
	}
}

#endif

// Getter and setter methods --------------------------------------------------
HBufC* CSTTConnector::GetIMEI()
{
	return iIMEI;
}

TConnectorStates CSTTConnector::GetState()
{
	return iState;
}

TConnectorSimpleStates CSTTConnector::GetConnectorState()
{
	return iSimpleState;
}

TInt CSTTConnector::GetLastErrorCode()
{
	return iLastErrorCode;
}

void CSTTConnector::SetConnectionTimeOut(TInt aConnectionTimeOut)
{
	iConnectionTimeOut = aConnectionTimeOut;
}

TInt CSTTConnector::GetConnectionTimeOut()
{
	return iConnectionTimeOut;
}

// Communication methods ------------------------------------------------------

TInt CSTTConnector::StartL(TConnectorType aConnectorType, RConnection* aConnection)
{
	TInetAddr addr;
	return StartL(aConnectorType, aConnection, addr);
}


TInt CSTTConnector::StartL(TConnectorType aConnectorType, RConnection* aConnection, TInetAddr& aAddr)
{
	aAddr = TInetAddr(0, 0);
	
	if (iState != EInitialized && iState != EInitializing) 
		return KErrGeneral;

	iConnection = aConnection;
	iConnectorType = aConnectorType;
	
	if (!iGettingImei)
	{			
#ifdef PROXYCONNECTOR_ENABLED
		if (iConnectorType==EConProxy)
		{
			if (iProxyAddress.Input((iBaseData->GetProxyAddress())) != KErrNone)
				return KErrGeneral;
			iProxyAddress.SetPort(iBaseData->GetConnectionPort());
			
			if (aConnection)
				User::LeaveIfError(iSocket.Open(*iSocketServ, KAfInet, KSockStream, KProtocolInetTcp, *aConnection));
			else
				User::LeaveIfError(iSocket.Open(*iSocketServ, KAfInet, KSockStream, KProtocolInetTcp));

			iSocket.Connect(iProxyAddress,iStatus);	// opening socket
			iSimpleState = EConConnecting;
			iState = EConnecting;
			
			SetActive();
			
			delete iTimeOutHandler;			
			iTimeOutHandler = NULL;
			
			aAddr = iProxyAddress;
			
			iTimeOutHandler = CTimeOutHandler::NewL();
			iTimeOutHandler->SetObserver(this);
			iTimeOutHandler->Start(TTimeIntervalMicroSeconds32(iConnectionTimeOut));
		}
		else if (iConnectorType == EConListener)
		{
#endif
			if (aConnection)
				User::LeaveIfError(iSocketListener.Open(*iSocketServ, KAfInet, KSockStream, KProtocolInetTcp, *aConnection));
			else
				User::LeaveIfError(iSocketListener.Open(*iSocketServ, KAfInet, KSockStream, KProtocolInetTcp));
			
			{
				TInetAddr addr;
				addr.SetPort(iBaseData->GetConnectionPort());
				//addr.SetPort(32323);
				TInt err = iSocketListener.Bind(addr);				
				if (err != KErrNone)
				 return err;
				
				err = iSocketListener.Listen(5);
				if (err != KErrNone)
				 return err;
			}
			
			if (aConnection)
			{
				if (GetLocalIPAddressL(aAddr, aConnection) == KErrNone)
				{
					aAddr.SetPort(iBaseData->GetConnectionPort());
				/*	LOG->WriteL(_L("[Connector] Starting to listen on: "));
					TBuf<128> ip;
					aAddr.Output(ip);
					LOG->WriteLineL(ip);*/
				}
			/*	else
					LOG->WriteLineL(_L("[Connector] Failed to get IP address"));*/
			}
			
			iBlankSocket = new RSocket();
			iBlankSocket->Open(*iSocketServ);
			iSocketListener.Accept(*iBlankSocket, iStatus); 
			SetActive(); 
			
	/*		LOG->WriteL(_L("[Connector] Listening on: "));
			TBuf<128> addrBuf;
			addr.Output(addrBuf);
			LOG->WriteLineL(addrBuf);*/

			iListenerState = EWaitingForConnection;
			iSimpleState = EConnectedAndInitialized;
			
#ifdef PROXYCONNECTOR_ENABLED
		}
#endif
	}
	else
		iConnectIssued = ETrue;
	
	return KErrNone;
}

void CSTTConnector::Stop()
{
	if (iSimpleState != EConOffline)
	{
#ifdef PROXYCONNECTOR_ENABLED
		if (iConnectorType==EConProxy)
		{
			Cancel();
			iSocket.Close();				// closing socket
						
    		iState = EInitialized;
			iSimpleState = EConOffline;
			
			if (iConnectorEventHandler != NULL) 
				if (iSimpleState == EConConnecting)
					iConnectorEventHandler->ReportEvent(EErrConnectionFailed);
				else
					iConnectorEventHandler->ReportEvent(EErrDisconnected);
		}
		else if (iConnectorType==EConListener)
		{
#endif
			Cancel();
			iSocketListener.Close();
			delete iBlankSocket;
			iBlankSocket = NULL;

			iListenerState = EOffline;
			iSimpleState = EConOffline;
#ifdef PROXYCONNECTOR_ENABLED
		}
#endif
	}	
}

#ifdef PROXYCONNECTOR_ENABLED
void CSTTConnector::ReceiveInitialization()
{
	if (iState!=ESendingInitialization) return;

	iState = EReceivingInitialization;
	iTemporaryBuffer.Copy(_L8(""));

	ContinueReading();
}

void CSTTConnector::ContinueReading()
{
	iSocket.RecvOneOrMore(iTransferingBuffer, 0, iStatus, iRecievedLength);
	SetActive();
}

void CSTTConnector::ReadIncoming()
{
	TBool cont = ETrue;

	iTemporaryBuffer.Append(iTransferingBuffer);
	TInt position;
	while ((position=iTemporaryBuffer.Locate('\n'))!=KErrNotFound)
	{
		if (iTemporaryBuffer.Left(6).Compare(_L8("#prtn#"))==0)
		{
			TInt tempport;
			if (TLex8(iTemporaryBuffer.Right(iTemporaryBuffer.Length()-6).Left(iTemporaryBuffer.Length()-7)).Val(tempport)==KErrNone)
			{
				iBaseData->SetPartnerPort(tempport);

				//-------------------------------
					TBuf<100> buf;
					buf.Copy(_L("Partnerport: "));
					buf.AppendNum(tempport);

					iSimpleState = EConnectedAndInitialized;
					
					if (iConnectorEventHandler!=NULL)
					{
						iConnectorEventHandler->ReportMessage(buf);
						iConnectorEventHandler->ReportEvent(ESucConnectedAndInitialized);
					}
						
				//-------------------------------

				WaitForFirstConnection();
				cont = EFalse;
			}
			else
			{
				iTemporaryBuffer.Copy(_L8(""));
				cont = ETrue;
			}
		}
		else
		{
			iTemporaryBuffer = iTemporaryBuffer.Right(iTemporaryBuffer.Length()-position-1);
			cont = ETrue;
		}
	}
		

	if (cont) ContinueReading();
}

void CSTTConnector::SendInitialization()
{
	if (iState!=EConnecting) return;

	iState = ESendingInitialization;
	iSimpleState = EConInitializing;


	TPtrC pImei = iIMEI->Des();
	iTransferingBuffer.Copy(_L8("#imei#"));
	MyAppend8_16(iTransferingBuffer, pImei);
	iTransferingBuffer.Append(_L8("\n"));
	iSocket.Write(iTransferingBuffer,iStatus);
	SetActive();
}

void CSTTConnector::WaitForFirstConnection()
{
	iTemporaryBuffer.Copy(_L8(""));
	WaitForNextConnection();
}

void CSTTConnector::WaitForNextConnection()
{
	iState = EWaitingForConnections;
	
	iSocket.RecvOneOrMore(iTransferingBuffer, 0, iStatus, iRecievedLength);
	SetActive();
}

void CSTTConnector::ReadIncomingMessage()
{
	iState = EProcessingIncomingMessage;

	ProcessIncoming();

	WaitForNextConnection();
}

void CSTTConnector::ProcessIncoming()
{
	iTemporaryBuffer.Append(iTransferingBuffer);

	TBufC8<120> buf;
	TInt position;
	while ((position=iTemporaryBuffer.Locate('\n'))!=KErrNotFound)
	{
		buf = iTemporaryBuffer.Left(position);

		if (buf.Left(6).Compare(_L8("#paid#"))==0)
		{
			buf = buf.Right(buf.Length()-6);

			TBuf<100> msg;
			msg.Copy(_L("Recieved connection: "));
			MyAppend16_8(msg,buf);
			if (iConnectorEventHandler!=NULL)
				iConnectorEventHandler->ReportMessage(msg);

			CheckAndAddTransferSocket(buf);
		}
		else
		{
			TBuf<100> msg;
			msg.Copy(_L("Unknown msg: "));
			MyAppend16_8(msg,buf);
			if (iConnectorEventHandler!=NULL)
				iConnectorEventHandler->ReportMessage(msg);
		}

		iTemporaryBuffer.Copy(iTemporaryBuffer.Right(iTemporaryBuffer.Length()-(position+1)));
	}
}

void CSTTConnector::CheckAndAddTransferSocket(TPtrC8 aConnectionId)
{
	CSTTTransferSocketInitializator* tempSocket;
	CSTTTransferSocketInitializator* lastSocket = NULL;

	tempSocket = iTransferSocket;
    while (tempSocket!=NULL) 
	{
		if (tempSocket->GetState()==ETSFinished)
		{
			if (lastSocket==NULL)
			{
				iTransferSocket = tempSocket->GetNextSibling();
				delete tempSocket;
				tempSocket = iTransferSocket;
				lastSocket = NULL;
			}
			else
			{
				lastSocket->SetNextSibling(tempSocket->GetNextSibling());
				delete tempSocket;
				tempSocket = lastSocket->GetNextSibling();
			}
		}
		else
		{
			lastSocket = tempSocket;
			tempSocket = tempSocket->GetNextSibling();
		}
	}

	tempSocket = CSTTTransferSocketInitializator::NewL(iBaseData->GetProxyAddress(), iBaseData->GetServicePort(), aConnectionId, iSocketCollector);
	tempSocket->SetNextSibling(iTransferSocket);
	iTransferSocket = tempSocket;
}

void CSTTConnector::ReleaseAllTransferSocket()
{
	CSTTTransferSocketInitializator* tempSocket;

    while (iTransferSocket!=NULL) 
	{
		tempSocket = iTransferSocket;
		iTransferSocket = iTransferSocket->GetNextSibling();
		delete tempSocket;
	}
}
#endif


// Other methods --------------------------------------------------------------

void CSTTConnector::MyAppend8_16(TDes8 &aTarget, TDesC16 &aSource)
{
	TInt i;
	for (i=0;i<aSource.Length();i++)
	{
		aTarget.Append(aSource[i]);
	}
}

void CSTTConnector::MyAppend16_8(TDes16 &aTarget,TDesC8 &aSource)
{
	TInt i;
	for (i=0;i<aSource.Length();i++)
	{
		aTarget.Append(aSource[i]);
	}
}

void CSTTConnector::ProcessConnection()
{
	iListenerState = EProcessingConnection;
	TInetAddr partnerAddress;
	iSocketCollector->Accept(iBlankSocket, partnerAddress);

	//delete iBlankSocket;
	iBlankSocket = NULL;
	iBlankSocket = new RSocket();
	iBlankSocket->Open(*iSocketServ);
	iSocketListener.Accept(*iBlankSocket, iStatus); 
	SetActive();
	iListenerState = EWaitingForConnection;
}

// from MTimeOutObserver ------------------------------------------------------
void CSTTConnector::TimeOutExpired()
{
	Stop();
}

// Active object methods ------------------------------------------------------

void CSTTConnector::DoCancel()
{
	if (iConnectorType==EConProxy)
		iSocket.CancelAll();
	else
		iSocketListener.CancelAll();
}

void CSTTConnector::RunL()
{
    switch (iStatus.Int())
    {
        case KErrNone:
			iLastErrorCode = 0;
			/* TODO: Handle succeed events */

#ifdef PROXYCONNECTOR_ENABLED
			if (iConnectorType==EConProxy)
			{
				switch (iState)
				{
					case EConnecting: // connection succeed
					
						if (iTimeOutHandler)
						{
							iTimeOutHandler->Stop();
							delete iTimeOutHandler;
							iTimeOutHandler = NULL;
						}						
						
						if (iConnectorEventHandler!=NULL)
							iConnectorEventHandler->ReportEvent(ESucConnectionSucceed);
						SendInitialization();
						break;
					case ESendingInitialization: // initialization sending succeed
						//if (iConnectorEventHandler!=NULL)
						//	iConnectorEventHandler->ReportEvent(ESucSendInitializationSucceed);
						ReceiveInitialization();
						break;
					case EReceivingInitialization: // initialization sending succeed
						if (iConnectorEventHandler!=NULL)
							iConnectorEventHandler->ReportMessage(_L("message arrived"));
						iSimpleState = EConConnected;
						ReadIncoming();
						break;
					case EWaitingForConnections:
						if (iConnectorEventHandler!=NULL)
							iConnectorEventHandler->ReportEvent(ERecievedMessage);
						ReadIncomingMessage();
						break;
					default:
						break;
				}
			}
			else if (iConnectorType==EConListener)
			{
#endif
				switch (iListenerState)
				{
					case EWaitingForConnection:
						ProcessConnection();
						break;
					default:
						break;
				}
#ifdef PROXYCONNECTOR_ENABLED
			}
#endif

			break;
		default:
			iLastErrorCode = iStatus.Int();
			/* TODO: Handle errors */

#ifdef PROXYCONNECTOR_ENABLED
			if (iConnectorType==EConProxy)
			{
				switch (iState)
				{
					case EConnecting:					
						Stop();
						break;
					default:
						TBuf<100> buf;
						buf.Copy(_L("Error: "));
						buf.AppendNum(iStatus.Int());

						if (iConnectorEventHandler!=NULL)
							iConnectorEventHandler->ReportMessage(buf);
						Stop();
						break;
				}
			}
			else if (iConnectorType==EConListener)
			{
#endif
				switch (iListenerState)
				{
					case EWaitingForConnection:
						Stop();
						break;
					default:
						Stop();
						break;
				}
#ifdef PROXYCONNECTOR_ENABLED
			}
#endif
			break;
	}
}

TInt CSTTConnector::GetLocalIPAddressL(TInetAddr& aAddr, RConnection* aConnection)
{
	RSocket sock;
	if (aConnection)
		User::LeaveIfError(sock.Open(*iSocketServ, KAfInet, KSockStream, KProtocolInetTcp, *aConnection));
	else
		User::LeaveIfError(sock.Open(*iSocketServ, KAfInet, KSockStream, KProtocolInetTcp));
	 
	// Get the IAP id of the underlying interface of this RConnection
	TUint32 iapId = 0;
	User::LeaveIfError((*aConnection).GetIntSetting(_L("IAP\\Id"), iapId));
	
	// Get IP information from the socket
	TSoInetInterfaceInfo ifinfo;
	TPckg<TSoInetInterfaceInfo> ifinfopkg(ifinfo);
	 
	TSoInetIfQuery ifquery;
	TPckg<TSoInetIfQuery> ifquerypkg(ifquery);
	 
	// To find out which interfaces are using our current IAP, we must
	// enumerate and go through all of them and make a query by name
	// for each.
	User::LeaveIfError(sock.SetOpt(KSoInetEnumInterfaces, KSolInetIfCtrl));
	while(sock.GetOpt(KSoInetNextInterface, KSolInetIfCtrl, ifinfopkg) == KErrNone)
	{
		ifquery.iName = ifinfo.iName;
		TInt err = sock.GetOpt(KSoInetIfQueryByName, KSolInetIfQuery, ifquerypkg);
	 
		if (err == KErrNone && ifquery.iZone[1] == iapId) // IAP ID is index 1 of iZone
		{
	 		// We have found an interface using the IAP we are interested in.
	    	TInetAddr ad = (TInetAddr&)ifinfo.iAddress;
			// must filter 0 and addresses 169.254.x.x (fake responses)
			if ( (ad.Address() != 0)&& ((ad.Address() & 0xFFFF0000) != 0xA9FE0000) )
			{
	    		// found a IPv4 address
	    		aAddr = ad;
	 
	    		sock.Close();
	    		return err; // stop & return KErrNone
	   		}
	  	}
	  	else if(err != KErrNone)
	  	{
	   		sock.Close();
	   		return err; // return with error
	  	}
	} // while
	 
	sock.Close();
	return KErrNotFound; // return with KErrNotFound
}

