/*
* ============================================================================
*  Name     : CSTTTorrentListItem from STTTorrentListItem.h
*  Part of  : SymTorrentTracker
*  Created  : 08.01.2006 by Zsolt Pszota
*  Version  :
*  Copyright: 
* ============================================================================
*/

// INCLUDE FILES
#include "STTTorrentListItem.h"

// Constructors and destructor ------------------------------------------------
CSTTTorrentListItem::CSTTTorrentListItem()
{
}

void CSTTTorrentListItem::ConstructL(TPtrC aTorrentFileName, TPtrC8 aInfoHash)
{
	iTorrentFileName = aTorrentFileName.AllocL();
	iInfoHash		 = aInfoHash.AllocL();
	iNextSibling = NULL;

	TBuf8<10> temp;
	iPeerListHead = CSTTTorrentPeerItem::NewL(TInetAddr(),temp);
}

CSTTTorrentListItem::~CSTTTorrentListItem()
{
	delete iTorrentFileName;
	delete iInfoHash;

	DeleteAllPeerItem();
	delete iPeerListHead;

}

CSTTTorrentListItem* CSTTTorrentListItem::NewL(TPtrC aTorrentFileName, TPtrC8 aInfoHash)
{
	CSTTTorrentListItem * self=new(ELeave) CSTTTorrentListItem();
	CleanupStack::PushL(self);
	self->ConstructL(aTorrentFileName, aInfoHash);
	CleanupStack::Pop();
	return self;
}

// Peer list handling
CSTTTorrentPeerItem* CSTTTorrentListItem::InsertPeerItemL(TInetAddr aAddress, TDesC8& aId)
{
	return iPeerListHead->InsertAfterL(aAddress, aId);
}

TBool CSTTTorrentListItem::DeletePeerItem(TInetAddr aAddress, TDesC8& aId)
{
	TBool res = EFalse;
	CSTTTorrentPeerItem* lastitem = iPeerListHead;
	CSTTTorrentPeerItem* tempitem = iPeerListHead->GetNextSibling();
	while (tempitem!=NULL && !res)
	{
		if (tempitem->PeerMatch(aAddress, aId))
		{
			lastitem->DeleteNext();
			res = ETrue;
		}
		else
		{
			lastitem = tempitem;
			tempitem = tempitem->GetNextSibling();
		}
	}
	return res;
}

void CSTTTorrentListItem::DeleteSimilarPeers(TInetAddr aAddress, TDesC8& aId)
{
	CSTTTorrentPeerItem* lastitem = iPeerListHead;
	CSTTTorrentPeerItem* tempitem = iPeerListHead->GetNextSibling();
	while (tempitem!=NULL)
	{
		if ((tempitem->PeerId())->Compare(aId) || tempitem->Address()==aAddress.Address())
		{
			lastitem->DeleteNext();
			tempitem = lastitem->GetNextSibling();
		}
		else
		{
			lastitem = tempitem;
			tempitem = tempitem->GetNextSibling();
		}
	}
}

void CSTTTorrentListItem::DeleteAllPeerItem()
{
	while (iPeerListHead->GetNextSibling()!=NULL)
		iPeerListHead->DeleteNext();
}

CSTTTorrentPeerItem* CSTTTorrentListItem::GetPeerListFirstItem()
{
	return iPeerListHead->GetNextSibling();
}

CSTTTorrentPeerItem* CSTTTorrentListItem::FindPeerItem(TInetAddr aAddress, TDesC8& aId)
{	
	TBool found = EFalse;
	CSTTTorrentPeerItem* tempitem = iPeerListHead->GetNextSibling();
	while (tempitem!=NULL && !found)
	{
		if (tempitem->PeerMatch(aAddress, aId))
			found = ETrue;
		tempitem = tempitem->GetNextSibling();
	}
	if (found)
		return tempitem;
	else
		return NULL;
}

void CSTTTorrentListItem::RemoveNeedlessPeers()
{
	CSTTTorrentPeerItem* lastitem = iPeerListHead;
	CSTTTorrentPeerItem* tempitem = iPeerListHead->GetNextSibling();
	while (tempitem!=NULL)
	{
		if (tempitem->SecondsFromLastCheckInTime()>10*60)
		{
			lastitem->DeleteNext();
			tempitem = lastitem->GetNextSibling();
		}
		else
		{
			lastitem = tempitem;
			tempitem = tempitem->GetNextSibling();
		}
	}

}

// Getter and setter functions ------------------------------------------------

TPtr CSTTTorrentListItem::GetTorrentFileName()
{
	return iTorrentFileName->Des();
}

void CSTTTorrentListItem::SetTorrentFileNameL(TPtrC aTorrentFileName)
{
	delete iTorrentFileName;
	iTorrentFileName = aTorrentFileName.AllocL();
}

TPtr8 CSTTTorrentListItem::GetInfoHash()
{
	return iInfoHash->Des();
}

void CSTTTorrentListItem::SetInfoHashL(TPtrC8 aInfoHash)
{
	delete iInfoHash;
	iInfoHash = aInfoHash.AllocL();
}

CSTTTorrentListItem* CSTTTorrentListItem::GetNextSibling()
{
	return iNextSibling;
}

void CSTTTorrentListItem::SetNextSibling(CSTTTorrentListItem* aNextSibling)
{
	iNextSibling = aNextSibling;
}

// New functions --------------------------------------------------------------

CSTTTorrentListItem* CSTTTorrentListItem::InsertAfterL(TPtrC aTorrentFileName, TPtrC8 aInfoHash)
{
	CSTTTorrentListItem* tempitem = CSTTTorrentListItem::NewL(aTorrentFileName,aInfoHash);
	tempitem->SetNextSibling(iNextSibling);
	iNextSibling = tempitem;
	return tempitem;
}

void CSTTTorrentListItem::DeleteNext()
{
	if (iNextSibling!=NULL)
	{
		CSTTTorrentListItem* tempitem = iNextSibling;
		iNextSibling = tempitem->GetNextSibling();
		delete tempitem;
	}
}
