/*
* ============================================================================
*  Name     : CSTTTorrentListItem from STTTorrentListItem.h
*  Part of  : SymTorrentTracker
*  Created  : 08.01.2006 by Zsolt Pszota
*  Version  :
*  Copyright: 
* ============================================================================
*/

// INCLUDE FILES
#include "STTTorrentPeerItem.h"

// Constructors and destructor ------------------------------------------------
CSTTTorrentPeerItem::CSTTTorrentPeerItem()
{
}

void CSTTTorrentPeerItem::ConstructL(TInetAddr aAddress, TDesC8& aId)
{
	iNextSibling	= NULL;
	iAddress		= aAddress;
	iId				= aId.AllocL();
	iCompleted		= EFalse;
	iLastCheckInTime.HomeTime();
	iIp				= KNullDesC8().AllocL();
}

CSTTTorrentPeerItem::~CSTTTorrentPeerItem()
{
	delete iId;
	if (iIp)
	{
		delete iIp;
		iIp = NULL;
	}
}

CSTTTorrentPeerItem* CSTTTorrentPeerItem::NewL(TInetAddr aAddress, TDesC8& aId)
{
	CSTTTorrentPeerItem * self=new(ELeave) CSTTTorrentPeerItem();
	CleanupStack::PushL(self);
	self->ConstructL(aAddress, aId);
	CleanupStack::Pop();
	return self;
}

// Getter and setter functions ------------------------------------------------

CSTTTorrentPeerItem* CSTTTorrentPeerItem::GetNextSibling()
{
	return iNextSibling;
}

void CSTTTorrentPeerItem::SetNextSibling(CSTTTorrentPeerItem* aNextSibling)
{
	iNextSibling = aNextSibling;
}

void CSTTTorrentPeerItem::SetLastCheckInTime(TTime& aLastCheckInTime)
{
	iLastCheckInTime = aLastCheckInTime;
}

TTime CSTTTorrentPeerItem::LastCheckInTime()
{
	return iLastCheckInTime;
}

void CSTTTorrentPeerItem::SetCompleted(TBool aCompleted)
{
	iCompleted = aCompleted;
}

TBool CSTTTorrentPeerItem::Completed()
{
	return iCompleted;
}

void CSTTTorrentPeerItem::SetLastCheckInTimeNow()
{
	iLastCheckInTime.HomeTime();
}

TInt CSTTTorrentPeerItem::SecondsFromLastCheckInTime()
{
	TTimeIntervalSeconds timeinterval;
	TTime now;
	now.HomeTime();
	if (now.SecondsFrom(iLastCheckInTime, timeinterval) == KErrNone)
	{
		return timeinterval.Int();
	}
	else
		return -1;
}

TBool CSTTTorrentPeerItem::PeerMatch(TInetAddr aAddress, TDesC8& aId)
{
	return (aAddress.Port()==iAddress.Port() && aAddress.Address()==iAddress.Address() && iId->Compare(aId)==0);
}

TUint32 CSTTTorrentPeerItem::Address()
{
	return iAddress.Address();
}

TInt CSTTTorrentPeerItem::Port()
{
	return iAddress.Port();
}

HBufC8* CSTTTorrentPeerItem::PeerId()
{
	return iId;
}

HBufC8* CSTTTorrentPeerItem::PeerIp()
{
	if (iIp)
	{
		delete iIp;
		iIp = NULL;
	}
	TBuf<60> tempip;
	TBuf8<60> tempip8;
	iAddress.Output(tempip);
	tempip8.Copy(tempip);
	iIp = tempip8.AllocL();
	return iIp;
}


// New functions --------------------------------------------------------------

CSTTTorrentPeerItem* CSTTTorrentPeerItem::InsertAfterL(TInetAddr aAddress, TDesC8& aId)
{
	CSTTTorrentPeerItem* tempitem = CSTTTorrentPeerItem::NewL(aAddress, aId);
	tempitem->SetNextSibling(iNextSibling);
	this->SetNextSibling(tempitem);
	return tempitem;
}

void CSTTTorrentPeerItem::DeleteNext()
{
	if (iNextSibling!=NULL)
	{
		CSTTTorrentPeerItem* tempitem = iNextSibling;
		iNextSibling = tempitem->GetNextSibling();
		delete tempitem;
	}
}
