/*****************************************************************************
 * Copyright (C) 2006 Imre Kelnyi
 *-------------------------------------------------------------------
 * This file is part of SymTorrent
 *
 * SymTorrent is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTorrent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Symella; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/
 
/* ====================================================================
 * File: SymTorrentLauncherDocument.h
 * Created: 06/11/06
 * ==================================================================== */

#ifndef __SYMTORRENTLAUNCHER_DOCUMENT_H__
#define __SYMTORRENTLAUNCHER_DOCUMENT_H__

#include <akndoc.h>
#include <apmrec.h>
#include <apmstd.h>

// Forward references
class CSymTorrentLauncherAppUi;
class CEikApplication;


/*! 
  @class CSymTorrentLauncherDocument
  
  @discussion An instance of class CSymTorrentLauncherDocument is the Document part of the AVKON
  application framework for the SymTorrentLauncher example application
  */
class CSymTorrentLauncherDocument : public CAknDocument
    {
public:

/*!
  @function NewL
  
  @discussion Construct a CSymTorrentLauncherDocument for the AVKON application aApp 
  using two phase construction, and return a pointer to the created object
  @param aApp application creating this document
  @result a pointer to the created instance of CSymTorrentLauncherDocument
  */
    static CSymTorrentLauncherDocument* NewL(CEikApplication& aApp);

/*!
  @function NewLC
  
  @discussion Construct a CSymTorrentLauncherDocument for the AVKON application aApp 
  using two phase construction, and return a pointer to the created object
  @param aApp application creating this document
  @result a pointer to the created instance of CSymTorrentLauncherDocument
  */
    static CSymTorrentLauncherDocument* NewLC(CEikApplication& aApp);

/*!
  @function ~CSymTorrentLauncherDocument
  
  @discussion Destroy the object and release all memory objects
  */
    ~CSymTorrentLauncherDocument();

public: // from CAknDocument
/*!
  @function CreateAppUiL 
  
  @discussion Create a CSymTorrentLauncherAppUi object and return a pointer to it
  @result a pointer to the created instance of the AppUi created
  */
    CEikAppUi* CreateAppUiL();
    
protected:  // Functions from base classes

#ifdef EKA2
	void OpenFileL(CFileStore*& aFileStore, RFile& aFile);
#else
	CFileStore* OpenFileL(TBool aDoOpen, const TDesC &aFilename, RFs &aFs);
#endif

	void OpenFileL(const TDesC &aFilename);

private:

/*!
  @function ConstructL
  
  @discussion Perform the second phase construction of a CSymTorrentLauncherDocument object
  */
    void ConstructL();
    
  //  static void StartThread();
	
//	static TInt StartAppThreadFunction(TAny* aParam);
	
	static void StartAppThreadFunctionL(const TDesC& aTorrentName);

/*!
  @function CSymTorrentLauncherDocument
  
  @discussion Perform the first phase of two phase construction 
  @param aApp application creating this document
  */
    CSymTorrentLauncherDocument(CEikApplication& aApp);

    };


#endif // __SYMTORRENTLAUNCHER_DOCUMENT_H__
