/*****************************************************************************
 * Copyright (C) 2006 Imre Kelnyi
 *-------------------------------------------------------------------
 * This file is part of SymTorrent
 *
 * SymTorrent is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTorrent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Symella; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/
 
/* ====================================================================
 * File: SymTorrentLauncherDocument.cpp
 * Created: 06/11/06
 * ==================================================================== */

#include "SymTorrentLauncherAppUi.h"
#include "SymTorrentLauncherDocument.h"
#include "STLogger.h"
#include <DocumentHandler.h>
#include <apmstd.h>
#include <apgtask.h>
#include <eikenv.h>
#include <eikappui.h>
#include <badesca.h>
#include <f32file.h>
#include <utf.h>
#include <apacmdln.h>
#include <apgcli.h>


// Standard Symbian OS construction sequence
CSymTorrentLauncherDocument* CSymTorrentLauncherDocument::NewL(CEikApplication& aApp)
    {
    CSymTorrentLauncherDocument* self = NewLC(aApp);
    CleanupStack::Pop(self);
    return self;
    }

CSymTorrentLauncherDocument* CSymTorrentLauncherDocument::NewLC(CEikApplication& aApp)
    {
    CSymTorrentLauncherDocument* self = new (ELeave) CSymTorrentLauncherDocument(aApp);
    CleanupStack::PushL(self);
    self->ConstructL();
    return self;
    }

void CSymTorrentLauncherDocument::ConstructL()
    {
    LOG->WriteLineL(_L("Document ContructL"));
	// no implementation required
    }    

CSymTorrentLauncherDocument::CSymTorrentLauncherDocument(CEikApplication& aApp) : CAknDocument(aApp) 
    {
	// no implementation required
    }

CSymTorrentLauncherDocument::~CSymTorrentLauncherDocument()
    {
	// no implementation required
    }

CEikAppUi* CSymTorrentLauncherDocument::CreateAppUiL()
    {
    // Create the application user interface, and return a pointer to it,
    // the framework takes ownership of this object
    CEikAppUi* appUi = new (ELeave) CSymTorrentLauncherAppUi;
    return appUi;
    }

#ifdef EKA2
void CSymTorrentLauncherDocument::OpenFileL(CFileStore*& /*aFileStore*/, RFile& aFile)
	{
	TFileName filename;
	aFile.FullName(filename);
	OpenFileL(filename);
	}
#else
CFileStore* CSymTorrentLauncherDocument::OpenFileL(TBool /*aDoOpen*/, const TDesC& aFilename, RFs& /*aFs*/)
	{
	OpenFileL(aFilename);
	return NULL;
	}
#endif

void CSymTorrentLauncherDocument::OpenFileL(const TDesC &aFilename)
{
	LOG->WriteL(_L("OpenFileL: "));
	LOG->WriteLineL(aFilename);
	
	// check if SymTorrent is already running
	TApaTaskList taskList(CEikonEnv::Static()->WsSession());
	
	TApaTask task = taskList.FindApp(_L("SymTorrent"));
	
	if (!task.Exists())
	{
		LOG->WriteLineL(_L("SymTorrent NOT running"));
		
		#ifdef EKA2
		{
			CDocumentHandler* handler = CDocumentHandler::NewL(NULL);
			CleanupStack::PushL(handler);
			
			TFileName descr;
			descr.Append(aFilename);
	
			_LIT8(KLitSymTorrentType, "application/x-symtorrent");
			TDataType dataType(KLitSymTorrentType);
			
			// Launch the appropriate application for this file
			handler->OpenFileL(descr, dataType);
			
			CleanupStack::PopAndDestroy(); // handler		*/
		}
		#else
		{
			StartAppThreadFunctionL(aFilename);		
		}
		#endif
	}
	else
//	#ifdef EKA2
//	else
//	{
//	#endif	
//		TApaTask task2 = taskList.FindApp(_L("SymTorrent"));
//		if (!task2.Exists())	
		{
			LOG->WriteLineL(_L("SymTorrent running"));
			
			// send the filename utf8 encoded
			HBufC8* encodedFilename = HBufC8::NewLC(aFilename.Length() * 3);
			
			TPtr8 encodedFilenamePtr = encodedFilename->Des();
			if (CnvUtfConverter::ConvertFromUnicodeToUtf8(encodedFilenamePtr, aFilename) == 0)
			{
				task.BringToForeground();
				task.SendMessage(TUid::Uid(KUidApaMessageSwitchOpenFileValue), *encodedFilename);
			}
			
			CleanupStack::PopAndDestroy(); // encodedFilename
		}
//	#ifdef EKA2
//	}
//	#endif
	
	CEikAppUi* appUi = CEikonEnv::Static()->EikAppUi();
	if (appUi)
		appUi->HandleCommandL(EEikCmdExit);
}

/*void CSymTorrentLauncherDocument::StartThread()
{
	TInt res = KErrNone;

	//create a new thread for starting our application
	RThread * startAppThread;
	startAppThread = new RThread();

	User::LeaveIfError( res = startAppThread->Create(
		_L("MobileAgentThread"), 
		CSymTorrentLauncherDocument::StartAppThreadFunction,
		KDefaultStackSize,
		KMinHeapSize,
		KMinHeapSize,
		NULL,
		EOwnerThread) );

	startAppThread->SetPriority(EPriorityNormal);

	startAppThread->Resume();

	startAppThread->Close();
}*/


//TInt CSymTorrentLauncherDocument::StartAppThreadFunction(TAny* /*aParam*/)
//{
	/*
	//wait 5 seconds...
	RTimer timer; // The asynchronous timer and ...
	TRequestStatus timerStatus; // ... its associated request status
	timer.CreateLocal(); // Always created for this thread.
	// get current time (microseconds since 0AD nominal Gregorian)
	TTime time;
	time.HomeTime();
	// add ten seconds to the time
	TTimeIntervalSeconds timeIntervalSeconds(5);
	time += timeIntervalSeconds;
	// issue and wait
	timer.At(timerStatus,time);
	User::WaitForRequest(timerStatus);
	*/

/*
	// create an active scheduler
	CActiveScheduler * scheduler = new CActiveScheduler();
	if( scheduler == NULL )
		return KErrNoMemory;

	CActiveScheduler::Install(scheduler);
	// create a TRAP cleanup
	CTrapCleanup * cleanup = CTrapCleanup::New();
	TInt err;
	if( cleanup == NULL )
	{
		err = KErrNoMemory;
	}
	else
	{
		TRAP( err, StartAppThreadFunctionL() );
	}
	delete cleanup;
	delete CActiveScheduler::Current();

	return err;
}*/

// thread function to start our application
void CSymTorrentLauncherDocument::StartAppThreadFunctionL(const TDesC& aTorrentName)
{
	// absolute file path to our application
	TFileName fnAppPath = _L("\\system\\apps\\SymTorrent\\SymTorrent.app");

	RFs fsSession; //file server session
	User::LeaveIfError(fsSession.Connect());
	CleanupClosePushL(fsSession);
	TFindFile findFile( fsSession );

	User::LeaveIfError( findFile.FindByDir(fnAppPath, KNullDesC) );

	CApaCommandLine* cmdLine = CApaCommandLine::NewLC();
	
	#ifdef EKA2
		cmdLine->SetExecutableNameL( findFile.File() );
	#else
		cmdLine->SetLibraryNameL( findFile.File() );
	#endif
	
	cmdLine->SetCommandL( EApaCommandOpen );
	
	if (aTorrentName.Length() > 0)
		cmdLine->SetDocumentNameL(aTorrentName);

	RApaLsSession ls;
	User::LeaveIfError(ls.Connect());
	CleanupClosePushL(ls);

	User::LeaveIfError( ls.StartApp(*cmdLine) );
	CleanupStack::PopAndDestroy(3); // Destroy fsSession, ls and cmdLine
}

