/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

#ifndef TORRENTMAKER_H
#define TORRENTMAKER_H

// INCLUDES
#include <e32base.h>
#include <f32file.h>
#include "STBencode.h"

// CONSTANTS

// CLASS DECLARATION

// ENUMERATIONS

enum TErrorCode
{
	ESucceed=0,
	EFileAlreadyExists,
	EUnknownError
};

//ENUMERATIONS
enum ENodeType
{
	EFileNode=1,
	EFolderNode,
	ERootNode
};

/**
*  CSTMPackageNode view class.
* 
*/
class CSTMPackageNode : public CBase
    {
    public: // Constructors and destructor
		static CSTMPackageNode* NewL(TPtrC aNodeName, TPtrC aOriginalPath,
			ENodeType aNodeType, CSTMPackageNode* aParentNode);

        /**
        * EPOC default constructor.
        */
        void ConstructL(TPtrC aNodeName, TPtrC aOriginalPath,
			ENodeType aNodeType, CSTMPackageNode* aParentNode);

        /**
        * Destructor.
        */
        ~CSTMPackageNode();

    public: // Functions from base classes

    public: // New functions
		IMPORT_C void SetNextSibling(CSTMPackageNode* aNewNextSibling);
		IMPORT_C void SetFirstChild(CSTMPackageNode* aNewFirstChild);
		IMPORT_C CSTMPackageNode* GetNextSibling();
		IMPORT_C CSTMPackageNode* GetFirstChild();
		IMPORT_C CSTMPackageNode* GetParent();
		IMPORT_C const TPtr GetNodeName();
		IMPORT_C const TPtr GetNodeFullName();
		IMPORT_C ENodeType GetNodeType();
		IMPORT_C void GetFullPathInto(TDes &aFullPath);
		IMPORT_C void DeleteChildren();
		IMPORT_C TInt GetFileSize();

		IMPORT_C CSTMPackageNode* AddChild(TPtrC aNodeName, TPtrC aOriginalPath, ENodeType aNodeType);
		CSTMPackageNode* AddAndReturnChild(TPtrC aNodeName, TPtrC aOriginalPath, ENodeType aNodeType);

    private:

    private: // Data
		HBufC							*iNodeName;
		HBufC							*iOriginalPath;
		ENodeType						iNodeType;
		CSTMPackageNode 					*iParent;
		CSTMPackageNode 					*iNextSibling;
		CSTMPackageNode 					*iFirstChild;

    };

/**
*  CSTMPackage view class.
* 
*/
class CSTMPackage : public CBase
    {
    public: // Constructors and destructor

		static CSTMPackage* NewL();

        /**
        * EPOC default constructor.
        */
        void ConstructL();

        /**
        * Destructor.
        */
        IMPORT_C ~CSTMPackage();

    public: // Functions from base classes

    public: // New functions
		IMPORT_C void SetTorrentFileName(const TDesC &aNewFileName);
		IMPORT_C void GetTorrentFileNameInto(TDes &aFileName);

		IMPORT_C void SetTorrentSavePath(const TDesC &aNewSavePath);
		void GetTorrentSavePathInto(TDes &aSavePath);

		IMPORT_C void SetAnnounceUrl(const TDesC &aNewAnnounceUrl);
		IMPORT_C void GetAnnounceUrlInto(TDes &aAnnounceUrl);

		IMPORT_C void SetPieceSize(TInt aNewPieceSize);
		IMPORT_C TInt GetPieceSize();

		IMPORT_C CSTMPackageNode* GetRootNode();	

    private:

    private: // Data
		TBuf<100>						iTorrentFileName;
		TBuf<100>						iTorrentSavePath;
		TBuf<100>						iAnnounceUrl;
		TInt							iPieceSize;
		CSTMPackageNode*					iRootNode;
		
    };

/**
*  CSTMTorrentHandler class.
* 
*/
class CSTMTorrentHandler : public CBase
    {
    public: // Constructors and destructor

		IMPORT_C static CSTMTorrentHandler* NewL();

        /**
        * EPOC default constructor.
        */
        void ConstructL();

        /**
        * Destructor.
        */
        IMPORT_C ~CSTMTorrentHandler();

    public: // Functions from base classes

    public: // New functions
		IMPORT_C void SaveTorrent(TBool aSaveSTorrent);
		IMPORT_C CSTMPackage* GetTorrentPackage();
		IMPORT_C CSTMPackageNode* GetCurrentNode();
		IMPORT_C void SetCurrentNode(CSTMPackageNode* aNewCurrentNode);
		IMPORT_C TInt GetLastPieceNum();
		CSTMPackageNode* GetNextFile(CSTMPackageNode* aCurrentNode);
		// Clears the virtual file system
		IMPORT_C void Cancel();

    private:
		CSTBencode* CreateBencodedTorrent(CSTMPackage* aPackage);
		void AddAnnounce(CSTBencodedDictionary* aTorrentroot, CSTMPackage* aPackage);
		void AddEncoding(CSTBencodedDictionary* aTorrentroot);
		void AddCreatedBy(CSTBencodedDictionary* aTorrentroot);
		void AddInfo(CSTBencodedDictionary* aTorrentroot, CSTMPackage* aPackage);
		void AddName(CSTBencodedDictionary* aLocalRoot, CSTMPackage* aPackage);
		void AddPieceLength(CSTBencodedDictionary* aLocalRoot, CSTMPackage* aPackage);
		void AddPieces(CSTBencodedDictionary* aLocalRoot, CSTMPackage* aPackage, TInt aFullsize);

		TInt AddFiles(CSTBencodedDictionary* aLocalRoot, CSTMPackage* aPackage);
		TInt AddFile(CSTBencodedList* aLocalRoot, CSTMPackageNode* aNode);
		void AddFilePath(CSTBencodedList* aPathList, CSTMPackageNode* aNode);
		TInt SearchFiles(CSTBencodedList* aLocalRoot, CSTMPackageNode* aParentNode);

		void CopyFile(TDesC &aSourceFile, TDesC &aTargetFile, RFs &aFileServer);
		void CopyContent(CSTMPackageNode* aLocalRoot, TDesC &aPath, RFs &aFileServer);

    protected: // Data
		CSTMPackage							*iTorrentPackage;
		CSTMPackageNode 					*iCurrentNode;
		TInt								iLastPieceNum;
		
    };

/**
*  CTorrentMaker view class.
* 
*/
class CTorrentMaker : public CSTMTorrentHandler
{
    public: // Constructors and destructor

		static CTorrentMaker* NewL();

        /**
        * EPOC default constructor.
        */
        void ConstructL();

    public: // New functions
		void SetAnnounceUrl(const TDesC &aNewAnnounceUrl);
		void GetAnnounceUrlInto(TDes &aAnnounceUrl);

		void SetPieceSizeInKilobytes(TInt aNewPieceSize); 
		TInt GetPieceSizeInKilobytes();

		TErrorCode AddFileL(const TDesC &aFilename);
		TErrorCode AddFileL(const TDesC &aFilename, const TDesC &aVirtualFilename);

		TErrorCode SaveTorrentL(const TDesC &aTorrentFilename, TBool aSaveSTorrent);
		
};

#endif

// End of File
