/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

// INCLUDE FILES
#include "TorrentMaker.h"
#include <aknnotewrappers.h> 

// ================= MEMBER FUNCTIONS =======================
//
// ----------------------------------------------------------
// CSymTorrentMakerAppUi::ConstructL()
// 
// ----------------------------------------------------------
//
void CSTMPackageNode::ConstructL(TPtrC aNodeName, TPtrC aOriginalPath,
			ENodeType aNodeType, CSTMPackageNode* aParentNode)
{
	iNodeName		= HBufC::NewL(60);
	iOriginalPath	= HBufC::NewL(160);

 	(iNodeName->Des()).Copy(aNodeName);
	(iOriginalPath->Des()).Copy(aOriginalPath);

	iNodeType		= aNodeType;
	iParent			= aParentNode;
	iNextSibling	= NULL;
	iFirstChild		= NULL;
}

CSTMPackageNode::~CSTMPackageNode()
{
	if (iNextSibling!=NULL)
	{
		delete iNextSibling;
		iNextSibling = NULL;
	}

	if (iFirstChild!=NULL)
	{
		delete iFirstChild;
		iFirstChild = NULL;
	}
	delete iNodeName;
	delete iOriginalPath;
}

CSTMPackageNode* CSTMPackageNode::NewL(TPtrC aNodeName, TPtrC aOriginalPath,
			ENodeType aNodeType, CSTMPackageNode* aParentNode)
{
	CSTMPackageNode * self=new(ELeave) CSTMPackageNode();
	CleanupStack::PushL(self);
	self->ConstructL(aNodeName, aOriginalPath, aNodeType, aParentNode);
	CleanupStack::Pop();
	return self;
}


EXPORT_C void CSTMPackageNode::SetNextSibling(CSTMPackageNode* aNewNextSibling)
{
	iNextSibling = aNewNextSibling;
}

EXPORT_C void CSTMPackageNode::SetFirstChild(CSTMPackageNode* aNewFirstChild)
{
	iFirstChild = aNewFirstChild;
}


EXPORT_C CSTMPackageNode* CSTMPackageNode::GetNextSibling()
{
	return iNextSibling;
}

EXPORT_C CSTMPackageNode* CSTMPackageNode::GetFirstChild()
{
	return iFirstChild;
}

EXPORT_C CSTMPackageNode* CSTMPackageNode::GetParent()
{
	return iParent;
}


EXPORT_C const TPtr CSTMPackageNode::GetNodeName()
{
	return iNodeName->Des();
}

EXPORT_C const TPtr CSTMPackageNode::GetNodeFullName()
{
	return iOriginalPath->Des();
}


EXPORT_C ENodeType CSTMPackageNode::GetNodeType()
{
	return iNodeType;
}

EXPORT_C void CSTMPackageNode::GetFullPathInto(TDes &aFullPath)
{
	HBufC* restOfPath = HBufC::NewL(200);
	TPtr ptrRestOfPath = restOfPath->Des();

	if (GetParent()!=NULL)
	{
		GetParent()->GetFullPathInto(ptrRestOfPath);
	}

	aFullPath.Copy(ptrRestOfPath);
	if (GetNodeType()==EFolderNode)
	{
		aFullPath.Append(_L("\\"));
		aFullPath.Append(GetNodeName());
	}

	delete restOfPath;
}


CSTMPackageNode* CSTMPackageNode::AddAndReturnChild(TPtrC aNodeName, TPtrC aOriginalPath, ENodeType aNodeType)
{
	CSTMPackageNode* tempresult = AddChild(aNodeName, aOriginalPath, aNodeType);
	CSTMPackageNode* tempChild;

	if (tempresult==NULL)
	{

		TBuf<100> myNodeName;
		myNodeName.Copy(aNodeName);
		for (int i=0; i<myNodeName.Length(); i++)
		if (myNodeName[i]==':') myNodeName[i]='_';

		tempChild = iFirstChild;
		while (tempresult==NULL && tempChild!=NULL)
		{
			if ((tempChild->GetNodeName().Compare(myNodeName))==0) 
			{ 
				tempresult = tempChild;
			}

			tempChild = tempChild->GetNextSibling();
		}
	}
	return tempresult;
}

EXPORT_C CSTMPackageNode* CSTMPackageNode::AddChild(TPtrC aNodeName, TPtrC aOriginalPath, ENodeType aNodeType)
{
	TBool freeName = ETrue;
	CSTMPackageNode* tempChild;
	CSTMPackageNode* retChild = NULL;
	tempChild = iFirstChild;
	
	TBuf<100> myNodeName;
	myNodeName.Copy(aNodeName);
	for (int i=0; i<myNodeName.Length(); i++)
		if (myNodeName[i]==':') myNodeName[i]='_';

	while (freeName && tempChild!=NULL)
	{
		if ((tempChild->GetNodeName().Compare(myNodeName))==0) 
		{ 
			freeName = EFalse; 
			retChild = tempChild;
		}

		tempChild = tempChild->GetNextSibling();
	}

	if  (freeName)
	{
		TInt comp;
		
		CSTMPackageNode* currentChild = CSTMPackageNode::NewL(myNodeName, aOriginalPath, aNodeType, this);
		CSTMPackageNode* selectedChild;

		if (iFirstChild!=NULL)
			comp = myNodeName.CompareC(TPtrC(iFirstChild->GetNodeName()));

		if (iFirstChild==NULL)
		{	// insert into the first place of an empty list
			iFirstChild	= currentChild;
		}
		else if ((iFirstChild->GetNodeType()==EFileNode && aNodeType==EFolderNode) || 
				 (iFirstChild->GetNodeType()==aNodeType && myNodeName.CompareC(iFirstChild->GetNodeName())<0))
		{	// insert into the first place of a non-empty list
			currentChild->SetNextSibling(iFirstChild);
			iFirstChild	= currentChild;
		}
		else
		{	// find the place and insert the new child after the selected
			selectedChild = iFirstChild;
			while ((selectedChild->GetNextSibling()!=NULL) && 
					 ((selectedChild->GetNextSibling()->GetNodeType()==EFolderNode && aNodeType==EFileNode) || 
					 (selectedChild->GetNextSibling()->GetNodeType()==aNodeType && myNodeName.CompareC(selectedChild->GetNextSibling()->GetNodeName())>0)))
			{
				selectedChild = selectedChild->GetNextSibling();
			}

			comp = myNodeName.CompareC(TPtrC(selectedChild->GetNodeName()));

			currentChild->SetNextSibling(selectedChild->GetNextSibling());
			selectedChild->SetNextSibling(currentChild);
		}

		return currentChild;
	}
	else
		return NULL;
}

EXPORT_C void CSTMPackageNode::DeleteChildren()
{
	if (iFirstChild!=NULL)
	{
		delete iFirstChild;
		iFirstChild = NULL;
	}
}

EXPORT_C TInt CSTMPackageNode::GetFileSize()
{
	TInt size;

	RFs fileServer;
	RFile fileTorrent;
	User::LeaveIfError(fileServer.Connect());
	TInt err = fileTorrent.Open(fileServer,GetNodeFullName(),EFileWrite);
	if (err !=KErrNone) return 0;

	fileTorrent.Size(size);

	fileTorrent.Close();
	fileServer.Close();

	return size;
}
