/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

// INCLUDE FILES
#include <AknQueryDialog.h> 
#include "TorrentMaker.h"
#include "STBencode.h"

#include <aknnotewrappers.h>
//#include <SymTorrentMaker.rsg>
//#include "SymTorrentMaker.hrh"
#include <avkon.hrh>
#include <CAknFileSelectionDialog.h>
#include <CAknMemorySelectionDialog.h> 
#include <aknmessagequerydialog.h> 
#include "hash.h"
#include <utf.h>

#define TEMPORARY_BUFFER_SIZE  10240
#define COPY_BUFFER_SIZE      409600

// storrent constants
_LIT8(KLitBitfield, "bitfield");
_LIT8(KLitPath, "path");


// ================= MEMBER FUNCTIONS =======================
//
// ----------------------------------------------------------
// CSTMTorrentHandler::ConstructL()
// 
// ----------------------------------------------------------
//
void CSTMTorrentHandler::ConstructL()
{
	iTorrentPackage = CSTMPackage::NewL();
	iCurrentNode    = iTorrentPackage->GetRootNode();
	iLastPieceNum	= 0;
}

// Clears the virtual file system
EXPORT_C void CSTMTorrentHandler::Cancel()
{
	delete iTorrentPackage;
	iTorrentPackage = CSTMPackage::NewL();
	iCurrentNode    = iTorrentPackage->GetRootNode();
}

EXPORT_C CSTMTorrentHandler::~CSTMTorrentHandler()
{
	delete iTorrentPackage;
}

EXPORT_C CSTMTorrentHandler* CSTMTorrentHandler::NewL()
{
	CSTMTorrentHandler * self=new(ELeave) CSTMTorrentHandler();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}

EXPORT_C CSTMPackage* CSTMTorrentHandler::GetTorrentPackage()
{
	return iTorrentPackage;
}


EXPORT_C CSTMPackageNode* CSTMTorrentHandler::GetCurrentNode()
{
	return iCurrentNode;
}

EXPORT_C void CSTMTorrentHandler::SetCurrentNode(CSTMPackageNode* aNewCurrentNode)
{
	iCurrentNode = aNewCurrentNode;
}

EXPORT_C TInt CSTMTorrentHandler::GetLastPieceNum()
{
	return iLastPieceNum;
}


EXPORT_C void CSTMTorrentHandler::SaveTorrent(TBool aSaveSTorrent)
{
	RFs fileServer;
	RFile fileTorrent;
	RFile fileSTorrent;
	TBuf<300>   basepath;
	TBuf<300>   torrentfilename;
	TBuf<300>   storrentfilename;
	TBuf<300>   filename;
	TBuf<256>   chosenfolder;

	User::LeaveIfError(fileServer.Connect());
	CleanupClosePushL(fileServer);

	iTorrentPackage->GetTorrentSavePathInto(chosenfolder);
	iTorrentPackage->GetTorrentFileNameInto(filename);

	basepath.Copy(chosenfolder);
	basepath.Append(filename);
	basepath.Append(_L("\\"));
	torrentfilename.Copy(basepath);

	fileServer.MkDirAll(torrentfilename);

	torrentfilename.Append(filename);
	storrentfilename.Copy(torrentfilename);
	torrentfilename.Append(_L(".torrent"));
	storrentfilename.Append(_L(".storrent"));

 	TInt err = fileTorrent.Replace(fileServer,torrentfilename,EFileWrite);
	if (err != KErrNone)
	{
		CleanupStack::PopAndDestroy(); // fileServer
		return;
	}
	CleanupClosePushL(fileTorrent);

	CSTBencode* torrentroot = CreateBencodedTorrent(iTorrentPackage);

	HBufC8* bencodedbuf = torrentroot->BencodeL();
	fileTorrent.Write(*bencodedbuf);

	if (aSaveSTorrent)
	{
 		TInt err2 = fileSTorrent.Replace(fileServer,storrentfilename,EFileWrite);
		if (err2 !=KErrNone) return;

		TInt torrentfilesize;
		User::LeaveIfError(fileTorrent.Size(torrentfilesize));

		TBuf8<10> lengthBuf;
		lengthBuf.Num(torrentfilesize);
		while (lengthBuf.Length() < 10) // fixed 10 decimals length
			lengthBuf.Insert(0, _L8("0"));
		fileSTorrent.Write(lengthBuf);
		fileSTorrent.Write(*bencodedbuf);
		
		// write the bencoded dictionary in the file
		while(1)
		{
			CSTBencodedDictionary* state = new (ELeave) CSTBencodedDictionary;
			CleanupStack::PushL(state);
			
			{
				TBuf8<100> field;										
				field.Copy(_L8(""));

				TInt lastPieceNum = GetLastPieceNum();
				TInt needone = 1+(lastPieceNum-1)/8;
				TChar ffchar = 255;
				for (TInt i=0;i<needone;i++)					
					field.Append(ffchar);

				CSTBencodedString* bitfield = 
					CSTBencodedString::NewL(field);
				CSTBencodedString* key1 = CSTBencodedString::NewL (KLitBitfield);
				state->AddEntryL(key1, bitfield);
			}
			{
				
				HBufC8* encodedPath = HBufC8::NewLC(900);
				TPtr8 encodedPathPtr = encodedPath->Des();
				TBuf<300> contentpath;

				contentpath.Copy(basepath);
				contentpath.Append(_L("content\\"));

				if (CnvUtfConverter::ConvertFromUnicodeToUtf8(encodedPathPtr, contentpath) != 0)
				{
					CleanupStack::PopAndDestroy(); // encodedPath
					break;
				}
				
				CSTBencodedString* path = CSTBencodedString::NewL(*encodedPath);
				CleanupStack::PopAndDestroy(); // encodedPath
				CSTBencodedString* key2 = CSTBencodedString::NewL(KLitPath);
				state->AddEntryL(key2, path);
			}

			HBufC8* stateBencoded = state->BencodeL();
			TBuf8<10> stateLengthBuf;
			stateLengthBuf.Num(stateBencoded->Length());
			while (stateLengthBuf.Length() < 10) // fixed 10 decimals length
				stateLengthBuf.Insert(0, _L8("0"));
			fileSTorrent.Write(stateLengthBuf);
			fileSTorrent.Write(*stateBencoded);
			delete stateBencoded;
			
			CleanupStack::PopAndDestroy(); // state				
			break;
		}
	}

	delete bencodedbuf;
	delete torrentroot;

	basepath.Append(_L("content\\"));

	fileServer.MkDirAll(basepath);

	CopyContent(iTorrentPackage->GetRootNode(), basepath, fileServer);

	CleanupStack::PopAndDestroy(); // fileTorrent
	CleanupStack::PopAndDestroy(); // fileServer
}

void CSTMTorrentHandler::CopyFile(TDesC &aSourceFile, TDesC &aTargetFile, RFs &aFileServer)
{
	RFile sourceFile, targetFile;
	TInt remaining, nextpiece, copied;

	TInt err = sourceFile.Open(aFileServer,aSourceFile,EFileWrite);
	if (err != KErrNone) return;
	err = targetFile.Replace(aFileServer,aTargetFile,EFileWrite);
	if (err != KErrNone) return;

	HBufC8* copybuf = HBufC8::NewL(COPY_BUFFER_SIZE);
	TPtr8 pcopybuf = copybuf->Des();

	copied = 0;
	err = sourceFile.Size(remaining);
	if (err != KErrNone) return;

	while (remaining > 0)
	{
		if (remaining > COPY_BUFFER_SIZE)
			nextpiece = COPY_BUFFER_SIZE;
		else
			nextpiece = remaining;

		sourceFile.Read(copied, pcopybuf, nextpiece);
		targetFile.Write(copied, pcopybuf, nextpiece);

		copied	  += nextpiece;
		remaining -= nextpiece;
	}

	delete copybuf;

	sourceFile.Close();
	targetFile.Close();
}

void CSTMTorrentHandler::CopyContent(CSTMPackageNode* aLocalRoot, TDesC &aPath, RFs &aFileServer)
{
	CSTMPackageNode* tempNode = aLocalRoot->GetFirstChild();
	TBuf<256> tempPath;
	TBuf<256> fullName;

	while (tempNode!=NULL)
	{
		if (tempNode->GetNodeType()==EFolderNode)
		{
			tempPath.Copy(aPath);
			tempPath.Append(tempNode->GetNodeName());
			tempPath.Append(_L("\\"));
			aFileServer.MkDirAll(tempPath);
			CopyContent(tempNode, tempPath, aFileServer);
		}
		else
		{
			tempPath.Copy(aPath);
			tempPath.Append(tempNode->GetNodeName());
			fullName.Copy(tempNode->GetNodeFullName());
			CopyFile(fullName, tempPath, aFileServer);
		}
		tempNode = tempNode->GetNextSibling();
	}
}

CSTBencode* CSTMTorrentHandler::CreateBencodedTorrent(CSTMPackage* aPackage)
{
	CSTBencodedDictionary* torrentroot = new (ELeave) CSTBencodedDictionary;

	AddAnnounce(torrentroot, aPackage);
	AddEncoding(torrentroot);
	AddCreatedBy(torrentroot);
	AddInfo(torrentroot, aPackage);

	return torrentroot;
}


void CSTMTorrentHandler::AddAnnounce(CSTBencodedDictionary* aTorrentroot, CSTMPackage* aPackage)
{
	TBuf8<100>  buf8;
	TBuf<100>	buf;
	TInt        i;

	aPackage->GetAnnounceUrlInto(buf);

	buf8.FillZ();

	for (i=0;i<buf.Length();i++)
	{
		buf8.Append(buf[i]);
	}

	CSTBencodedString* key = CSTBencodedString::NewL(_L8("announce"));
	CSTBencodedString* value = CSTBencodedString::NewL(buf8);

	aTorrentroot->AddEntryL(key,value);
}

void CSTMTorrentHandler::AddEncoding(CSTBencodedDictionary* aTorrentroot)
{
	CSTBencodedString* key = CSTBencodedString::NewL(_L8("encoding"));
	CSTBencodedString* value = CSTBencodedString::NewL(_L8("UTF-8"));

	aTorrentroot->AddEntryL(key,value);
}

void CSTMTorrentHandler::AddCreatedBy(CSTBencodedDictionary* aTorrentroot)
{
	CSTBencodedString* key = CSTBencodedString::NewL(_L8("created by"));
	CSTBencodedString* value = CSTBencodedString::NewL(_L8("SymTorrentMaker 0.1b"));

	aTorrentroot->AddEntryL(key,value);
}

void CSTMTorrentHandler::AddInfo(CSTBencodedDictionary* aTorrentroot, CSTMPackage* aPackage)
{
	CSTBencodedString* key = CSTBencodedString::NewL(_L8("info"));
	CSTBencodedDictionary* value = new (ELeave) CSTBencodedDictionary;
	CSTBencodedDictionaryEntry* entry = new (ELeave) CSTBencodedDictionaryEntry(key, value);

	aTorrentroot->AddEntryL(entry);

	TInt fullsize = AddFiles(value,aPackage);
	AddName(value,aPackage);
	AddPieceLength(value,aPackage);
	AddPieces(value,aPackage,fullsize);
}

TInt CSTMTorrentHandler::AddFiles(CSTBencodedDictionary* aLocalRoot, CSTMPackage* aPackage)
{
	CSTBencodedString* key = CSTBencodedString::NewL(_L8("files"));
	CSTBencodedList* value = new (ELeave) CSTBencodedList;
	CSTBencodedDictionaryEntry* entry = new (ELeave) CSTBencodedDictionaryEntry(key, value);

	aLocalRoot->AddEntryL(entry);

	return SearchFiles(value,aPackage->GetRootNode());
}

TInt CSTMTorrentHandler::SearchFiles(CSTBencodedList* aLocalRoot, CSTMPackageNode* aParentNode)
{
	TInt sumfilesize = 0;

	CSTMPackageNode* aNode = aParentNode->GetFirstChild();

	while (aNode!=NULL)
	{
		switch (aNode->GetNodeType())
		{
			case EFolderNode :
			sumfilesize += SearchFiles(aLocalRoot, aNode);
			break;
			case EFileNode :
			sumfilesize += AddFile(aLocalRoot,aNode);
			break;
		}

		aNode = aNode->GetNextSibling();
	}
	return sumfilesize;
}

TInt CSTMTorrentHandler::AddFile(CSTBencodedList* aLocalRoot, CSTMPackageNode* aNode)
{
	TInt filesize = aNode->GetFileSize();
	CSTBencodedDictionary* dict = new (ELeave) CSTBencodedDictionary;

	CSTBencodedString* key1 = CSTBencodedString::NewL(_L8("length"));
	CSTBencodedInteger* value1 = new (ELeave) CSTBencodedInteger(filesize);
	dict->AddEntryL(key1,value1);

	CSTBencodedString* key2 = CSTBencodedString::NewL(_L8("path"));
	CSTBencodedList* value2 = new (ELeave) CSTBencodedList;
	dict->AddEntryL(key2,value2);

	AddFilePath(value2, aNode);

	aLocalRoot->AppendL(dict);

	return filesize;
}

void CSTMTorrentHandler::AddFilePath(CSTBencodedList* aPathList, CSTMPackageNode* aNode)
{
	if (aNode->GetParent()!=NULL)
	{
		AddFilePath(aPathList, aNode->GetParent());

		TBuf8<100>  buf8;
		TBuf<100>	buf;
		TInt        i;

		buf.Copy(aNode->GetNodeName());

		buf8.FillZ();

		for (i=0;i<buf.Length();i++)
		{
			buf8.Append(buf[i]);
		}

		CSTBencodedString* subpath = CSTBencodedString::NewL(buf8);
		aPathList->AppendL(subpath);
	}
}

void CSTMTorrentHandler::AddName(CSTBencodedDictionary* aLocalRoot, CSTMPackage* aPackage)
{
	TBuf8<100>  buf8;
	TBuf<100>	buf;
	TInt        i;

	aPackage->GetTorrentFileNameInto(buf);

	buf8.FillZ();

	for (i=0;i<buf.Length();i++)
	{
		buf8.Append(buf[i]);
	}

	CSTBencodedString* key = CSTBencodedString::NewL(_L8("name"));
	CSTBencodedString* value = CSTBencodedString::NewL(buf8);

	aLocalRoot->AddEntryL(key,value);
}

void CSTMTorrentHandler::AddPieceLength(CSTBencodedDictionary* aLocalRoot, CSTMPackage* aPackage)
{
	CSTBencodedString* key    = CSTBencodedString::NewL(_L8("piece length"));
	CSTBencodedInteger* value = new (ELeave) CSTBencodedInteger(1024*(aPackage->GetPieceSize()));

	aLocalRoot->AddEntryL(key,value);
}

CSTMPackageNode* CSTMTorrentHandler::GetNextFile(CSTMPackageNode* aCurrentNode)
{
	CSTMPackageNode* current = aCurrentNode;
	TInt done;

	if (current!=NULL && current->GetNodeType()==EFileNode)
	{
		if (current->GetNextSibling()!=NULL)
			current = current->GetNextSibling();
		else
		{
			done = 0;
			while (!done && current!=NULL)
			{
				current = current->GetParent();
				if (current!=NULL && current->GetNextSibling()!=NULL)
				{
					current = current->GetNextSibling();
					done = 1;
				}
			}
		}
	}

	while (current!=NULL && current->GetNodeType()!=EFileNode)
	{
		if (current->GetFirstChild()!=NULL)
			current = current->GetFirstChild();
		else
		{
			if (current->GetNextSibling()!=NULL)
				current = current->GetNextSibling();
			else
			{
				done = 0;
				while (!done && current!=NULL)
				{
					current = current->GetParent();
					if (current!=NULL && current->GetNextSibling()!=NULL)
					{
						current = current->GetNextSibling();
						done = 1;
					}
				}
			}
		}
	}
	return current;
}

void CSTMTorrentHandler::AddPieces(CSTBencodedDictionary* aLocalRoot, CSTMPackage* aPackage, TInt aFullsize)
{

	TInt piecesize = 1024*(aPackage->GetPieceSize());

	TInt piecenum = 1+(((aFullsize-1)/piecesize));
	iLastPieceNum	= piecenum;
	TBuf8<25> onePieceSha1;
//	HBufC8* pieceSha1 = HBufC8::NewL(20*piecenum+5); // TODO
	HBufC8* pieceSha1 = HBufC8::NewL(140*1024);
	TPtr8 pPieceSha1(pieceSha1->Des());
	pPieceSha1.Copy(_L8(""));

	HBufC8* piece = HBufC8::NewL(piecesize+5);
	TPtr8 pPiece(piece->Des());
	pPiece.FillZ();
	pPiece.Copy(_L8(""));

	HBufC8* tempbuf = HBufC8::NewL(TEMPORARY_BUFFER_SIZE+5);
	TPtr8 pTempbuf(tempbuf->Des());

	RFs fileServer;
	RFile fileContent;
	User::LeaveIfError(fileServer.Connect());

	CSTMPackageNode* current = aPackage->GetRootNode();
	TInt actFilePos = 0, actFileSize=0, readSize,
		 actPieceSize = 0, fullPieceSize = piecesize;

	current = GetNextFile(current);
	if (current!=NULL)
	{
		fileContent.Open(fileServer, current->GetNodeFullName(), EFileWrite);
		fileContent.Size(actFileSize);
	}

	while (current!=NULL)
	{

		while (actPieceSize<fullPieceSize && current!=NULL)
		{

			while (actFilePos<actFileSize && actPieceSize<fullPieceSize)
			{
				if (actFileSize-actFilePos>fullPieceSize-actPieceSize)
					readSize = fullPieceSize-actPieceSize;
				else
					readSize = actFileSize-actFilePos;

				if (readSize>TEMPORARY_BUFFER_SIZE)
					readSize = TEMPORARY_BUFFER_SIZE;

				pTempbuf.FillZ();
				pTempbuf.Copy(_L8(""));
				fileContent.Read(actFilePos, pTempbuf, readSize);
				actFilePos += readSize;
				actPieceSize +=readSize;
				pPiece.Append(pTempbuf);
			}

			if (actFilePos>=actFileSize)
			{
				fileContent.Close();
				current = GetNextFile(current);

				if (current!=NULL)
				{
					fileContent.Open(fileServer, current->GetNodeFullName(), EFileRead);
					fileContent.Size(actFileSize);
					actFilePos = 0;
				}
			}
		}

//		pPieceSha1.Append(mySha->Hash(pPiece));

		CSHA1* mySha = CSHA1::NewL();
		CleanupStack::PushL(mySha);
		pPieceSha1.Append(mySha->Hash(pPiece));
		CleanupStack::PopAndDestroy();
		
		pPiece.FillZ();
		pPiece.Copy(_L8(""));
		actPieceSize = 0;

	}

	fileServer.Close();

	CSTBencodedString* key   = CSTBencodedString::NewL(_L8("pieces"));
	CSTBencodedString* value = CSTBencodedString::NewL(pPieceSha1);

	aLocalRoot->AddEntryL(key,value);

	delete tempbuf;
	delete piece;
	delete pieceSha1;
}
