/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

#define PROXYCONNECTOR_ENABLED

#ifndef STTCONNECTOR_H
#define STTCONNECTOR_H

// INCLUDES
#include <e32base.h>
#include <es_sock.h>
#include <in_sock.h>
#include "STTConnectorEvents.h"
#include "STTTransferSocketInitializator.h"
#include "STTSocketCollector.h"
#include "STTBaseData.h"
#include "STTTimeOutHandler.h"

#ifdef EKA2
#include "STTImeiInfoSupplier.h"
#endif


// CONSTANTS

// FORWARD DECLARATIONS

// CLASS DECLARATION

enum TConnectorStates 
{
	EInitializing = 1,
	EInitialized,
	EConnecting,
	EReceivingInitialization,
	ESendingInitialization,
	EWaitingForConnections,
	EProcessingIncomingMessage
};

enum TConnectorListenerStates 
{
	EOffline = 1000,
	EWaitingForConnection,
	EProcessingConnection
};

enum TConnectorSimpleStates 
{
	EConOffline = 2000,
	EConConnecting,
	EConInitializing,
	EConConnected,
	EConnectedAndInitialized
};

enum TConnectorType
{
	EConProxy = 3000,
	EConListener
};

/**
*  CSTTConnectorProxy class.
* 
*/
class CSTTConnector : public CActive, public MTimeOutObserver
#ifdef EKA2
,public MImeiInfoObserver
#endif
    {
    public: // Constructors and destructor

		static CSTTConnector* NewL(RSocketServ* aSocketServ, MSTTSocketCollector* aSocketCollector, CSTTBaseData* aBaseData, TDesC& aIMEIPostFix, RConnection* aConnection=NULL, MSTTConnectorEvents* aConnectorEventHandler=NULL);

        /**
        * Destructor.
        */
        ~CSTTConnector();

	protected:
		// -- active object --
		void DoCancel();
        /**
        * EPOC default constructor.
        */
        void ConstructL(TDesC& aIMEIPostFix);
		CSTTConnector(RSocketServ* aSocketServ, MSTTSocketCollector* aSocketCollector, CSTTBaseData* aBaseData, RConnection* aConnection, MSTTConnectorEvents* aConnectorEventHandler);

#ifdef EKA2
	private:
		void ImeiInfoReceivedL(CTelephony::TPhoneIdV1Pckg& aImeiInfo);
		void GettingImeiFailedL();
#endif


		void CheckAndAddTransferSocket(TPtrC8 aConnectionId);
		void ReleaseAllTransferSocket();

		void MyAppend8_16(TDes8 &aTarget, TDesC16 &aSource);
		void MyAppend16_8(TDes16 &aTarget,TDesC8 &aSource);

    public: // New functions
    
    	/**
    	 * @return KErrNone if no errors occured
    	 */
		TInt StartL(TConnectorType aConnectorType, RConnection* aConnection = NULL);
		
		/**
		 * @param aLocalAddress contains the address of the device on which it accepts incoming connections
		 *        if getting the address fails it will be set to 0 
		 * 
    	 * @return KErrNone if no errors occured
    	 */
		TInt StartL(TConnectorType aConnectorType, RConnection* aConnection, TInetAddr& aLocalAddress);

		void Stop();
		void ProcessConnection();
		void SetConnectionTimeOut(TInt aConnectionTimeOut);
		TInt GetConnectionTimeOut();
		TInt GetLocalIPAddressSetConenctionL(TInetAddr& aAddr);
		inline TConnectorType GetConnectorType();

	public: // from MSTTConnectorBase
		TConnectorSimpleStates GetConnectorState();
		HBufC* GetIMEI();
		TInt GetLastErrorCode();

	public: // from MTimeOutObserver
		void TimeOutExpired();

	private:
		TConnectorStates GetState();
		TInt GetLocalIPAddressL(TInetAddr& aAddr, RConnection* aConnection = NULL);
		

#ifdef PROXYCONNECTOR_ENABLED
		void ReceiveInitialization();
		void ContinueReading();
		void ReadIncoming();
		void SendInitialization();
		void WaitForFirstConnection();
		void WaitForNextConnection();
		void ReadIncomingMessage();
		void ProcessIncoming();
#endif

    private:
		// -- active object --
		void RunL();

    private: // Data
		TConnectorSimpleStates				iSimpleState;
		HBufC*								iIMEI;
		HBufC*								iIMEIPostFix;
		RConnection*						iConnection;
		TInetAddr 							iProxyAddress;
		
		#ifdef EKA2
		CImeiInfoSupplier*					iImeiInfoSupplier;
		#endif
		
		TBool								iGettingImei;
		TBool								iConnectIssued;
		TInt								iLastErrorCode;
		TInt								iConnectionTimeOut;
		RSocketServ*						iSocketServ;
		RSocket								iSocket;
		CSTTBaseData*						iBaseData;
		MSTTConnectorEvents*				iConnectorEventHandler;
		
#ifdef PROXYCONNECTOR_ENABLED
		CSTTTransferSocketInitializator*	iTransferSocket;
#endif
		MSTTSocketCollector*				iSocketCollector;

		CTimeOutHandler*					iTimeOutHandler;
		TConnectorStates					iState;
		TBuf8<1024>							iTransferingBuffer;
		TBuf8<1024>							iTemporaryBuffer;
		TSockXfrLength						iRecievedLength;

		TConnectorListenerStates			iListenerState;
		RSocket								iSocketListener;
		TConnectorType						iConnectorType;

		RSocket*							iBlankSocket;
    };

inline TConnectorType CSTTConnector::GetConnectorType()
{
	return iConnectorType;
}

#endif

// End of File
