/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

#ifndef STTSERVICESOCKET_H
#define STTSERVICESOCKET_H

// INCLUDES
#include <e32base.h>
#include <es_sock.h>
#include <in_sock.h>
#include "STBencode.h"

class CSymTorrentTrackerManager;
class CSTTTorrentPeerItem;
class CSTTTorrentListItem;
class CKiLogger;

/* TORRENT TRACKER PROTOCOL ************************************************ */
enum TSSServiceStates 
{
	ESSInitializing = 1,
	ESSWaitingForIncoming,
	ESSProcessIncoming,
	ESSPrepareSending,
	ESSSendingResponseHeader,
	ESSSendingResponse,
	ESSFinished
};

enum TSSTorrentEvent 
{
	ESSStarted = 1000,
	ESSCompleted,
	ESSStopped,
	ESSNotSepecified
};

struct TTorrentRequest
{
	TBool			finished;
	HBufC8*			requestedUrl;
	HBufC8*			infoHash;
	HBufC8*			peerId;
	TInt			port;
	TInt			uploaded;
	TInt			downloaded;
	TInt			left;
	HBufC8*			compact;  // ??
	TSSTorrentEvent event;
	HBufC8*			ip;
	TInt			numwant;
	HBufC8*			key;
	HBufC8*			trackerId;
};

struct TTorrentResponse
{
	HBufC8*					errorMessage;
	TBool					torrentFound;
	HBufC8*					warningMessage;
	TInt					interval;
	TInt					minInterval;
	HBufC8*					trackerId;
	TInt					complete;
	TInt					incomplete;
	CSTTTorrentPeerItem*	firstPeer;
};

/**
*  CSTTServiceSocket class.
* 
*/
class CSTTServiceSocket : public CActive
{
    public: // Constructors and destructor

		static CSTTServiceSocket* NewL(RSocket& aSocket, TInetAddr &aPartnerAddress, CSymTorrentTrackerManager* aTrackerManager);

        /**
        * Destructor.
        */
        ~CSTTServiceSocket();

	protected:
        /**
        * EPOC default constructor.
        */
        void ConstructL();

		CSTTServiceSocket(RSocket& aSocket, TInetAddr &aPartnerAddress, CSymTorrentTrackerManager* aTrackerManager);

	public: // from base classes
		// -- active object --
		void DoCancel();

	public: // new functions
		TBool IsFinished();
		CSTTServiceSocket* GetNextSibling();
		void SetNextSibling(CSTTServiceSocket* aNewNextSibling);

    private: // from base class
		// -- active object --
		void RunL();

	private: // new functions
		// new functions
		void TransferDisconnect();
		void ReceiveMessage();
		void ContinueReading();
		void ReadIncoming();
		TBool ProcessIncomingL();

		void SendRequestedInformationL();
		void SendRequestedContentL();
		void AfterSending();

		// Add peer to list
		void AddPeer(CSTBencodedList* aPeerList, const TDesC8& aPeerId, const TDesC8& aIp, TInt aPort);
		// Process request
		void ProcessRequestL();
		// Generate response to the request
		HBufC8* GenerateResponseL();
		// Decode Request
		void DecodeRequestL();
		// Appends a 8bit descriptor to a 16bit descriptor
		void MyAppend16_8(TDes16 &aTarget, TDesC8 &aSource);
		// Encode Url
		void EncodeUrl(TDesC8 &aData, TDes8 &aResult);
		// Decode Url
		void DecodeUrl(TDesC8 &aData, TDes8 &aResult);
		// Returns the needed parameter value
		void GetParameter(TDesC8 &aUrl, TDesC8 &aParameter, TDes8 &aValue);

		CSTTTorrentListItem* FindTorrent(TDesC8 &aInfoHash);
		void ComposeResponse(CSTTTorrentListItem* aListitem);

    private:
		TSSServiceStates					iState;
		RSocket								iSocket;
		CSTTServiceSocket*					iNextSibling;

		TInetAddr							iPartnerAddress;

		TTorrentRequest						iCurrentRequest;
		TTorrentResponse					iCurrentResponse;

		TBuf8<1024>							iTemporaryBuffer;
		TSockXfrLength						iRecievedLength;
		HBufC8*								iRecievedBuf;

		HBufC8*								iResponseHeader;
		HBufC8*								iResponse;
		
		TInetAddr							iRemoteAddressFromSocket;
		
		CKiLogger*							iLog;
		
		CSymTorrentTrackerManager* iTrackerManager;
};

#endif

// End of File
