/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

#ifndef STTTORRENTLISTITEM_H
#define STTTORRENTLISTITEM_H

// INCLUDES
#include <e32base.h>
#include "STTTorrentPeerItem.h"
#include "STTTorrentItemObserver.h"

// CLASS DECLARATION

/**
*  CSTTTorrentListItem class.
* 
*/
class CSTTTorrentListItem : public CBase
{
    public: // Constructors and destructor

		static CSTTTorrentListItem* NewL(TPtrC aTorrentFileName, TPtrC8 aInfoHash, MSTTTorrentItemObserver* aTorrentObserver);

        /**
        * Destructor.
        */
        ~CSTTTorrentListItem();

	protected:
        /**
        * EPOC default constructor.
        */
        void ConstructL(TPtrC aTorrentFileName, TPtrC8 aInfoHash, MSTTTorrentItemObserver* aTorrentObserver);

		CSTTTorrentListItem();

    public: // New functions
		TPtr GetTorrentFileName();
		void SetTorrentFileNameL(TPtrC aTorrentFileName);
		TPtr8 GetInfoHash();
		void SetInfoHashL(TPtrC8 aInfoHash);

		CSTTTorrentListItem* GetNextSibling();
		void SetNextSibling(CSTTTorrentListItem* aNextSibling);

		CSTTTorrentListItem* InsertAfterL(TPtrC aTorrentFileName, TPtrC8 aInfoHash, MSTTTorrentItemObserver* aTorrentObserver);
		void DeleteNext();

		// Peer list handling
		CSTTTorrentPeerItem* InsertPeerItemL(TInetAddr aAddress, TDesC8& aId);
		TBool DeletePeerItem(TInetAddr aAddress, TDesC8& aId);
		void DeleteSimilarPeers(TInetAddr aAddress, TDesC8& aId);
		void DeleteAllPeerItem();
		CSTTTorrentPeerItem* GetPeerListFirstItem();
		
		/**
		 * @return a peer item with the given address and peer id, NULL if no matching peer
		 * is found
		 */
		CSTTTorrentPeerItem* FindPeerItem(TInetAddr aAddress, TDesC8& aId);
		
		/**
		 * @return a peer item with the given address, NULL if no matching peer is found
		 */
		CSTTTorrentPeerItem* FindPeerItem(TInetAddr aAddress);

		void RemoveNeedlessPeers();
		TInt PeerListLength();


    private: // Data
		HBufC*						iTorrentFileName;
		HBufC8*						iInfoHash;

		CSTTTorrentListItem*		iNextSibling;

		CSTTTorrentPeerItem*		iPeerListHead;
		MSTTTorrentItemObserver*	iTorrentObserver;

};

#endif

// End of File
