/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

#ifndef STTTORRENTPEERITEM_H
#define STTTORRENTPEERITEM_H

// INCLUDES
#include <e32base.h>
#include <in_sock.h>

// CLASS DECLARATION

/**
*  CSTTTorrentListItem class.
* 
*/
class CSTTTorrentPeerItem : public CBase
{
    public: // Constructors and destructor

		static CSTTTorrentPeerItem* NewL(TInetAddr aAddress, TDesC8& aId);

        /**
        * Destructor.
        */
        ~CSTTTorrentPeerItem();

	protected:
        /**
        * EPOC default constructor.
        */
        void ConstructL(TInetAddr aAddress, TDesC8& aId);

		CSTTTorrentPeerItem();

    public: // New functions

		inline CSTTTorrentPeerItem* GetNextSibling();
		inline void SetNextSibling(CSTTTorrentPeerItem* aNextSibling);

		CSTTTorrentPeerItem* InsertAfterL(TInetAddr aAddress, TDesC8& aId);
		void DeleteNext();

		inline void SetLastCheckInTime(TTime& aLastCheckInTime);
		inline TTime LastCheckInTime();
		inline void SetLastCheckInTimeNow();	
		inline TBool Completed();
		inline TBool PeerMatch(TInetAddr aAddress, TDesC8& aId);
		inline TBool PeerMatch(TInetAddr aAddress);
		inline TUint32 Address();
		inline TInt Port();
		inline HBufC8* PeerId();
		
		TInt SecondsFromLastCheckInTime();
		void SetCompleted(TBool aCompleted);
		HBufC8* PeerIp();

    private: // Data
		CSTTTorrentPeerItem*		iNextSibling;

		TInetAddr					iAddress;
		HBufC8*						iId;
		HBufC8*						iIp;
		TBool						iCompleted;
		TTime						iLastCheckInTime;

};

// INLINE DEFINITIONS

inline TBool CSTTTorrentPeerItem::PeerMatch(TInetAddr aAddress, TDesC8& aId) {
	return (	(aAddress.Port()==iAddress.Port() && aAddress.Address()==iAddress.Address()) 
			|| 	(*iId == aId));
}

inline TBool CSTTTorrentPeerItem::PeerMatch(TInetAddr aAddress) {
	return (aAddress.Port()==iAddress.Port() && aAddress.Address()==iAddress.Address());
}

inline TUint32 CSTTTorrentPeerItem::Address() {
	return iAddress.Address();
}

inline TInt CSTTTorrentPeerItem::Port() {
	return iAddress.Port();
}

inline HBufC8* CSTTTorrentPeerItem::PeerId() {
	return iId;
}

inline CSTTTorrentPeerItem* CSTTTorrentPeerItem::GetNextSibling() {
	return iNextSibling;
}

inline void CSTTTorrentPeerItem::SetNextSibling(CSTTTorrentPeerItem* aNextSibling) {
	iNextSibling = aNextSibling;
}

inline void CSTTTorrentPeerItem::SetLastCheckInTime(TTime& aLastCheckInTime) {
	iLastCheckInTime = aLastCheckInTime;
}

inline TTime CSTTTorrentPeerItem::LastCheckInTime() {
	return iLastCheckInTime;
}

inline void CSTTTorrentPeerItem::SetCompleted(TBool aCompleted) {
	iCompleted = aCompleted;
}

inline TBool CSTTTorrentPeerItem::Completed() {
	return iCompleted;
}

inline void CSTTTorrentPeerItem::SetLastCheckInTimeNow() {
	iLastCheckInTime.HomeTime();
}


#endif

// End of File
