/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

#ifndef STTTRANSFERSOCKETINITIALIZATOR_H
#define STTTRANSFERSOCKETINITIALIZATOR_H

// INCLUDES
#include <e32base.h>
#include <es_sock.h>
#include <in_sock.h>

// CONSTANTS

// FORWARD DECLARATIONS
class MSTTSocketCollector;

// CLASS DECLARATION

enum TTransferSocketStates 
{
	ETSInitializing = 1,
	ETSConnecting,
	ETSSendingInitialization,
	ETSWaitingForData,
	ETSFinished
};

/**
 *  CSTTConnector
 */
class CSTTTransferSocketInitializator : public CActive
    {
    public: // Constructors and destructor

		static CSTTTransferSocketInitializator* NewL( 
			TPtrC aServerName, TInt aServicePort, TPtrC8 aConnectionId, MSTTSocketCollector* aSocketCollector);

        /**
        * Destructor.
        */
        ~CSTTTransferSocketInitializator();

		// -- active object --
		void DoCancel();

	protected:
        /**
        * EPOC default constructor.
        */
        void ConstructL(TPtrC aServerName, TInt aServicePort, TPtrC8 aConnectionId);

		CSTTTransferSocketInitializator(MSTTSocketCollector* aSocketCollector);


    public: // Functions from base classes

    public: // New functions
		TTransferSocketStates GetState();
		CSTTTransferSocketInitializator* GetNextSibling();
		void SetNextSibling(CSTTTransferSocketInitializator* aNewNextSibling);

    private:
		// -- active object --
		void RunL();

		void TransferConnectL();
		void TransferDisconnect();
		void SendInitialization();
		void StartWaitingForData();
		void WaitForData();
		void ProcessData();
		void FinishInitialization();

		void Error();

    private: // Data
		TTransferSocketStates				iState;
		RSocketServ							iSockServ;
		RSocket*							iSocket;
		
		HBufC*								iServerName;
		HBufC8*								iConnectionId;
		TInt								iServicePort;
		TBool								iIpSet;
		TBool								iPortSet;
		TInetAddr							iPartnerAddress;

		TBuf8<1024>							iSendingBuffer;
		TBuf8<1>							iReceivingBuffer;
		TBuf8<100>							iTemporaryBuffer;
		TSockXfrLength						iRecievedLength;
		CSTTTransferSocketInitializator*	iNextSibling;
		MSTTSocketCollector*				iSocketCollector;
		
		/**
		 * Inidicates that the ownership of the socket has been taken
		 */
		TBool								iSocketPassed;
    };

#endif

// End of File
