/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

#ifndef SYMTORRENTMAKERDETAILSCONTAINER_H
#define SYMTORRENTMAKERDETAILSCONTAINER_H

// INCLUDES
#include <coecntrl.h>
#include <eikrted.h>
#include "SymTorrentTrackerManager.h"

// FORWARD DECLARATIONS
class CSymTorrentMakerFileListView;
class MAknsControlContext; // for skins support

// CLASS DECLARATION

enum TTextStyles 
{
	ENormalTextWithLeftMargin = 1,
	ESmallTitleText
};

/**
*  CSymTorrentMakerContainer  container control class.
*  
*/
class CSymTorrentMakerDetailsContainer : public CCoeControl, MCoeControlObserver 
    {
    public: // Constructors and destructor
        /**
        * EPOC default constructor.        
        */
        void ConstructL(const TRect& aRect, CSymTorrentTrackerManager* aTrackerManager);

        /**
        * Destructor.
        */
        ~CSymTorrentMakerDetailsContainer();

    public: // Functions from base classes

       /**
        * From CoeControl,SizeChanged.
        */
        void SizeChanged();

       /**
        * From CoeControl,CountComponentControls.
        */
        TInt CountComponentControls() const;

       /**
        * From CCoeControl,ComponentControl.
        */
        CCoeControl* ComponentControl(TInt aIndex) const;

       /**
        * From CCoeControl,Draw.
        */
        void Draw(const TRect& aRect) const;
		// Pass skin information if needed.
		TTypeUid::Ptr MopSupplyObject(TTypeUid aId);     

		/**
		* From MCoeControlObserver
		* Acts upon changes in the hosted control's state. 
		*
		* @param	aControl	The control changing its state
		* @param	aEventType	The type of control event 
		*/

        void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);
		/**
		* Key events
		*/
		TKeyResponse OfferKeyEventL(const TKeyEvent &aKeyEvent,TEventCode aType);     
	private: // new functions
		void InitializeText();
		void SetTextStyle(TTextStyles aTextStyle);
		void AppendText(const TDesC& aBuf);
		void AppendText(const TChar& aChar);
    private: //data
		CEikRichTextEditor			*iTextWindow;
		MAknsControlContext*		iBackGround; // for skins support
		
		CSymTorrentTrackerManager* iTrackerManager;
    };

#endif

// End of File
