/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

#ifndef SYMTORRENTMAKERFILELISTCONTAINER_H
#define SYMTORRENTMAKERFILELISTCONTAINER_H

// INCLUDES
#include <coecntrl.h>
#include <aknlists.h> 
#include "TorrentMaker.h"
#include "SymTorrentMakerFileListView.h"

// FORWARD DECLARATIONS
class CSymTorrentMakerFileListView;

// CLASS DECLARATION

/**
*  CSymTorrentMakerContainer  container control class.
*  
*/
class CSymTorrentMakerFileListContainer : public CCoeControl, MCoeControlObserver
{
    public: // Constructors and destructor
        
        /**
        * EPOC default constructor.
        * @param aRect Frame rectangle for container.
        */
        void ConstructL(const TRect& aRect, CSymTorrentMakerFileListView *aMyView);

        /**
        * Destructor.
        */
        ~CSymTorrentMakerFileListContainer();

    public: // New functions
		/**
		* Add a given node to the list
		*/
		void AddListItem(CSTMPackageNode* aNode, TBool aRootItem);
		/**
		* Remove all elements from the list
		*/
		void ClearList();
		/**
		* Sets the cursor to the specified position
		*/
		void SetCursorPosition(TInt aPosition);
		/**
		* Gets the cursor position
		*/
		TInt GetCursorPosition();
		/**
		* Gets the current item's type
		*/
		TInt GetCurrentItemType();


    public: // Functions from base classes

        /**
        * OfferKeyHandle
        */
		TKeyResponse OfferKeyEventL(const TKeyEvent &aKeyEvent,TEventCode aType);

    private: // Functions from base classes

       /**
        * From CoeControl,SizeChanged.
        */
        void SizeChanged();

       /**
        * From CoeControl,CountComponentControls.
        */
        TInt CountComponentControls() const;

       /**
        * From CCoeControl,ComponentControl.
        */
        CCoeControl* ComponentControl(TInt aIndex) const;

       /**
        * From CCoeControl,Draw.
        */
        void Draw(const TRect& aRect) const;

       	
       /**
	* From MCoeControlObserver
	* Acts upon changes in the hosted control's state. 
	*
	* @param	aControl	The control changing its state
	* @param	aEventType	The type of control event 
	*/
        void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);
        
    private: //data
	CAknSingleGraphicStyleListBox	*iListBox;
	CSymTorrentMakerFileListView	*iMyView;

};

#endif

// End of File
