/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

#ifndef SYMTORRENTMAKERFILELISTVIEW_H
#define SYMTORRENTMAKERFILELISTVIEW_H

// INCLUDES
#include <aknview.h>
//#include <CAknMemorySelectionDialog.h> 
//#include "SymTorrentTrackerManager.h"

// CONSTANTS

// FORWARD DECLARATIONS
class CSymTorrentMakerFileListContainer;
class CSymTorrentAppUi;
class CSymTorrentTrackerManager;
class CSTMPackageNode;
class CSTMTorrentHandler;

// CLASS DECLARATION

/**
*  CSymTorrentMakerFileListView view class.
* 
*/
class CSymTorrentMakerFileListView : public CAknView
    {
    public: // Constructors and destructor
    
    	CSymTorrentMakerFileListView(CSymTorrentTrackerManager* aTrackerManager);

        /**
        * EPOC default constructor.
        */
        void ConstructL();

        /**
        * Destructor.
        */
        ~CSymTorrentMakerFileListView();

    public: // Functions from base classes
        
        /**
        * From CAknView returns Uid of View
        * @return TUid uid of the view
        */
        TUid Id() const;

        /**
        * From MEikMenuObserver delegate commands from the menu
        * @param aCommand a command emitted by the menu 
        * @return void
        */
        void HandleCommandL(TInt aCommand);

        /**
        * From CAknView reaction if size change
        * @return void
        */
        void HandleClientRectChange();

		void DynInitMenuPaneL(TInt aResourceId, CEikMenuPane* aMenuPane );

    public: // New functions 

		/**
		* Refreshing list from nodes
		*/
		void RefreshList();
		/**
		* Adding a chosen file to nodes
		*/
		void AddFileToListL();
		/**
		* Adding a new empty folder to nodes
		*/
		void AddNewFolderToListL();
		/**
		* Adding an existing folder to nodes
		*/
		void AddFolderToListL();
		/**
		* Enter specified subfolder node
		*/
		void EnterFolderOrShowDetailNode(TInt aPosition);
		/**
		* Remove specified node
		*/
		void RemoveNode(TInt aPosition);
    private:
		/**
		* Adding an existing folder to nodes with full content
		*/
		void AddSpecificFolderToListL(TPtrC aFolderPath, CSTMPackageNode* aParentNode);
		/**
		* File select dialog with memory card selection
		*/
		//TBool FileSelectQueryL(TFileName &aFileName, TCommonDialogType aDialogType);

    private:

        /**
        * From CAknView activate the view
        * @param aPrevViewId 
        * @param aCustomMessageId 
        * @param aCustomMessage 
        * @return void
        */
        void DoActivateL(const TVwsViewId& aPrevViewId,TUid aCustomMessageId,
            const TDesC8& aCustomMessage);

        /**
        * From CAknView deactivate the view (free resources)
        * @return void
        */
        void DoDeactivate();


    private: // Data
        CSymTorrentMakerFileListContainer	*iContainer;
		CSymTorrentAppUi					*iAppUi;
		CSTMTorrentHandler					*iTorrentHandler;
		
		CSymTorrentTrackerManager*          iTrackerManager;
    };

#endif

// End of File
