/*****************************************************************************
 * Copyright (C) 2006-2007 Imre Kelnyi
 *-------------------------------------------------------------------
 * This file is part of SymTorrent
 *
 * SymTorrent is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTorrent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTorrent; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

#ifndef SYMTORRENTTRACKERMANAGER_H__
#define SYMTORRENTTRACKERMANAGER_H__

#include "SymTorrentTracker.hrh"
#include "STTBaseData.h"
//#include "STTConnector.h"
#include "STTSocketCollector.h"
#include "SymTorrentTrackerTorrentListView.h"
#include "STTConnectorEvents.h"

//#include "TorrentMaker.h"
#include "SymTorrentMakerFileListView.h"
//#include "STBencode.h"
#include <CAknCommonDialogsBase.h>
#include <ES_SOCK.H>


// TRACKER FORWARD DECLARATIONS
class CSymTorrentTrackerContainer;
class CSymTorrentTrackerTorrentListView;
class CAknViewAppUi;
class MSymTorrentViewChanger;
class CSTTorrentManager;
class CSTTConnector;

struct TFileDetails
{
	TBuf<128> fullPath;
	TInt	  filesize;
};

class MTrackerUIConnection
{
public:
	virtual void OnTrackerViewChangedL(TInt aViewId) = 0;
	virtual void TrackerChangeNaviPaneTextL(const TDesC& aText) = 0;
	virtual void TrackerRemoveNaviPaneTextL() = 0;
	virtual void TrackerActiveViewL(TSymTorrentTrackerViewIds aViewId) = 0;
};

/**
 * CSymTorrentTrackerManager
 */
class CSymTorrentTrackerManager : public CBase
{
public:

	IMPORT_C CSymTorrentTrackerManager(CAknViewAppUi* aAppUi, CSTTorrentManager* aTorrentMgr, MTrackerUIConnection* aUiConn);

	IMPORT_C void ConstructL();
	
	IMPORT_C ~CSymTorrentTrackerManager();
	
	inline MTrackerUIConnection* UiConn();
	
public: // Tracker functions

	// Returns the BaseData object
	inline CSTTBaseData* GetBaseData() const;

	// Returns the Connector's state
	IMPORT_C TConnectorSimpleStates GetConnectorState();

	// TorrentMaker functions
	inline CSTMTorrentHandler* GetTorrentHandler() const;
	TBool EnterFolderOrShowDetails(TInt aPosition);
	inline TFileDetails GetFileDetails() const;
	TBool FileSelectQueryL(TFileName &aFileName, TCommonDialogType aDialogType, TBuf<64> aTitle);
	// returns ETrue, if the TorrentMaker is acitve, else returns EFalse
	inline TBool InTorrentMaker() const;
	// returns ETrue, if the Tracker is acitve, else returns EFalse
	inline TBool InTracker() const;
	// Selects torrent file
	TBool FileSelectQuery(TFileName &aFileName);
	// sets the titlepane label
	void setTitlePane(TBuf<256> aTitle);
	// Cancel maker duties
	void CancelMaker();

	void TrackerCmdCreateTorrent();

	void TrackerCmdStartTracker();

	void TrackerCmdStopTracker();

	void TrackerCmdAddTorrent();

	void MakerCmdBack();

	void MakerCmdSave();

	void SaveTorrent();

	CSTMTorrentHandler* TorrentHandler();
	
	IMPORT_C void SetPieceSize(TInt aPieceSize);
	
	IMPORT_C void NetworkConnectionStartedL(TBool aResult, RConnection& aConnection);
	
	inline CAknViewAppUi* AppUi() const;
	
	inline void SetInTracker(TBool aInTracker);
	
//	inline MSymTorrentViewChanger* ViewChanger();
	
public:

	IMPORT_C void StartTrackerL(RConnection* aConnection = NULL);
	
	// Saves tracker datas (settings, torrents) to file
	IMPORT_C void SaveTrackerDatasToFile();

private: // Tracker functions

	// Initialize base data
	void InitializeL();

	// Loads tracker datas (settings, torrents) from file
	void LoadTrackerDatasFromFileL();

	// Add new torrent file to list
	TBool AddTorrentFileL(const TDesC& aFilename);

	// Reads settingName
	TBool ReadSettingName(RFile& aFile, TDes8& aSettingName, TInt& aSettingLength);
	// Converts to utf8
	HBufC8* ConvertToUtf8L(const TDesC& aData);
	// Converts from utf8
	HBufC* ReadAndConvertFromUtf8ToUnicodeL(RFile& aFile, TInt aLength);
	// Create storrent
	TBool CreateSTorrent(const TDesC& torrentfilename, const TDesC& storrentfilename, const TDesC& contentpath);

	// Saves a setting item to file
	void SaveSettingL(RFile& aFile, const TDesC8& aSettingName, const TDesC& aSettingValue);
	
private:

	CAknViewAppUi* iAppUi;
	
	CSTTorrentManager* iTorrentMgr;
	
//	MSymTorrentViewChanger* iViewChanger;
	
	
	// --- Tracker components

	CSymTorrentTrackerTorrentListView*  iTorrentListView;
	CSTTBaseData*						iBaseData;
	CSTTConnector*						iConnector;
	MSTTSocketCollector*				iSocketCollector;
	// TorrentMaker components
	TFileDetails						iFileDetails;
	CSTMTorrentHandler					*iTorrentHandler;
	TBool								iInTorrentMaker;
	TBool								iInTracker;
	TBool								iStartTrackerIssued;
	
	MTrackerUIConnection*				iUiConn;
};


// Returns the BaseData object
inline CSTTBaseData* CSymTorrentTrackerManager::GetBaseData() const {
	return iBaseData;
}

inline CSTMTorrentHandler* CSymTorrentTrackerManager::GetTorrentHandler() const {
	return iTorrentHandler;
}

inline TFileDetails CSymTorrentTrackerManager::GetFileDetails() const {
	return iFileDetails;
}

// returns ETrue, if the TorrentMaker is acitve, else returns EFalse
inline TBool CSymTorrentTrackerManager::InTorrentMaker() const {
	return iInTorrentMaker;
}

// returns ETrue, if the Tracker is acitve, else returns EFalse
inline TBool CSymTorrentTrackerManager::InTracker() const {
	return iInTracker;
}

inline void CSymTorrentTrackerManager::SetInTracker(TBool aInTracker) {
	iInTracker = aInTracker;
}

inline CAknViewAppUi* CSymTorrentTrackerManager::AppUi() const {
	return iAppUi;
}

//inline MSymTorrentViewChanger* CSymTorrentTrackerManager::ViewChanger() {
//	return iViewChanger;
//}

inline MTrackerUIConnection* CSymTorrentTrackerManager::UiConn() {
	return iUiConn;
}


#endif

