/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

#ifndef SYMTORRENTTRACKERTORRENTLISTCONTAINER_H
#define SYMTORRENTTRACKERTORRENTLISTCONTAINER_H

// INCLUDES
#include <coecntrl.h>
#include <aknlists.h> 

#include  "STTBaseData.h"
#include  "SymTorrentTrackerTorrentListView.h"
 
// FORWARD DECLARATIONS

class CSymTorrentTrackerTorrentListView;

// CLASS DECLARATION

enum TTorrentStates 
{
	ETorOffline = 1,
	ETorNoPeer,
	ETorFewPeer,
	ETorManyPeer
};

/**
*  CSymTorrentTrackerTorrentListContainer  container control class.
*  
*/
class CSymTorrentTrackerTorrentListContainer : public CCoeControl, MCoeControlObserver
    {
    public: // Constructors and destructor
        
		CSymTorrentTrackerTorrentListContainer(CSTTBaseData* aBaseData, CSymTorrentTrackerTorrentListView* aView);

        /**
        * EPOC default constructor.
        * @param aRect Frame rectangle for container.
        */
        void ConstructL(const TRect& aRect);

        /**
        * Destructor.
        */
        ~CSymTorrentTrackerTorrentListContainer();

    public: // New functions
       /**
        * Returns the TorrentListBox
        */
		CAknSingleGraphicStyleListBox* TorrentListBox();

		// Insert a torrent to the list
		void InsertTorrentL(const TDesC& aName, TTorrentStates aTorrentState);

		// Remove a torrent from the list
		void RemoveTorrentL(TInt aIndex);

		// Clears the torrentList
		void ClearTorrentList();

		// Refreshes the torrentList
		void RefreshTorrentList();

		// Refreshes one item of the torrentList
		void RefreshTorrentListItem(CSTTTorrentListItem* aTorrentItem);

		// Refreshes one item of the torrentList
		void RefreshTorrentListItem(TInt aIndex);

    private: // New functions
		TInt IconIndexFromTorrentState(TTorrentStates aTorrentState);

		TTorrentStates TorrentItemState(CSTTTorrentListItem* aTorrentItem);

    public: // Functions from base classes

    private: // Functions from base classes

       /**
        * From CoeControl,SizeChanged.
        */
        void SizeChanged();

       /**
        * From CoeControl,CountComponentControls.
        */
        TInt CountComponentControls() const;

       /**
        * From CCoeControl,ComponentControl.
        */
        CCoeControl* ComponentControl(TInt aIndex) const;

       /**
        * From CCoeControl,Draw.
        */
        void Draw(const TRect& aRect) const;

		/**
		* From MCoeControlObserver
		* Acts upon changes in the hosted control's state. 
		*
		* @param	aControl	The control changing its state
		* @param	aEventType	The type of control event 
		*/
        void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);

		/**
		* Key events
		*/
		TKeyResponse OfferKeyEventL(const TKeyEvent &aKeyEvent,TEventCode aType);

    private: //data
        CAknSingleGraphicStyleListBox*			iTorrentListBox; // torrent listbox 
		CSTTBaseData*							iBaseData;
		CSymTorrentTrackerTorrentListView*		iView;
	};

#endif
