/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

#ifndef SYMTORRENTTRACKERTORRENTLISTVIEW_H
#define SYMTORRENTTRACKERTORRENTLISTVIEW_H

// INCLUDES
#include <aknview.h>
#include "STTTorrentItemObserver.h"
#include "STTConnector.h"


// CONSTANTS

// FORWARD DECLARATIONS
class CSymTorrentTrackerTorrentListContainer;
class CSymTorrentTrackerManager;

// CLASS DECLARATION

/**
*  CSymTorrentTrackerTorrentListView view class.
* 
*/
class CSymTorrentTrackerTorrentListView : public CAknView, public MSTTTorrentItemObserver
    {
    public: // Constructors and destructor
    
    	CSymTorrentTrackerTorrentListView(CSymTorrentTrackerManager* aTrackerManager);

        /**
        * EPOC default constructor.
        */
        void ConstructL();

        /**
        * Destructor.
        */
        ~CSymTorrentTrackerTorrentListView();

    public: // Functions from base classes
                        
		/**
        * From CAknView returns Uid of View
        * @return TUid uid of the view
        */
        TUid Id() const;

        /**
        * From MEikMenuObserver delegate commands from the menu
        * @param aCommand a command emitted by the menu 
        * @return void
        */
        void HandleCommandL(TInt aCommand);

        /**
        * From CAknView reaction if size change
        * @return void
        */
        void HandleClientRectChange();
	
		// Calls this function whenever new peer was added or removed
		void TorrentPeersChanged(CSTTTorrentListItem* aTorrentItem); 
		
		/**
		 * Updates the status label in the navigation pane according to the
		 * tracker's status.
		 */
		void UpdateTrackerStatusNaviPaneL();
		
	public: // New functions
		// Refreshs the torrentList
		void RefreshTorrentList();

		TConnectorSimpleStates GetConnectorState();

    private:
        /**
        * From CAknView activate the view
        * @param aPrevViewId 
        * @param aCustomMessageId 
        * @param aCustomMessage 
        * @return void
        */
        void DoActivateL(const TVwsViewId& aPrevViewId,TUid aCustomMessageId,
            const TDesC8& aCustomMessage);
        /**
        * From CAknView deactivate the view (free resources)
        * @return void
        */

        void DoDeactivate();

        // From CAknView MenuObserver. Dynamic sets which menu is visible.
        void DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane);

    private: // Data
        CSymTorrentTrackerTorrentListContainer* iContainer;
		TInt									iLastItemIndex;
		
		CSymTorrentTrackerManager* 				iTrackerManager;
    };

#endif


