
/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

// INCLUDE FILES
#include "STTBaseData.h"
#include "STTTorrentListItem.h"

// Constructors and destructor ------------------------------------------------
CSTTBaseData::CSTTBaseData():iTorrentObserver(NULL)
{
}

void CSTTBaseData::ConstructL()
{
	iPartnerPort	= -1;

	iItemListHead = CSTTTorrentListItem::NewL(_L(""),_L8(""),NULL);
}

CSTTBaseData::~CSTTBaseData()
{
	DeleteAllListItem();
	delete iItemListHead;
}

CSTTBaseData* CSTTBaseData::NewL()
{
	CSTTBaseData * self=new(ELeave) CSTTBaseData();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}

// New functions --------------------------------------------------------------

CSTTTorrentListItem* CSTTBaseData::InsertListItemL(TPtrC aTorrentFileName, TPtrC8 aInfoHash)
{
	TInt compare=-1;
	CSTTTorrentListItem* lastitem = iItemListHead;
	CSTTTorrentListItem* tempitem = iItemListHead->GetNextSibling();

	while (tempitem!=NULL && (compare=tempitem->GetTorrentFileName().Compare(aTorrentFileName))<0)
	{
		lastitem = tempitem;
		tempitem = tempitem->GetNextSibling();
	}
	if (compare>0 || tempitem==NULL)
		return lastitem->InsertAfterL(aTorrentFileName,aInfoHash,iTorrentObserver);
	else
		return NULL;
}

TBool CSTTBaseData::DeleteListItem(TPtrC aTorrentFileName)
{
	TBool res = EFalse;
	CSTTTorrentListItem* lastitem = iItemListHead;
	CSTTTorrentListItem* tempitem = iItemListHead->GetNextSibling();
	while (tempitem!=NULL && !res)
	{
		if (tempitem->GetTorrentFileName().Compare(aTorrentFileName)==0)
		{
			lastitem->DeleteNext();
			res = ETrue;
		}
		else
		{
			lastitem = tempitem;
			tempitem = tempitem->GetNextSibling();
		}
	}
	return res;
}

void CSTTBaseData::DeleteAllListItem()
{
	while (iItemListHead->GetNextSibling()!=NULL)
		iItemListHead->DeleteNext();
}

CSTTTorrentListItem* CSTTBaseData::GetTorrentListFirstItem()
{
	return iItemListHead->GetNextSibling();
}

// Getter and setter methods --------------------------------------------------

void CSTTBaseData::SetTorrentObserver(MSTTTorrentItemObserver* aTorrentObserver)
{
	iTorrentObserver = aTorrentObserver;
}

TInt CSTTBaseData::GetPartnerPort()
{
	return iPartnerPort;
}

void CSTTBaseData::SetPartnerPort(TInt aPartnerPort)
{
	iPartnerPort = aPartnerPort;
}
