/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

// INCLUDE FILES

#include  "STTConnectionSocketCollector.h"

// Constructors and destructor ------------------------------------------------
CSTTConnectionSocketCollector::CSTTConnectionSocketCollector()
{
}

void CSTTConnectionSocketCollector::ConstructL()
{
}

CSTTConnectionSocketCollector::~CSTTConnectionSocketCollector()
{
	ReleaseAllSocket();
}

CSTTConnectionSocketCollector* CSTTConnectionSocketCollector::NewL()
{
	CSTTConnectionSocketCollector* self=new(ELeave) CSTTConnectionSocketCollector();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}

// Communication methods ------------------------------------------------------

void CSTTConnectionSocketCollector::Initialize(CSymTorrentTrackerManager* aTrackerManager)
{
	iTrackerManager = aTrackerManager;
}

void CSTTConnectionSocketCollector::Accept(RSocket* aInitializedSocket, TInetAddr &aPartnerAddress)
{
	CheckAndAddNewSocket(aInitializedSocket, aPartnerAddress);
}

void CSTTConnectionSocketCollector::CheckAndAddNewSocket(RSocket* aInitializedSocket, TInetAddr &aPartnerAddress)
{
	CSTTServiceSocket* tempSocket;
	CSTTServiceSocket* lastSocket = NULL;

	tempSocket = iFirstSocket;
    while (tempSocket!=NULL) 
	{
		if (tempSocket->IsFinished())
		{
			if (lastSocket==NULL)
			{
				iFirstSocket = tempSocket->GetNextSibling();
				delete tempSocket;
				tempSocket = iFirstSocket;
				lastSocket = NULL;
			}
			else
			{
				lastSocket->SetNextSibling(tempSocket->GetNextSibling());
				delete tempSocket;
				tempSocket = lastSocket->GetNextSibling();
			}
		}
		else
		{
			lastSocket = tempSocket;
			tempSocket = tempSocket->GetNextSibling();
		}
	}

	tempSocket = CSTTServiceSocket::NewL(*aInitializedSocket, aPartnerAddress, iTrackerManager);
	tempSocket->SetNextSibling(iFirstSocket);
	iFirstSocket = tempSocket;
}

void CSTTConnectionSocketCollector::ReleaseAllSocket()
{
	CSTTServiceSocket* tempSocket;

    while (iFirstSocket!=NULL) 
	{
		tempSocket = iFirstSocket;
		iFirstSocket = iFirstSocket->GetNextSibling();
		delete tempSocket;
	}
}
