/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

#include "STTTimeOutHandler.h"

CTimeOutHandler* CTimeOutHandler::NewL()
{
	CTimeOutHandler * self=new(ELeave) CTimeOutHandler();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}

CTimeOutHandler::CTimeOutHandler() 
	: CActive(EPriorityStandard), 
		iState(ETimerNotSet)
{}

void CTimeOutHandler::ConstructL()
{
	CActiveScheduler::Add(this);
	User::LeaveIfError(iTimer.CreateLocal());
}

CTimeOutHandler::~CTimeOutHandler()
{
	Cancel(); // Cancel any request, if outstanding
	iTimer.Close();
}

void CTimeOutHandler::DoCancel()
{
	iTimer.Cancel();
}

void CTimeOutHandler::Start(TTimeIntervalMicroSeconds32 aTimeInterval)
{
	iTimer.Cancel();
	iTimer.After(iStatus,aTimeInterval);
	iState = ETimerWaiting;
	SetActive();
}

void CTimeOutHandler::Stop()
{
	Cancel();
}

void CTimeOutHandler::RunL()
{
	iObserver->TimeOutExpired();
	iState = ETimerExpired;
}
