/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

// INCLUDE FILES
#include "STTTorrentListItem.h"
#include "STTTorrentPeerItem.h"

// Constructors and destructor ------------------------------------------------
CSTTTorrentListItem::CSTTTorrentListItem():iTorrentObserver(NULL)
{
}

void CSTTTorrentListItem::ConstructL(TPtrC aTorrentFileName, TPtrC8 aInfoHash, MSTTTorrentItemObserver* aTorrentObserver)
{
	iTorrentObserver = aTorrentObserver;
	iTorrentFileName = aTorrentFileName.AllocL();
	iInfoHash		 = aInfoHash.AllocL();
	iNextSibling = NULL;

	TBuf8<10> temp;
	iPeerListHead = CSTTTorrentPeerItem::NewL(TInetAddr(),temp);
}

CSTTTorrentListItem::~CSTTTorrentListItem()
{
	delete iTorrentFileName;
	delete iInfoHash;

	DeleteAllPeerItem();
	delete iPeerListHead;

}

CSTTTorrentListItem* CSTTTorrentListItem::NewL(TPtrC aTorrentFileName, TPtrC8 aInfoHash, MSTTTorrentItemObserver* aTorrentObserver)
{
	CSTTTorrentListItem * self=new(ELeave) CSTTTorrentListItem();
	CleanupStack::PushL(self);
	self->ConstructL(aTorrentFileName, aInfoHash, aTorrentObserver);
	CleanupStack::Pop();
	return self;
}

// Peer list handling
CSTTTorrentPeerItem* CSTTTorrentListItem::InsertPeerItemL(TInetAddr aAddress, TDesC8& aId)
{
	CSTTTorrentPeerItem* peerTemp = iPeerListHead->InsertAfterL(aAddress, aId);
	if (iTorrentObserver) iTorrentObserver->TorrentPeersChanged(this);
	return peerTemp;
}

TBool CSTTTorrentListItem::DeletePeerItem(TInetAddr aAddress, TDesC8& aId)
{
	TBool res = EFalse;
	CSTTTorrentPeerItem* lastitem = iPeerListHead;
	CSTTTorrentPeerItem* tempitem = iPeerListHead->GetNextSibling();
	while (tempitem!=NULL && !res)
	{
		if (tempitem->PeerMatch(aAddress, aId))
		{
			lastitem->DeleteNext();
			res = ETrue;
		}
		else
		{
			lastitem = tempitem;
			tempitem = tempitem->GetNextSibling();
		}
	}
	if (iTorrentObserver) iTorrentObserver->TorrentPeersChanged(this);
	return res;
}

void CSTTTorrentListItem::DeleteSimilarPeers(TInetAddr aAddress, TDesC8& aId)
{
	CSTTTorrentPeerItem* lastitem = iPeerListHead;
	CSTTTorrentPeerItem* tempitem = iPeerListHead->GetNextSibling();
	while (tempitem!=NULL)
	{
		if ((tempitem->PeerId())->Compare(aId) || tempitem->Address()==aAddress.Address())
		{
			lastitem->DeleteNext();
			tempitem = lastitem->GetNextSibling();
		}
		else
		{
			lastitem = tempitem;
			tempitem = tempitem->GetNextSibling();
		}
	}
	if (iTorrentObserver) iTorrentObserver->TorrentPeersChanged(this);
}

void CSTTTorrentListItem::DeleteAllPeerItem()
{
	while (iPeerListHead->GetNextSibling()!=NULL)
		iPeerListHead->DeleteNext();
	if (iTorrentObserver) iTorrentObserver->TorrentPeersChanged(this);
}

CSTTTorrentPeerItem* CSTTTorrentListItem::GetPeerListFirstItem()
{
	return iPeerListHead->GetNextSibling();
}

CSTTTorrentPeerItem* CSTTTorrentListItem::FindPeerItem(TInetAddr aAddress, TDesC8& aId)
{	
	TBool found = EFalse;
	CSTTTorrentPeerItem* tempitem = iPeerListHead->GetNextSibling();
	while (tempitem!=NULL && !found)
	{
		if (tempitem->PeerMatch(aAddress, aId))
			found = ETrue;
		tempitem = tempitem->GetNextSibling();
	}
	if (found)
		return tempitem;
	else
		return NULL;
}

CSTTTorrentPeerItem* CSTTTorrentListItem::FindPeerItem(TInetAddr aAddress)
{	
	TBool found = EFalse;
	CSTTTorrentPeerItem* tempitem = iPeerListHead->GetNextSibling();
	while (tempitem!=NULL && !found)
	{
		if (tempitem->PeerMatch(aAddress))
			found = ETrue;
		tempitem = tempitem->GetNextSibling();
	}
	if (found)
		return tempitem;
	else
		return NULL;
}

void CSTTTorrentListItem::RemoveNeedlessPeers()
{
	CSTTTorrentPeerItem* lastitem = iPeerListHead;
	CSTTTorrentPeerItem* tempitem = iPeerListHead->GetNextSibling();
	while (tempitem!=NULL)
	{
		if (tempitem->SecondsFromLastCheckInTime()>10*60)
		{
			lastitem->DeleteNext();
			tempitem = lastitem->GetNextSibling();
		}
		else
		{
			lastitem = tempitem;
			tempitem = tempitem->GetNextSibling();
		}
	}
	if (iTorrentObserver) iTorrentObserver->TorrentPeersChanged(this);
}

// Getter and setter functions ------------------------------------------------

TPtr CSTTTorrentListItem::GetTorrentFileName()
{
	return iTorrentFileName->Des();
}

void CSTTTorrentListItem::SetTorrentFileNameL(TPtrC aTorrentFileName)
{
	delete iTorrentFileName;
	iTorrentFileName = aTorrentFileName.AllocL();
}

TPtr8 CSTTTorrentListItem::GetInfoHash()
{
	return iInfoHash->Des();
}

void CSTTTorrentListItem::SetInfoHashL(TPtrC8 aInfoHash)
{
	delete iInfoHash;
	iInfoHash = aInfoHash.AllocL();
}

CSTTTorrentListItem* CSTTTorrentListItem::GetNextSibling()
{
	return iNextSibling;
}

void CSTTTorrentListItem::SetNextSibling(CSTTTorrentListItem* aNextSibling)
{
	iNextSibling = aNextSibling;
}

// New functions --------------------------------------------------------------

CSTTTorrentListItem* CSTTTorrentListItem::InsertAfterL(TPtrC aTorrentFileName, TPtrC8 aInfoHash, MSTTTorrentItemObserver* aTorrentObserver)
{
	CSTTTorrentListItem* tempitem = CSTTTorrentListItem::NewL(aTorrentFileName,aInfoHash, aTorrentObserver);
	tempitem->SetNextSibling(iNextSibling);
	iNextSibling = tempitem;
	return tempitem;
}

void CSTTTorrentListItem::DeleteNext()
{
	if (iNextSibling!=NULL)
	{
		CSTTTorrentListItem* tempitem = iNextSibling;
		iNextSibling = tempitem->GetNextSibling();
		delete tempitem;
	}
}

TInt CSTTTorrentListItem::PeerListLength()
{
	TInt num = 0;

	CSTTTorrentPeerItem* tempitem = GetPeerListFirstItem();
	while (tempitem!=NULL)
	{
		tempitem = tempitem->GetNextSibling();
		num++;
	}

	return num;
}
