/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

// INCLUDE FILES
#include "STTTorrentPeerItem.h"

// Constructors and destructor ------------------------------------------------
CSTTTorrentPeerItem::CSTTTorrentPeerItem()
{
}

void CSTTTorrentPeerItem::ConstructL(TInetAddr aAddress, TDesC8& aId)
{
	iNextSibling	= NULL;
	iAddress		= aAddress;
	iId				= aId.AllocL();
	iCompleted		= EFalse;
	iLastCheckInTime.HomeTime();
	iIp				= KNullDesC8().AllocL();
}

CSTTTorrentPeerItem::~CSTTTorrentPeerItem()
{
	delete iId;
	if (iIp)
	{
		delete iIp;
		iIp = NULL;
	}
}

CSTTTorrentPeerItem* CSTTTorrentPeerItem::NewL(TInetAddr aAddress, TDesC8& aId)
{
	CSTTTorrentPeerItem * self=new(ELeave) CSTTTorrentPeerItem();
	CleanupStack::PushL(self);
	self->ConstructL(aAddress, aId);
	CleanupStack::Pop();
	return self;
}

// Getter and setter functions ------------------------------------------------


TInt CSTTTorrentPeerItem::SecondsFromLastCheckInTime()
{
	TTimeIntervalSeconds timeinterval;
	TTime now;
	now.HomeTime();
	if (now.SecondsFrom(iLastCheckInTime, timeinterval) == KErrNone)
	{
		return timeinterval.Int();
	}
	else
		return -1;
}

HBufC8* CSTTTorrentPeerItem::PeerIp()
{
	if (iIp)
	{
		delete iIp;
		iIp = NULL;
	}
	TBuf<60> tempip;
	TBuf8<60> tempip8;
	iAddress.Output(tempip);
	tempip8.Copy(tempip);
	iIp = tempip8.AllocL();
	return iIp;
}


// New functions --------------------------------------------------------------

CSTTTorrentPeerItem* CSTTTorrentPeerItem::InsertAfterL(TInetAddr aAddress, TDesC8& aId)
{
	CSTTTorrentPeerItem* tempitem = CSTTTorrentPeerItem::NewL(aAddress, aId);
	tempitem->SetNextSibling(iNextSibling);
	this->SetNextSibling(tempitem);
	return tempitem;
}

void CSTTTorrentPeerItem::DeleteNext()
{
	if (iNextSibling!=NULL)
	{
		CSTTTorrentPeerItem* tempitem = iNextSibling;
		iNextSibling = tempitem->GetNextSibling();
		delete tempitem;
	}
}
