/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

// INCLUDE FILES
#include <eiklabel.h>
#include <eikenv.h>
#include <aknutils.h>
#include <txtetext.h>
#include <txtrich.h>
#include <txtfrmat.h>
#include <AknsDrawUtils.h>// skin
#include <AknsBasicBackgroundControlContext.h> //skin 

#include "SymtorrentmakerDetailsContainer.h"


// ================= MEMBER FUNCTIONS =======================

// ---------------------------------------------------------
// CSymTorrentMakerDetailsContainer::ConstructL(const TRect& aRect)
// EPOC two phased constructor
// ---------------------------------------------------------
//
void CSymTorrentMakerDetailsContainer::ConstructL(const TRect& aRect, CSymTorrentTrackerManager* aTrackerManager)
{
	iTrackerManager = aTrackerManager;
    CreateWindowL();

    iTextWindow = 
		new (ELeave) CEikRichTextEditor(TGulBorder(TGulBorder::ENone));

	iTextWindow->ConstructL(this, 0, 0,
		//EEikEdwinOwnsWindow|
		//EEikEdwinInclusiveSizeFixed|
		//EEikEdwinKeepDocument|
		//EEikEdwinUserSuppliedText|
		EEikEdwinNoAutoSelection|
		EEikEdwinDisplayOnly|
		EEikEdwinReadOnly|
		EEikEdwinAvkonDisableCursor|
		EEikEdwinNoHorizScrolling|
		EAknEditorNullInputMode);

	iTextWindow->CreateScrollBarFrameL()->
		SetScrollBarVisibilityL(CEikScrollBarFrame::EOff,CEikScrollBarFrame::EAuto);
		
	InitializeText();

	iTextWindow->SetFocus(ETrue);

    SetRect(aRect);
	iBackGround = CAknsBasicBackgroundControlContext::NewL( KAknsIIDQsnBgAreaMain, Rect(), EFalse );
    ActivateL();
}

// Destructor
CSymTorrentMakerDetailsContainer::~CSymTorrentMakerDetailsContainer()
{
    delete iTextWindow;
	delete iBackGround;
}


void CSymTorrentMakerDetailsContainer::InitializeText()
{
	TFileDetails details = iTrackerManager->GetFileDetails();

//	iTextWindow->RichText()->AppendParagraphL();
	SetTextStyle(ESmallTitleText);
	AppendText(_L("Original path:"));
//	AppendText(CEditableText::ELineBreak);

	iTextWindow->RichText()->AppendParagraphL();
	SetTextStyle(ENormalTextWithLeftMargin);
	AppendText(details.fullPath);
//	AppendText(CEditableText::ELineBreak);

	iTextWindow->RichText()->AppendParagraphL();
	SetTextStyle(ESmallTitleText);
	AppendText(_L("File size:"));
//	AppendText(CEditableText::ELineBreak);

	iTextWindow->RichText()->AppendParagraphL();
	SetTextStyle(ENormalTextWithLeftMargin);
	TBuf<80> filesize;
	filesize.Format(_L("%d Kb"),(TInt)details.filesize/1024);
	AppendText(filesize);
//	AppendText(CEditableText::ELineBreak);
}

void CSymTorrentMakerDetailsContainer::SetTextStyle(TTextStyles aTextStyle)
{
	CCharFormatLayer* formatLayer = CEikonEnv::NewDefaultCharFormatLayerL();
	CleanupStack::PushL(formatLayer);
	TCharFormat		charFormat; 
	TCharFormatMask charFormatMask; 
	CParaFormat*	paraFormat = CParaFormat::NewLC();
	TParaFormatMask	paraFormatMask;
	formatLayer->Sense(charFormat, charFormatMask);

	charFormatMask.ClearAll();
	paraFormatMask.ClearAll();

	switch (aTextStyle)
	{
		case ENormalTextWithLeftMargin:
			// Color
			charFormat.iFontPresentation.iTextColor = KRgbDarkGray;
			charFormatMask.SetAttrib(EAttColor);
			
			// Font
			charFormat.iFontSpec.iTypeface = LatinPlain12()->FontSpecInTwips().iTypeface;
			charFormatMask.SetAttrib(EAttFontTypeface);


			// Paragraph Format

			break;
		case ESmallTitleText:
			// Color
			charFormat.iFontPresentation.iTextColor = KRgbBlack;
			charFormatMask.SetAttrib(EAttColor);
			
			// Font
			charFormatMask.SetAttrib(EAttFontStrokeWeight);

			// Paragraph Format

			break;
	}


	iTextWindow->RichText()->ApplyCharFormatL(charFormat, charFormatMask, iTextWindow->RichText()->DocumentLength(), 0);
	iTextWindow->RichText()->ApplyParaFormatL(paraFormat, paraFormatMask, iTextWindow->RichText()->DocumentLength(), 0);

	CleanupStack::PopAndDestroy(); 
	CleanupStack::PopAndDestroy(); // paraFormat
}


void CSymTorrentMakerDetailsContainer::AppendText(const TDesC& aBuf)
{
	iTextWindow->RichText()->InsertL(iTextWindow->RichText()->DocumentLength(), aBuf);
}

void CSymTorrentMakerDetailsContainer::AppendText(const TChar& aChar)
{
	iTextWindow->RichText()->InsertL(iTextWindow->RichText()->DocumentLength(), aChar);
}


// ---------------------------------------------------------
// CSymTorrentMakerDetailsContainer::SizeChanged()
// Called by framework when the view size is changed
// ---------------------------------------------------------
//
void CSymTorrentMakerDetailsContainer::SizeChanged()
{
	TRect outputRect(5, 5, 
		Rect().Width()-10, Rect().Height()-5);
    iTextWindow->SetRect(outputRect);
}

// ---------------------------------------------------------
// CSymTorrentMakerDetailsContainer::CountComponentControls() const
// ---------------------------------------------------------
//
TInt CSymTorrentMakerDetailsContainer::CountComponentControls() const
{
    return 1; 
}

// ---------------------------------------------------------
// CSymTorrentMakerDetailsContainer::ComponentControl(TInt aIndex) const
// ---------------------------------------------------------
//
CCoeControl* CSymTorrentMakerDetailsContainer::ComponentControl(TInt aIndex) const
{
    switch ( aIndex )
	{
        case 0:
            return iTextWindow;
        default:
            return NULL;
	}
}

// ---------------------------------------------------------
// CSymTorrentMakerDetailsContainer::Draw(const TRect& aRect) const
// ---------------------------------------------------------
//
void CSymTorrentMakerDetailsContainer::Draw(const TRect& aRect) const
{
    CWindowGc& gc = SystemGc();

	// draw background
	MAknsSkinInstance* skin = AknsUtils::SkinInstance();
	MAknsControlContext* cc = AknsDrawUtils::ControlContext( this );
    AknsDrawUtils::Background( skin, cc, this, gc, aRect );
}

// ---------------------------------------------------------
// CSymTorrentMakerDetailsContainer::MopSupplyObject()
// Pass skin information if needed.
// ---------------------------------------------------------
//
TTypeUid::Ptr CSymTorrentMakerDetailsContainer::MopSupplyObject(TTypeUid aId)
{
    if(aId.iUid == MAknsControlContext::ETypeId && iBackGround)
        {
        return MAknsControlContext::SupplyMopObject( aId, iBackGround);
        }

    return CCoeControl::MopSupplyObject( aId );
}

// ---------------------------------------------------------
// CSymTorrentMakerDetailsContainer::HandleControlEventL(
//     CCoeControl* aControl,TCoeEvent aEventType)
// ---------------------------------------------------------
//
void CSymTorrentMakerDetailsContainer::HandleControlEventL(
    CCoeControl* /*aControl*/,TCoeEvent /*aEventType*/)
{
}


// ---------------------------------------------------------
// CSymTorrentMakerDetailsContainer::OfferKeyEventL(const TKeyEvent &aKeyEvent,
//	 TEventCode aType)
// ---------------------------------------------------------
//
TKeyResponse CSymTorrentMakerDetailsContainer::OfferKeyEventL(const TKeyEvent &aKeyEvent,TEventCode aType)
{
	if (aType == EEventKey) 
	{
		switch (aKeyEvent.iCode) 
		{
			case EKeyUpArrow:
			{				
				iTextWindow->MoveCursorL(TCursorPosition::EFPageUp, EFalse);
				return EKeyWasConsumed;
			}

			case EKeyDownArrow:
			{				
				iTextWindow->MoveCursorL(TCursorPosition::EFPageDown, EFalse);
				return EKeyWasConsumed;
			}
			case EKeyNo:
				return EKeyWasNotConsumed;	
			default:;
		}
	}
	return EKeyWasNotConsumed;
}

// End of File  
