/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

// INCLUDE FILES
#include <aknviewappui.h>
#include <akntitle.h>
#include <avkon.hrh>
#include <SymTracker.rsg>
#include "SymtorrentmakerDetailsView.h"
#include "SymtorrentmakerDetailsContainer.h"
#include "SymTorrentTracker.hrh"

// ================= MEMBER FUNCTIONS =======================

// ---------------------------------------------------------
// CSymTorrentMakerDetailsView::ConstructL
// EPOC two-phased constructor
// ---------------------------------------------------------
//
void CSymTorrentMakerDetailsView::ConstructL(CSymTorrentTrackerManager* aTrackerManager)
{
	iTrackerManager = aTrackerManager;
    BaseConstructL( R_SYMTORRENTMAKER_VIEWDETAILS );
}

// ---------------------------------------------------------
// CSymTorrentMakerDetailsView::~CSymTorrentMakerDetailsView()
// destructor
// ---------------------------------------------------------
//
CSymTorrentMakerDetailsView::~CSymTorrentMakerDetailsView()
{
    if ( iContainer )
	{
        AppUi()->RemoveFromViewStack( *this, iContainer );
	}

    delete iContainer;
}

// ---------------------------------------------------------
// TUid CSymTorrentMakerDetailsView::Id()
// 
// ---------------------------------------------------------
//
TUid CSymTorrentMakerDetailsView::Id() const
{
    return TUid::Uid(ESymTorrentMakerDetailsView);
}

// ---------------------------------------------------------
// CSymTorrentMakerDetailsView::HandleCommandL(TInt aCommand)
// takes care of view command handling
// ---------------------------------------------------------
//
void CSymTorrentMakerDetailsView::HandleCommandL(TInt aCommand)
{   
    switch ( aCommand )
	{
        case EAknSoftkeyBack:
			AppUi()->ActivateLocalViewL(iPrevViewId.iViewUid);
            break;
        default:
            AppUi()->HandleCommandL( aCommand );
            break;
	}
}

// ---------------------------------------------------------
// CSymTorrentMakerDetailsView::HandleClientRectChange()
// ---------------------------------------------------------
//
void CSymTorrentMakerDetailsView::HandleClientRectChange()
{
    if ( iContainer )
	{
        iContainer->SetRect( ClientRect() );
	}
}

// ---------------------------------------------------------
// CSymTorrentMakerDetailsView::DoActivateL(...)
// 
// ---------------------------------------------------------
//
void CSymTorrentMakerDetailsView::DoActivateL(
   const TVwsViewId& aPrevViewId,TUid /*aCustomMessageId*/,
   const TDesC8& /*aCustomMessage*/)
{
    if (!iContainer)
    {
		iPrevViewId = aPrevViewId;

        iContainer = new (ELeave) CSymTorrentMakerDetailsContainer;
        iContainer->SetMopParent(this);
        iContainer->ConstructL( ClientRect(), iTrackerManager ); 
        AppUi()->AddToStackL( *this, iContainer );

		CEikStatusPane* statusPane = StatusPane();
		TUid paneUid;
		paneUid.iUid = EEikStatusPaneUidTitle;
		if (statusPane->PaneCapabilities(paneUid).IsAppOwned())
		{
			CAknTitlePane* titlePane = (CAknTitlePane*) statusPane->ControlL(paneUid);
			titlePane->SetTextL(_L("Details"));
		}
    
	}
}

// ---------------------------------------------------------
// CSymTorrentMakerDetailsView::DoDeactivate()
// 
// ---------------------------------------------------------
//
void CSymTorrentMakerDetailsView::DoDeactivate()
{
    if ( iContainer )
	{
        AppUi()->RemoveFromViewStack( *this, iContainer );
	}
    
    delete iContainer;
    iContainer = NULL;
}

// End of File

