/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

// INCLUDE FILES
#include  "SymTorrentMakerFileListContainer.h"
#include <badesca.h>
#include <CAknFileSelectionDialog.h>
#include <e32std.h>
#include <e32base.h>
#include <eikclb.h>
#include "SymTorrentTracker.hrh"
#include <eikclbd.h>

#include <akniconutils.h> 
#include <aknlists.h> 
#include <AknsUtils.h>
#include <akniconarray.h>
#include <aknnotewrappers.h> 

#include <fbs.h>

#include <SymTracker.mbg>

_LIT(KBitmapFile, "\\resource\\apps\\SymTracker.mif");


// ================= MEMBER FUNCTIONS =======================

// ---------------------------------------------------------
// CSymTorrentMakerContainer::ConstructL(const TRect& aRect)
// EPOC two phased constructor
// ---------------------------------------------------------
//
void CSymTorrentMakerFileListContainer::ConstructL(const TRect& aRect, CSymTorrentMakerFileListView *aMyView)
{
    CreateWindowL();
	iMyView = aMyView;

	iListBox = new (ELeave) CAknSingleGraphicStyleListBox;
	iListBox->ConstructL(this, EAknListBoxSelectionList);
	iListBox->SetContainerWindowL(*this);

	iListBox->CreateScrollBarFrameL(ETrue);
	iListBox->ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto);

	
    CArrayPtr<CGulIcon>* icons = new (ELeave) CAknIconArray(3);
	// array of the icons
	CleanupStack::PushL(icons);

		CGulIcon* folderIcon = CGulIcon::NewL();
		CleanupStack::PushL(folderIcon);
		CGulIcon* fileIcon = CGulIcon::NewL();
		CleanupStack::PushL(fileIcon);
		CGulIcon* rootIcon = CGulIcon::NewL();
		CleanupStack::PushL(rootIcon);

		CFbsBitmap* fileBmp = NULL;
		CFbsBitmap* fileMask = NULL;
		AknIconUtils::CreateIconLC(fileBmp, fileMask, KBitmapFile,
			EMbmSymtrackerFile, EMbmSymtrackerFile_mask);
		fileIcon->SetBitmap(fileBmp);
		fileIcon->SetMask(fileMask);
		CleanupStack::Pop(2);
		icons->AppendL(fileIcon);
		CleanupStack::Pop();

		CFbsBitmap* folderBmp = NULL;
		CFbsBitmap* folderMask = NULL;
		AknIconUtils::CreateIconLC(folderBmp, folderMask, KBitmapFile,
			EMbmSymtrackerFolder, EMbmSymtrackerFolder_mask);	
		folderIcon->SetBitmap(folderBmp);
		folderIcon->SetMask(folderMask);
		CleanupStack::Pop(2);
		icons->AppendL(folderIcon);
		CleanupStack::Pop();
	
		CFbsBitmap* rootBmp = NULL;
		CFbsBitmap* rootMask = NULL;
		AknIconUtils::CreateIconLC(rootBmp, rootMask, KBitmapFile,
			EMbmSymtrackerRoot, EMbmSymtrackerRoot_mask);		
		rootIcon->SetBitmap(rootBmp);
		rootIcon->SetMask(rootMask);
		CleanupStack::Pop(2);
		icons->AppendL(rootIcon);
		CleanupStack::Pop();
	/*
	icons->AppendL(iEikonEnv->CreateIconL(KBitmapIconsC,EFileIcon,	EFileIcon_Mask)); 
	icons->AppendL(iEikonEnv->CreateIconL(KBitmapIconsC,EFolderIcon,	EFolderIcon_Mask));
	icons->AppendL(iEikonEnv->CreateIconL(KBitmapIconsC,ERootIcon,	ERootIcon_Mask));
	*/

	CleanupStack::Pop(icons);

	iListBox->ItemDrawer()->ColumnData()->SetIconArray(icons);
    SetRect(aRect);
    ActivateL();
}

// Destructor
CSymTorrentMakerFileListContainer::~CSymTorrentMakerFileListContainer()
{
	delete iListBox;
}

// ---------------------------------------------------------
// CSymTorrentMakerContainer::SizeChanged()
// Called by framework when the view size is changed
// ---------------------------------------------------------
//
void CSymTorrentMakerFileListContainer::SizeChanged()
{
    iListBox->SetSize(Rect().Size());
}

// ---------------------------------------------------------
// CSymTorrentMakerContainer::CountComponentControls() const
// ---------------------------------------------------------
//
TInt CSymTorrentMakerFileListContainer::CountComponentControls() const
{
    return 1; // return nbr of controls inside this container
}

// ---------------------------------------------------------
// CSymTorrentMakerContainer::ComponentControl(TInt aIndex) const
// ---------------------------------------------------------
//
CCoeControl* CSymTorrentMakerFileListContainer::ComponentControl(TInt aIndex) const
{
    switch ( aIndex )
	{
        case 0:
            return iListBox;

        default:
            return NULL;
	}
}

// ---------------------------------------------------------
// CSymTorrentMakerContainer::Draw(const TRect& aRect) const
// ---------------------------------------------------------
//
void CSymTorrentMakerFileListContainer::Draw(const TRect& aRect) const
{
    CWindowGc& gc = SystemGc();
    gc.SetPenStyle( CGraphicsContext::ENullPen );
    gc.SetBrushColor( KRgbWhite );
    gc.SetBrushStyle( CGraphicsContext::ESolidBrush );
    gc.DrawRect( aRect );
}

// ---------------------------------------------------------
// CSymTorrentMakerContainer::HandleControlEventL(
//     CCoeControl* aControl,TCoeEvent aEventType)
// ---------------------------------------------------------
//
void CSymTorrentMakerFileListContainer::HandleControlEventL(
    CCoeControl* /*aControl*/,TCoeEvent /*aEventType*/)
{
}


// ---------------------------------------------------------
// CSymTorrentMakerFileListContainer::OfferKeyEventL(const TKeyEvent &aKeyEvent,TEventCode aType)
// takes care of key handling
// ---------------------------------------------------------
//
TKeyResponse CSymTorrentMakerFileListContainer::OfferKeyEventL(const TKeyEvent &aKeyEvent,TEventCode aType)
{
    switch ( aKeyEvent.iCode )
	{
		case 8:
			iMyView->RemoveNode(iListBox->CurrentItemIndex());
			iMyView->RefreshList();
			break;
//		case EKeyLeftArrow:
		case '4':
		case '6':
			return EKeyWasNotConsumed;
			break;
		case EKeyLeftArrow:
			iMyView->EnterFolderOrShowDetailNode(0); // local root 
			break;
		case EKeyRightArrow:
		case EKeyOK:
		case '5':
			iMyView->EnterFolderOrShowDetailNode(iListBox->CurrentItemIndex());
			break; 
        default:
			return iListBox->OfferKeyEventL(aKeyEvent,aType);
	}
	return EKeyWasConsumed;
}

/**
* Add a given node to the list
*/
void CSymTorrentMakerFileListContainer::AddListItem(CSTMPackageNode* aNode, TBool aRootItem)
{
	TPtrC name = TPtrC(aNode->GetNodeName());

	TBuf<200> listitem;
	TInt imageIndex=0;

	if (aRootItem==EFalse)
	{
		switch (aNode->GetNodeType())
		{	
			case EFolderNode:
				imageIndex=1;
				break;
			case ERootNode:
				imageIndex=2;
				break;
			default:
			case EFileNode:
				imageIndex=0;
				break;
		}
	}
	else
	{
		imageIndex=2;
	}

	listitem.Copy(_L(""));
	listitem.AppendNum(imageIndex);
	listitem.Append(_L("\t"));
	listitem.Append(name);

	CDesCArray* listarray = static_cast<CDesCArray*> (iListBox->Model()->ItemTextArray());
	listarray->AppendL(listitem);
	iListBox->HandleItemAdditionL();
}

/**
* Remove all elements from the list
*/
void CSymTorrentMakerFileListContainer::ClearList()
{
	CDesCArray* listarray = static_cast<CDesCArray*> (iListBox->Model()->ItemTextArray());
	listarray->Reset();
	iListBox->HandleItemRemovalL();

	AknListBoxUtils::HandleItemRemovalAndPositionHighlightL(iListBox, 0, ETrue);
}

/**
* Sets the cursor to the specified position
*/
void CSymTorrentMakerFileListContainer::SetCursorPosition(TInt aPosition)
{
	if (aPosition>=0)
	{
		iListBox->SetCurrentItemIndex(aPosition);
		iListBox->HandleItemAdditionL();
		AknListBoxUtils::HandleItemRemovalAndPositionHighlightL(iListBox, aPosition, ETrue);
	}
	else
	{
		iListBox->SetCurrentItemIndex(0);
		iListBox->HandleItemAdditionL();
		AknListBoxUtils::HandleItemRemovalAndPositionHighlightL(iListBox, 0, ETrue);
	}
}

/**
* Sets the cursor to the last position
*/
TInt CSymTorrentMakerFileListContainer::GetCursorPosition()
{
	return iListBox->CurrentItemIndex();
}


/**
* Gets the current item's type
*/
TInt CSymTorrentMakerFileListContainer::GetCurrentItemType()
{
	CDesCArray* listarray = static_cast<CDesCArray*> (iListBox->Model()->ItemTextArray());
	TPtrC16 currItemText = (*listarray)[iListBox->CurrentItemIndex()];

	TInt firstOcc;
	firstOcc = currItemText.Locate('\t');
	if (firstOcc == KErrNotFound) firstOcc=-1;

	if (firstOcc>=0)
	{
		TPtrC16 currItemType = currItemText.Left(firstOcc);
		TLex* num = new TLex(currItemType);
		TInt typenum;

		if (!(num->Val(typenum)==KErrNone))
		{
			typenum = -1;
		}

		delete num;

		return typenum;
	}
	else
	{
		return -1;
	}
}


// End of File  
