/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

// INCLUDE FILES
#include <aknviewappui.h>
#include <eikmenup.h>
#include <avkon.hrh>
#include <SymTracker.rsg>
#include <AknQueryDialog.h> 
#include <CAknFileSelectionDialog.h>
#include <CAknMemorySelectionDialog.h> 
#include <f32file.h>
#include <akntitle.h> 
#include <PathInfo.h> 
#include "SymTorrentMakerFileListView.h"
#include "SymTorrentMakerFileListContainer.h" 
//#include "SymTorrentAppui.h"
#include "SymTorrentTracker.hrh"
#include <aknnotewrappers.h>
#include  "SymTorrentTrackerManager.h"

// ================= MEMBER FUNCTIONS =======================

CSymTorrentMakerFileListView::CSymTorrentMakerFileListView(CSymTorrentTrackerManager* aTrackerManager)
 : iTrackerManager(aTrackerManager)
{	
}

// ---------------------------------------------------------
// CSymTorrentMakerView::ConstructL(const TRect& aRect)
// EPOC two-phased constructor
// ---------------------------------------------------------
//
void CSymTorrentMakerFileListView::ConstructL()
{
    BaseConstructL( R_SYMTORRENTMAKER_VIEWFILELIST );
	iAppUi			= (CSymTorrentAppUi *)AppUi();
	iTorrentHandler = iTrackerManager->GetTorrentHandler();
}

// ---------------------------------------------------------
// CSymTorrentMakerView::~CSymTorrentMakerView()
// destructor
// ---------------------------------------------------------
//
CSymTorrentMakerFileListView::~CSymTorrentMakerFileListView()
{
    if ( iContainer )
	{
        AppUi()->RemoveFromViewStack( *this, iContainer );
	}

    delete iContainer;
}

// ---------------------------------------------------------
// TUid CSymTorrentMakerView::Id()
//
// ---------------------------------------------------------
//
TUid CSymTorrentMakerFileListView::Id() const
{
    return TUid::Uid(ESymTorrentMakerFileListView);
}

// ---------------------------------------------------------
// CSymTorrentMakerView::HandleCommandL(TInt aCommand)
// takes care of view command handling
// ---------------------------------------------------------
//
void CSymTorrentMakerFileListView::HandleCommandL(TInt aCommand)
{   
    switch ( aCommand )
	{
		case EAknSoftkeyBack:
			iTrackerManager->MakerCmdBack();	
			break;
		//case ESymTorrentCmdSettings:
		//	iTrackerManager->ViewChanger()->ActivateViewL(ESymTorrentSettingsView);
		//	break;
		case ESymTorrentMakerCmdSaveTorrent:
			iTrackerManager->MakerCmdSave();			
			break;
		case ESymTorrentMakerCmdCancel:
			iTrackerManager->CancelMaker();
			break;
        case ESymTorrentMakerCmdEnterFolder:
			EnterFolderOrShowDetailNode(iContainer->GetCursorPosition());
            break;
        case ESymTorrentMakerCmdExitFolder:
			EnterFolderOrShowDetailNode(0);
            break;
        case ESymTorrentMakerCmdFileDetails:
			EnterFolderOrShowDetailNode(iContainer->GetCursorPosition());
            break;
        case ESymTorrentMakerCmdAddFile:
			AddFileToListL();
			RefreshList();
            break;
        case ESymTorrentMakerCmdAddFolder:
			AddNewFolderToListL();
			RefreshList();
            break;
        case ESymTorrentMakerCmdAddExistingFolder:
			AddFolderToListL();
			RefreshList();
            break;
		case ESymTorrentMakerCmdDeleteChildren:
        case ESymTorrentMakerCmdDelCurrent:
			RemoveNode(iContainer->GetCursorPosition());
			RefreshList();
            break;
        default:
            AppUi()->HandleCommandL( aCommand );
            break;
    }
}

// ---------------------------------------------------------
// CSymTorrentMakerView::HandleClientRectChange()
// ---------------------------------------------------------
//
void CSymTorrentMakerFileListView::HandleClientRectChange()
{
    if ( iContainer )
	{
        iContainer->SetRect( ClientRect() );
	}
}

// ---------------------------------------------------------
// CSymTorrentMakerView::DoActivateL(...)
// 
// ---------------------------------------------------------
//
void CSymTorrentMakerFileListView::DoActivateL(
   const TVwsViewId& /*aPrevViewId*/,TUid /*aCustomMessageId*/,
   const TDesC8& /*aCustomMessage*/)
{
	//CSymTorrentAppUi* appUi = static_cast<CSymTorrentAppUi*>(AppUi());
	iTrackerManager->UiConn()->OnTrackerViewChangedL(ESymTorrentMakerFileListView);
	//appUi->SetCurrentViewId(ESymTorrentMakerFileListView);
	
	iTrackerManager->SetInTracker(ETrue);
	
    if (!iContainer)
	{
        iContainer = new (ELeave) CSymTorrentMakerFileListContainer;
        iContainer->SetMopParent(this);
        iContainer->ConstructL( ClientRect(), this);
        AppUi()->AddToStackL( *this, iContainer );
	} 
	RefreshList();
}

// ---------------------------------------------------------
// CSymTorrentMakerView::DoDeactivate()
// 
// ---------------------------------------------------------
//
void CSymTorrentMakerFileListView::DoDeactivate()
{
    if ( iContainer )
	{
        AppUi()->RemoveFromViewStack( *this, iContainer );
	}
    
    delete iContainer;
    iContainer = NULL;
}


/**
* File select dialog with memory card selection
*//*
TBool CSymTorrentMakerFileListView::FileSelectQueryL(TFileName &aFileName, TCommonDialogType aDialogType)
{
	// Select memory
	CAknMemorySelectionDialog* memSelectionDialog = CAknMemorySelectionDialog::NewL
		(ECFDDialogTypeNormal, EFalse);
	CleanupStack::PushL(memSelectionDialog);
	CAknMemorySelectionDialog::TMemory mem(CAknMemorySelectionDialog::EPhoneMemory);

	TInt ret = memSelectionDialog->ExecuteL(mem);
	CleanupStack::PopAndDestroy(memSelectionDialog);
	if (!ret) return EFalse;
	
	// get root on phone memory or card
	TFileName pathname;
	if (mem == CAknMemorySelectionDialog::EMemoryCard)
		pathname.Append(PathInfo::MemoryCardRootPath());
	else
		pathname.Append(PathInfo::PhoneMemoryRootPath());

	aFileName.Append(pathname);

	// launch file select dialog
	CAknFileSelectionDialog* dialog = CAknFileSelectionDialog::NewL(aDialogType,R_FILE_SELECTION_DIALOG);
	dialog->SetDefaultFolderL(pathname);
	

	TBool ok = dialog->ExecuteL(aFileName);

	delete dialog;

	return ok;
}
*/

/**
* Adding a chosen file to nodes
*/
void CSymTorrentMakerFileListView::AddFileToListL()
{
	TBuf<256> chosenfile;
	TBuf<200> listitem;
	TInt lastocc;

	if (iTrackerManager->FileSelectQueryL(chosenfile,ECFDDialogTypeNormal,_L("Select file:")))
	{
		lastocc=chosenfile.LocateReverse('\\');
		if (lastocc==KErrNotFound) lastocc=-1;

		TPtrC16 shortname=chosenfile.Right(chosenfile.Length()-(lastocc+1));

		if (iTorrentHandler->GetCurrentNode()->AddChild(shortname,chosenfile,EFileNode)==NULL)
		{
			CAknWarningNote* note = new(ELeave) CAknWarningNote;

			TBuf<100> warningText;
			warningText.Copy(_L("File/folder already exists: "));
			warningText.Append(shortname);
			note->ExecuteLD(warningText);
		}
	}
}

/**
* Adding a new empty folder to nodes
*/
void CSymTorrentMakerFileListView::AddNewFolderToListL()
{
	TBuf<45> listitem;

	CAknTextQueryDialog* dlg = CAknTextQueryDialog::NewL(listitem);
	dlg->SetMaxLength(40);
	dlg->SetPromptL(_L("Enter folder name:")); 
	dlg->ExecuteLD(R_DATAINPUT);

	if (listitem.Length()>0)
	{
		if (iTorrentHandler->GetCurrentNode()->AddChild(listitem,listitem,EFolderNode)==NULL)
		{
			CAknWarningNote* note = new(ELeave) CAknWarningNote;

			TBuf<100> warningText;
			warningText.Copy(_L("File/folder already exists: "));
			warningText.Append(listitem);
			note->ExecuteLD(warningText);
		}
	}
}

/**
* Adding an existing folder to nodes
*/
void CSymTorrentMakerFileListView::AddFolderToListL()
{
	TBuf<256> chosenfile;

	if (iTrackerManager->FileSelectQueryL(chosenfile,ECFDDialogTypeBrowse,_L("Select folder:")))
	{
		AddSpecificFolderToListL(chosenfile, iTorrentHandler->GetCurrentNode());
	}
}

/**
* Adding an existing folder to nodes with full content
*/
void CSymTorrentMakerFileListView::AddSpecificFolderToListL(TPtrC aFolderPath, CSTMPackageNode* aParentNode)
{
	HBufC* chosenFolder = HBufC::NewL(200);
	TPtr ptrChosenFolder = chosenFolder->Des();
	TInt lastOcc;

	ptrChosenFolder.Copy(aFolderPath);

	ptrChosenFolder.SetLength(ptrChosenFolder.Length()-1);

	lastOcc=ptrChosenFolder.LocateReverse('\\');
	if (lastOcc==KErrNotFound) lastOcc=-1;

	TPtrC16 shortname=ptrChosenFolder.Right(ptrChosenFolder.Length()-(lastOcc+1));

	CSTMPackageNode* actualNode = aParentNode->AddChild(shortname,shortname,EFolderNode);

	if (actualNode!=NULL)
	{
		RFs session;
		HBufC* temp = HBufC::NewL(200);
		TPtr ptrTemp = temp->Des();
		ptrTemp.Copy(aFolderPath);
		ptrTemp.Append(_L("*"));
		session.Connect();
		CDir* dirList;
		session.GetDir(ptrTemp, KEntryAttMaskSupported, ESortByName, dirList);
		for (TInt i=0; i<dirList->Count(); i++)
		{
			if ((*dirList)[i].IsDir())
			{
				ptrTemp.FillZ();
				ptrTemp.Copy(aFolderPath);
				ptrTemp.Append((*dirList)[i].iName);
				ptrTemp.Append(_L("\\"));

				AddSpecificFolderToListL(ptrTemp, actualNode);
			}
			else
			{
				HBufC* temp2 = HBufC::NewL(200);
				TPtr ptrTemp2 = temp2->Des();
				ptrTemp2.Copy(aFolderPath);
				ptrTemp2.Append((*dirList)[i].iName);

				actualNode->AddChild((*dirList)[i].iName,ptrTemp2,EFileNode);

				delete temp2;
			}
		}
		delete temp;
		delete dirList;
		session.Close();
	}
	else
	{
	}

	delete chosenFolder;
}


/**
* Enter specified subfolder node
*/
void CSymTorrentMakerFileListView::EnterFolderOrShowDetailNode(TInt aPosition)
{
	if (aPosition<0) return;

	TInt newPos=0, i=1;
	TBuf<80> temp;
	temp.Copy(iTorrentHandler->GetCurrentNode()->GetNodeName());

	if (iTrackerManager->EnterFolderOrShowDetails(aPosition))
	{
		RefreshList();

		CSTMPackageNode *tempnode = iTorrentHandler->GetCurrentNode()->GetFirstChild();

		while (tempnode!=NULL)
		{
			if (tempnode->GetNodeName().Compare(temp)==0)
				newPos = i;

			tempnode = tempnode->GetNextSibling();
			i++;
		}		

		iContainer->SetCursorPosition(newPos);
	}
}

/**
* Remove specified node
*/
void CSymTorrentMakerFileListView::RemoveNode(TInt aPosition)
{
	TInt cursorpos = iContainer->GetCursorPosition();
	cursorpos--;

	if (aPosition<0) return;

	if (aPosition==0)
	{
		(iTorrentHandler->GetCurrentNode())->DeleteChildren();
		return;	
	}

	CSTMPackageNode *tempnode = (iTorrentHandler->GetCurrentNode())->GetFirstChild();
	if (tempnode==NULL) return;

	if (aPosition==1)
	{
		if (tempnode!=NULL)
		{
			(tempnode->GetParent())->SetFirstChild(tempnode->GetNextSibling());
			tempnode->SetNextSibling(NULL);

			delete tempnode;
			tempnode = NULL;
		}
		else 
			return;
	}
	else
	{
		TInt i=2;

		while (i<aPosition && tempnode->GetNextSibling()!=NULL)
		{
			tempnode = tempnode->GetNextSibling();
			i++;
		}

		if (tempnode->GetNextSibling()!=NULL)
		{
			CSTMPackageNode *tempnode2 = tempnode->GetNextSibling();
			tempnode->SetNextSibling(tempnode2->GetNextSibling());
			tempnode2->SetNextSibling(NULL);

			delete tempnode2;
			tempnode2 = NULL;
		}
		else 
			return;
	}

	RefreshList();
	iContainer->SetCursorPosition(cursorpos);

}

/**
* Menu visibility settings
*/
void CSymTorrentMakerFileListView::DynInitMenuPaneL(TInt aResourceId, CEikMenuPane* aMenuPane )
{
	if ( aResourceId == R_SYMTORRENTMAKER_VIEWFILELIST_MENU )
	{
		aMenuPane->SetItemDimmed( ESymTorrentMakerCmdExitFolder,	  iTorrentHandler->GetCurrentNode()==iTorrentHandler->GetTorrentPackage()->GetRootNode() );
		aMenuPane->SetItemDimmed( ESymTorrentMakerCmdEnterFolder,     (iContainer->GetCursorPosition() == 0 || iContainer->GetCurrentItemType()!=1) );
		aMenuPane->SetItemDimmed( ESymTorrentMakerCmdFileDetails,	  (iContainer->GetCursorPosition() == 0 || iContainer->GetCurrentItemType()!=0) );

		aMenuPane->SetItemDimmed( ESymTorrentMakerCmdDelCurrent,	  iContainer->GetCursorPosition()==0);
		aMenuPane->SetItemDimmed( ESymTorrentMakerCmdDeleteChildren,  (iContainer->GetCursorPosition()>0  || iTorrentHandler->GetCurrentNode()->GetFirstChild()==NULL));
		
	}
//	else if ( aResourceId == R_SYMTRACKER_APP_MENU )
//	{
//	    aMenuPane->SetItemDimmed( ESymTorrentCmdTracker, ETrue );		
//	}
}

/**
* Refreshing list from nodes
*/
void CSymTorrentMakerFileListView::RefreshList()
{

	CSTMPackageNode*	referenceNode = iTorrentHandler->GetCurrentNode();
	iContainer->ClearList();

	if (referenceNode==NULL) return;

	iContainer->AddListItem(referenceNode,ETrue);

	CSTMPackageNode*	childNode = referenceNode->GetFirstChild();

	while (childNode!=NULL)
	{
		iContainer->AddListItem(childNode,EFalse);
		childNode=childNode->GetNextSibling();
	}

	TBuf<300> fullPath;
	iTorrentHandler->GetCurrentNode()->GetFullPathInto(fullPath);

	CEikStatusPane* statusPane = AppUi()->StatusPane();
	TUid paneUid;
	paneUid.iUid = EEikStatusPaneUidTitle;
	if (statusPane->PaneCapabilities(paneUid).IsAppOwned())
	{
		CAknTitlePane* titlePane = (CAknTitlePane*) statusPane->ControlL(paneUid);
		titlePane->SetTextL(fullPath);
	}
}


// End of File

