/*****************************************************************************
 * Copyright (C) 2006-2007 Imre Kelnyi
 *-------------------------------------------------------------------
 * This file is part of SymTorrent
 *
 * SymTorrent is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTorrent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Symella; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

#include "SymTorrentTrackerManager.h"
#include "SymtorrentmakerDetailsView.h"
#include "hash.h"
#include "STTConnectionSocketCollector.h"
#include "SymTorrentMakerFileListView.h"
#include "SymtorrentmakerDetailsView.h"
#include "STTorrent.h"
#include "TorrentMaker.h"
#include "STTTorrentListItem.h"
#include "SymTorrentTracker.hrh"
#include "KiLogger.h"
#include "KiLogManager.h"

#include <caknmemoryselectiondialog.h>
#include <caknfileselectiondialog.h> 
#include <akntitle.h> 
#include <SymTracker.rsg>
#include <utf.h>
#include <aknquerydialog.h>
#include <aknnotewrappers.h> 
#include <aknviewappui.h>

// the maximum length of the setting names is 50 characters!
const TInt KMaxSettingNameLength = 50;
_LIT8(KSettingProxyAddress, "proxyaddress");
_LIT8(KSettingConnectionPort, "connectionport");
_LIT8(KSettingServicePort, "serviceport");
_LIT8(KSettingPartnerPort, "partnerport");
_LIT8(KSettingTorrent, "torrent");
_LIT(KTrackerDataFile, "C:\\trackerdatas.tdt");
_LIT(KAnnounceUrlFormat,"http://%S:%d/announce");
// storrent constants
_LIT8(KLitBitfield, "bitfield");
_LIT8(KLitPath, "path");
// File handling constants
_LIT8(KLit8Colon, ":");
_LIT8(KLit8EqualSign, "=");
_LIT8(KLit8EndLine, "\r\n");


EXPORT_C CSymTorrentTrackerManager::CSymTorrentTrackerManager(CAknViewAppUi* aAppUi, CSTTorrentManager* aTorrentMgr, MTrackerUIConnection* aUiConn)
 : iAppUi(aAppUi), iTorrentMgr(aTorrentMgr), iUiConn(aUiConn)
{
	
}

EXPORT_C void CSymTorrentTrackerManager::ConstructL()
{
	iTorrentHandler = CSTMTorrentHandler::NewL();
	// initializing tracker data
	InitializeL();	

	// program torrentlist view
    iTorrentListView = new (ELeave) CSymTorrentTrackerTorrentListView(this);

    CleanupStack::PushL( iTorrentListView );
    iTorrentListView->ConstructL();
    iAppUi->AddViewL( iTorrentListView );			// transfer ownership to CAknViewAppUi
    CleanupStack::Pop();			// iTorrentListView

	// torrentMaker views

    CSymTorrentMakerFileListView* viewFileList = new (ELeave) CSymTorrentMakerFileListView(this);
    CleanupStack::PushL( viewFileList );
    viewFileList->ConstructL();
    iAppUi->AddViewL( viewFileList );      // transfer ownership to CAknViewAppUi
    CleanupStack::Pop();           // viewFileList
    
    CSymTorrentMakerDetailsView* detailsView = new (ELeave) CSymTorrentMakerDetailsView;
    CleanupStack::PushL(detailsView);
    detailsView->ConstructL(this);
    iAppUi->AddViewL(detailsView);
    CleanupStack::Pop();			// detailsView

	iBaseData->SetTorrentObserver(iTorrentListView);

	LoadTrackerDatasFromFileL();
}

EXPORT_C CSymTorrentTrackerManager::~CSymTorrentTrackerManager()
{
	// iConnector must be deleted prior to deleting the engine (which happens before deleting tha appui!)
	delete iConnector;
	iConnector = NULL;

	delete iBaseData;

	delete ((CSTTConnectionSocketCollector*)iSocketCollector);

	delete iTorrentHandler; 
}


/**
* File select dialog with memory card selection
*/
TBool CSymTorrentTrackerManager::FileSelectQueryL(TFileName &aFileName, TCommonDialogType aDialogType, TBuf<64> aTitle)
{
	// Select memory
	CAknMemorySelectionDialog* memSelectionDialog = CAknMemorySelectionDialog::NewL
		(ECFDDialogTypeNormal, /*aShowUnavailableDrives*/EFalse);
	CleanupStack::PushL(memSelectionDialog);
	CAknMemorySelectionDialog::TMemory mem(CAknMemorySelectionDialog::EPhoneMemory);

	TInt ret = memSelectionDialog->ExecuteL(mem);
	CleanupStack::PopAndDestroy(memSelectionDialog);
	if (!ret) return EFalse;

	// get root on phone memory or card
	CAknFileSelectionDialog* iFileSelectionDialog;	
	if (mem == CAknMemorySelectionDialog::EMemoryCard)
	{		
		iFileSelectionDialog= CAknFileSelectionDialog::NewL(aDialogType,R_FILE_SELECTION_DIALOG_E );
	}
	else
	{		
		iFileSelectionDialog= CAknFileSelectionDialog::NewL(aDialogType,R_FILE_SELECTION_DIALOG_C );
	}		

	iFileSelectionDialog->SetTitleL(aTitle);
	TBool ok = iFileSelectionDialog->ExecuteL(aFileName);

	delete iFileSelectionDialog;

	return ok;
}

TBool CSymTorrentTrackerManager::EnterFolderOrShowDetails(TInt aPosition)
{
	CSTMPackageNode* iCurrentNode = iTorrentHandler->GetCurrentNode();

	if (aPosition==0)
	{
		if (iCurrentNode->GetParent()!=NULL)
		{
			iTorrentHandler->SetCurrentNode(iCurrentNode->GetParent());
			return ETrue;
		}
		else
			return EFalse;
	}
	else
	{
		TInt i=1;
		CSTMPackageNode* current = iCurrentNode->GetFirstChild();
		while (i < aPosition)
		{
			if (current!=NULL)
				current = current->GetNextSibling();
			else
			    return EFalse;

			i++;
		}

		if (current!=NULL && current->GetNodeType()==EFolderNode)
		{
			iTorrentHandler->SetCurrentNode(current);
			return ETrue;
		}
		else
		{
			if (current->GetNodeType()==EFileNode)
			{
				iFileDetails.fullPath.Copy(current->GetNodeFullName());
				iFileDetails.filesize = current->GetFileSize();
				iAppUi->ActivateLocalViewL(TUid::Uid(ESymTorrentMakerDetailsView));
			}
			return EFalse;
		}
	}
}

// Initialize base data
void CSymTorrentTrackerManager::InitializeL()
{
	delete iConnector;
	iConnector = NULL;

	iBaseData = CSTTBaseData::NewL();

	iSocketCollector = CSTTConnectionSocketCollector::NewL(); 
	((CSTTConnectionSocketCollector*)iSocketCollector)->Initialize(this);

	TBuf<20> prefix=_L("_SymTracker");
	iConnector = CSTTConnector::NewL(&iTorrentMgr->NetworkManager()->SocketServ(), 
		iSocketCollector, iBaseData, prefix, NULL, NULL);

	iInTorrentMaker = EFalse;
	iInTracker = EFalse;
}

// ----------------------------------------------------
// CSymTorrentTrackerManager::ActivateTrackerTorrentListViewL()
// Shows the TrackerTorrentListView
// ----------------------------------------------------
//


// Loads tracker datas (settings, torrents) from file
void CSymTorrentTrackerManager::LoadTrackerDatasFromFileL()
{
	RFs fs;
	User::LeaveIfError(fs.Connect());
	CleanupClosePushL(fs);
	
	RFile file;
	if (file.Open(fs, KTrackerDataFile, EFileRead|EFileShareReadersOnly) == KErrNone)
	{
		CleanupClosePushL(file);
		TBuf8<KMaxSettingNameLength> settingName;
		TInt settingLength;
		
		while (ReadSettingName(file, settingName, settingLength))
		{			
			if (settingName == KSettingProxyAddress)
			{
				HBufC* proxyAddress = ReadAndConvertFromUtf8ToUnicodeL(file, settingLength);
				
				if (proxyAddress)
				{					
					iTorrentMgr->Preferences()->SetProxyAddressL(proxyAddress->Des());
				}
				delete proxyAddress;
			}
			else if (settingName == KSettingConnectionPort)
			{
				HBufC* connectionPort = ReadAndConvertFromUtf8ToUnicodeL(file, settingLength);
				
				if (connectionPort)
				{
					TLex lex(*connectionPort);
					lex.Mark();
					
					TInt connectionPortInt = 0;
					if (lex.Val(connectionPortInt) == KErrNone)
						iTorrentMgr->Preferences()->SetProxyConnectionPort(connectionPortInt);
				}				
				delete connectionPort;
			}
			else if (settingName == KSettingServicePort)
			{
				HBufC* servicePort = ReadAndConvertFromUtf8ToUnicodeL(file, settingLength);
				
				if (servicePort)
				{
					TLex lex(*servicePort);
					lex.Mark();
					
					TInt servicePortInt = 0;
					if (lex.Val(servicePortInt) == KErrNone)
						iTorrentMgr->Preferences()->SetProxyServicePort(servicePortInt);
				}				
				delete servicePort;
			}
			else if (settingName == KSettingPartnerPort)
			{
				HBufC* partnerPort = ReadAndConvertFromUtf8ToUnicodeL(file, settingLength);
				
				if (partnerPort)
				{
					TLex lex(*partnerPort);
					lex.Mark();
					
					TInt partnerPortInt = 0;
					if (lex.Val(partnerPortInt) == KErrNone)
						iBaseData->SetPartnerPort(partnerPortInt);
				}				
				delete partnerPort;
			}
			else if (settingName == KSettingTorrent)
			{
				HBufC* torrent = ReadAndConvertFromUtf8ToUnicodeL(file, settingLength);
				CleanupStack::PushL(torrent);
				
				AddTorrentFileL(torrent->Des());
				
				CleanupStack::PopAndDestroy(); // torrent
			}
		}		
		CleanupStack::PopAndDestroy(); // file
	}	
	CleanupStack::PopAndDestroy(); // fs
}

// Add new torrent file to list
TBool CSymTorrentTrackerManager::AddTorrentFileL(const TDesC& aFilename)
{
	TBool succ = ETrue;
	HBufC8* infobuf = NULL;
	TBuf8<100> infohash;

    RFs fs;
	User::LeaveIfError(fs.Connect());
	CleanupClosePushL(fs);
	
	RFile file;
	if (file.Open(fs, aFilename, EFileRead|EFileShareReadersOnly) == KErrNone)
	{
		CleanupClosePushL(file);
		
		TInt fileSize;
		User::LeaveIfError(file.Size(fileSize));
		HBufC8* buf = HBufC8::NewLC(fileSize);
		TPtr8 ptr(buf->Des());
		User::LeaveIfError(file.Read(ptr));
		
		CSTBencode* torrent = CSTBencode::ParseL(*buf);
	    if (torrent)
		{
			infobuf  = (((CSTBencodedDictionary*)torrent)->EntryValue(_L8("info")))->BencodeL();
			CSHA1* mySha = CSHA1::NewL();
			infohash.Copy(_L8(""));
			infohash.Append(mySha->Hash(*infobuf));
			delete mySha;
	    	delete torrent;
	    }
		else
			succ = EFalse;
		
	   	CleanupStack::PopAndDestroy(); // buf

		CleanupStack::PopAndDestroy(); // file
	}	
	else
		succ = EFalse;
	CleanupStack::PopAndDestroy(); // fs

	if (infobuf) delete infobuf;

	if (succ)
		return (iBaseData->InsertListItemL(aFilename,infohash)!=NULL);
	else
		return EFalse;
}

// Reads settingName
TBool CSymTorrentTrackerManager::ReadSettingName(RFile& aFile, TDes8& aSettingName, TInt& aSettingLength)
{
	aSettingName.SetLength(0);
	
	TBuf8<KMaxSettingNameLength + 10> buffer;
	
	if (aFile.Read(buffer) != KErrNone)
		return EFalse;
	
	TInt filePos = 0;
	if (aFile.Seek(ESeekCurrent, filePos) != KErrNone)
		return EFalse;
	
	TLex8 lex(buffer);
	lex.SkipSpaceAndMark();
	
	while (lex.Peek())
	{
		if (lex.Peek() == ':')
		{
			aSettingName = lex.MarkedToken();
			lex.Inc();
			break;
		}		
		lex.Inc();
	}
	
	if (lex.Val(aSettingLength) != KErrNone)
		return EFalse;

	while (lex.Peek())
	{
		if (lex.Peek() == '=')
			break;		
		lex.Inc();
	}
	
	if (lex.Peek() != '=')
		return EFalse;
	
	lex.Inc();
	
	TInt offset = filePos - (buffer.Length() - lex.Offset());
	if (aFile.Seek(ESeekStart, offset) != KErrNone)
		return EFalse;
	
	return ETrue;
}

// Converts to utf8
HBufC8* CSymTorrentTrackerManager::ConvertToUtf8L(const TDesC& aData)
{	
	HBufC8* encodedData = HBufC8::NewLC(aData.Length() * 3);
	
	TPtr8 encodedDataPtr = encodedData->Des();
	if (CnvUtfConverter::ConvertFromUnicodeToUtf8(encodedDataPtr, aData) != 0)
	{
		CleanupStack::PopAndDestroy(); // encodedData
		return NULL;
	}	
	CleanupStack::Pop(); // encodedData		
	return encodedData;
}

// Converts from utf8
HBufC* CSymTorrentTrackerManager::ReadAndConvertFromUtf8ToUnicodeL(RFile& aFile, TInt aLength)
{
	HBufC8* utfBuffer = HBufC8::NewLC(aLength);
	TPtr8 utfBufferPtr = utfBuffer->Des();
	
	if ((aFile.Read(utfBufferPtr, aLength) != KErrNone) ||
		(utfBuffer->Length() != aLength))
	{
		CleanupStack::PopAndDestroy(); // utfBuffer
		return NULL;
	}			
	
	HBufC* unicodeBuffer = HBufC::NewLC(aLength);
	TPtr unicodeBufferPtr = unicodeBuffer->Des();
	
	if (CnvUtfConverter::ConvertToUnicodeFromUtf8(unicodeBufferPtr, *utfBuffer) != KErrNone)
	{
		CleanupStack::PopAndDestroy(); // unicodeBuffer
		CleanupStack::PopAndDestroy(); // utfBuffer
		return NULL;
	}

	CleanupStack::Pop(); // unicodeBuffer
	CleanupStack::PopAndDestroy(); // utfBuffer
	return unicodeBuffer;
}

// Selects torrent file
TBool CSymTorrentTrackerManager::FileSelectQuery(TFileName &aFileName)
{
	// Select memory
	CAknMemorySelectionDialog* memSelectionDialog = CAknMemorySelectionDialog::NewL
		(ECFDDialogTypeNormal, /*aShowUnavailableDrives*/EFalse);
	CleanupStack::PushL(memSelectionDialog);

	CAknMemorySelectionDialog::TMemory mem(CAknMemorySelectionDialog::EPhoneMemory);

	TInt ret = memSelectionDialog->ExecuteL(mem);
	CleanupStack::PopAndDestroy(memSelectionDialog);
	if (!ret) return EFalse;

	// get root on phone memory or card
	CAknFileSelectionDialog* iFileSelectionDialog;	
	if (mem == CAknMemorySelectionDialog::EMemoryCard)
	{		
		iFileSelectionDialog= CAknFileSelectionDialog::NewL(ECFDDialogTypeNormal,R_TORRENTFILE_SELECTION_DIALOG_E );
	}
	else
	{		
		iFileSelectionDialog= CAknFileSelectionDialog::NewL(ECFDDialogTypeNormal,R_TORRENTFILE_SELECTION_DIALOG_C );
	}	

	// launch file select dialog
	TBool result=iFileSelectionDialog->ExecuteL(aFileName);
	delete iFileSelectionDialog;
	return result;
}

// sets the titlepane label
EXPORT_C void CSymTorrentTrackerManager::setTitlePane(TBuf<256> aTitle)
{
	CEikStatusPane* statusPane = iAppUi->StatusPane();
	TUid paneUid;
	paneUid.iUid = EEikStatusPaneUidTitle;
	if (statusPane->PaneCapabilities(paneUid).IsAppOwned())
	{
		CAknTitlePane* titlePane = (CAknTitlePane*) statusPane->ControlL(paneUid);
		titlePane->SetTextL(aTitle);
	}
}

void CSymTorrentTrackerManager::TrackerCmdCreateTorrent()
{
	iAppUi->ActivateLocalViewL(TUid::Uid(ESymTorrentMakerFileListView));
	//iAppUi->ActivateTorrentMakerFileListView();
	iInTorrentMaker = ETrue;	
}


EXPORT_C void CSymTorrentTrackerManager::StartTrackerL(RConnection* aConnection)
{
	((CSTTConnectionSocketCollector* )iSocketCollector)->Initialize(this);

//	if (iTorrentMgr->Preferences()->IncomingConnectionsMode()==EEnabledWithProxy)
//		iConnector->StartL(EConProxy, aConnection);
//	else
		iConnector->StartL(EConListener, aConnection);
	
	if (InTracker() || InTorrentMaker())
		static_cast<CSymTorrentTrackerTorrentListView*>(iAppUi->View(TUid::Uid(ESymTorrentTrackerTorrentListView)))->UpdateTrackerStatusNaviPaneL();
}

// Stop the tracker
void CSymTorrentTrackerManager::TrackerCmdStopTracker()
{
	iConnector->Stop();
	
	if (InTracker() || InTorrentMaker())
		static_cast<CSymTorrentTrackerTorrentListView*>(iAppUi->View(TUid::Uid(ESymTorrentTrackerTorrentListView)))->UpdateTrackerStatusNaviPaneL();
	
	/*if ((iAppUi->FocusedView() == TUid::Uid(ESymTorrentTrackerTorrentListView)) || (iAppUi->FocusedView() == TUid::Uid(ESymTorrentMakerFileListView)))
		static_cast<CSymTorrentTrackerTorrentListView*>(iAppUi->View(TUid::Uid(ESymTorrentTrackerTorrentListView)))->UpdateTrackerStatusNaviPaneL();*/
}

// Call file selection dialog
void CSymTorrentTrackerManager::TrackerCmdAddTorrent()
{
	TBuf<256> filename;
	if (FileSelectQuery(filename))
	{
 		if (AddTorrentFileL(filename)) 
			iTorrentListView->RefreshTorrentList();
	}
}

void CSymTorrentTrackerManager::MakerCmdBack()
{
	if (!((iTorrentHandler->GetTorrentPackage()->GetRootNode()->GetFirstChild() == NULL)
		&& (iTorrentHandler->GetTorrentPackage()->GetRootNode()->GetNextSibling() == NULL)))
	{
		if (((CEikonEnv*) CCoeEnv::Static())->QueryWinL(_L("Save torrent?"), _L("")))
			SaveTorrent();
	}
	iAppUi->ActivateLocalViewL(TUid::Uid(ESymTorrentTrackerTorrentListView));
	iInTorrentMaker = EFalse;
	setTitlePane(_L("Tracker"));	
}

// Save maker duties
void CSymTorrentTrackerManager::MakerCmdSave()
{
	if (!((iTorrentHandler->GetTorrentPackage()->GetRootNode()->GetFirstChild() == NULL)
		&& (iTorrentHandler->GetTorrentPackage()->GetRootNode()->GetNextSibling() == NULL)))
	{
		SaveTorrent();
		iAppUi->ActivateLocalViewL(TUid::Uid(ESymTorrentTrackerTorrentListView));
		iInTorrentMaker = EFalse;
		setTitlePane(_L("Tracker"));	
	}
	else
	{
		CAknWarningNote* warningNote = new (ELeave) CAknWarningNote;
    	warningNote->ExecuteLD(_L("No content added!"));
	}
}

// Maker functions ----------------------------------------------------------------------

// Cancel maker duties
void CSymTorrentTrackerManager::CancelMaker()
{
	if ((iTorrentHandler->GetTorrentPackage()->GetRootNode()->GetFirstChild() == NULL)
		&& (iTorrentHandler->GetTorrentPackage()->GetRootNode()->GetNextSibling() == NULL))
	{
		iTorrentHandler->Cancel();
		iAppUi->ActivateLocalViewL(TUid::Uid(ESymTorrentTrackerTorrentListView));
		iInTorrentMaker = EFalse;
		setTitlePane(_L("Tracker"));
	}
	else
	{
		if (((CEikonEnv*) CCoeEnv::Static())->QueryWinL(_L("Are you sure?"), _L("")))
		{
			iTorrentHandler->Cancel();
			iAppUi->ActivateLocalViewL(TUid::Uid(ESymTorrentTrackerTorrentListView));
			iInTorrentMaker = EFalse;
			setTitlePane(_L("Tracker"));
		}
	}
}

void CSymTorrentTrackerManager::SaveTorrent()
{
	TBuf<256> chosenfolder;

	if (!FileSelectQueryL(chosenfolder,ECFDDialogTypeBrowse,_L("Select folder:")))
		return;

	CSTMPackage* package;
	TBuf<300>   filename;

	package = iTorrentHandler->GetTorrentPackage();
	
	CAknTextQueryDialog * dialog = CAknTextQueryDialog::NewL(filename);
	CleanupStack::PushL(dialog);
	dialog->SetPromptL(_L("Enter torrent filename:"));
	CleanupStack::Pop(dialog);

	if (!dialog->ExecuteLD(R_SYMTORRENTMAKER_FILENAME_QUERY))
		return;

	TBuf<100> announce;
	TBuf<100> addresstemp;

	// set announce from proxy address and port
	if (iConnector->GetConnectorType()==EConProxy)
	{
		addresstemp.Copy(iTorrentMgr->Preferences()->ProxyHostName());
		announce.Format(KAnnounceUrlFormat,&addresstemp,iBaseData->GetPartnerPort());
	}
	else
	{
		TInetAddr addr;
		iConnector->GetLocalIPAddressSetConenctionL(addr);
		addr.Output(addresstemp);
		announce.Format(KAnnounceUrlFormat,&addresstemp,iTorrentMgr->Preferences()->TrackerServicePort());
	}

	iTorrentHandler->GetTorrentPackage()->SetAnnounceUrl(announce);

	package->SetTorrentSavePath(chosenfolder);
	package->SetTorrentFileName(filename);

	iTorrentHandler->SaveTorrent(ETrue);


	// -------
	TBuf<300> torrentfilename;
	TBuf<300> storrentfilename;
	TBuf<300> contentpath;

	torrentfilename.Copy(chosenfolder);
	torrentfilename.Append(filename);
	torrentfilename.Append(_L("\\"));
	contentpath.Copy(torrentfilename);
	contentpath.Append(_L("content\\"));
	torrentfilename.Append(filename);
	storrentfilename.Copy(torrentfilename);
	torrentfilename.Append(_L(".torrent"));
	storrentfilename.Append(_L(".storrent"));

 	if (AddTorrentFileL(torrentfilename)) 
		iTorrentListView->RefreshTorrentList();

//	if (CreateSTorrent(torrentfilename,storrentfilename,contentpath))
//	{
		CSTTorrent* torrent = NULL;
		if (iTorrentMgr->OpenSavedTorrentL(storrentfilename, torrent) == KErrNone)
			torrent->StartL();
//	}
		
}

/*
// Create storrent
TBool CSymTorrentTrackerManager::CreateSTorrent(const TDesC& torrentfilename, const TDesC& storrentfilename, const TDesC& contentpath)
{
	TBool result=EFalse;
	RFs fs;
	User::LeaveIfError(fs.Connect());
	CleanupClosePushL(fs);

	RFile torrentFile;

	TInt result1 = torrentFile.Open(fs, torrentfilename, 
		EFileRead|EFileShareReadersOnly);

	if (result1 == KErrNone)
	{		
		CleanupClosePushL(torrentFile);

		TInt fileSize;
		User::LeaveIfError(torrentFile.Size(fileSize));
		HBufC8* torrentBuf = HBufC8::NewLC(fileSize);
		TPtr8 torrentBufPtr(torrentBuf->Des());
		User::LeaveIfError(torrentFile.Read(torrentBufPtr));
		
		RFile stFile;
		TInt result2 = 
			stFile.Replace(fs, storrentfilename, EFileWrite|EFileShareExclusive);
			
		if (result2 == KErrNone)
		{
			CleanupClosePushL(stFile);
			
			TBuf8<10> lengthBuf;
			lengthBuf.Num(torrentBuf->Length());
			while (lengthBuf.Length() < 10) // fixed 10 decimals length
				lengthBuf.Insert(0, _L8("0"));
			stFile.Write(lengthBuf);
			stFile.Write(*torrentBuf);
			
			// write the bencoded dictionary in the file
			while(1)
			{
				CSTBencodedDictionary* state = new (ELeave) CSTBencodedDictionary;
				CleanupStack::PushL(state);
				
				{
					TBuf8<100> field;										
					field.Copy(_L8(""));

					TInt lastPieceNum = iTorrentHandler->GetLastPieceNum();
					TInt needone = 1+(lastPieceNum-1)/8;
					TChar ffchar = 255;
					for (TInt i=0;i<needone;i++)					
						field.Append(ffchar);

					CSTBencodedString* bitfield = 
						CSTBencodedString::NewL(field);
					CSTBencodedString* key1 = CSTBencodedString::NewL (KLitBitfield);
				    state->AddEntryL(key1, bitfield);
				}
				{
					
					HBufC8* encodedPath = HBufC8::NewLC(900);
					TPtr8 encodedPathPtr = encodedPath->Des();
					if (CnvUtfConverter::ConvertFromUnicodeToUtf8(encodedPathPtr, contentpath) != 0)
					{
						CleanupStack::PopAndDestroy(); // encodedPath
						break;
					}
					
					CSTBencodedString* path = 
						CSTBencodedString::NewL(*encodedPath);
					CleanupStack::PopAndDestroy(); // encodedPath
					CSTBencodedString* key2 = CSTBencodedString::NewL(KLitPath);
					state->AddEntryL(key2, path);
				}

				HBufC8* stateBencoded = state->BencodeL();
				TBuf8<10> stateLengthBuf;
				stateLengthBuf.Num(stateBencoded->Length());
				while (stateLengthBuf.Length() < 10) // fixed 10 decimals length
					stateLengthBuf.Insert(0, _L8("0"));
				stFile.Write(stateLengthBuf);
				stFile.Write(*stateBencoded);
				delete stateBencoded;
				
				result=ETrue;

				CleanupStack::PopAndDestroy(); // state				
				break;
			}
			CleanupStack::PopAndDestroy(); // stFile
		}		

		CleanupStack::PopAndDestroy(); // torrentBuf
		CleanupStack::PopAndDestroy(); // torrentFile
	}	
	CleanupStack::PopAndDestroy(); // fs

	return result;
}
*/
// Saves tracker datas (settings, torrents) to file
EXPORT_C void CSymTorrentTrackerManager::SaveTrackerDatasToFile()
{	
	RFs fs;
	User::LeaveIfError(fs.Connect());
	CleanupClosePushL(fs);
	
	RFile file;
	if ( file.Replace(fs, KTrackerDataFile, EFileWrite) == KErrNone)
	{
		CleanupClosePushL(file);	

		// Proxy address
		SaveSettingL(file, KSettingProxyAddress, iTorrentMgr->Preferences()->ProxyHostName());

		// Connection port
		TBuf<16> connectionPort;
		connectionPort.Num(iTorrentMgr->Preferences()->ProxyConnectionPort());
		SaveSettingL(file, KSettingConnectionPort, connectionPort);	

		// Service port
		TBuf<16> servicePort;
		servicePort.Num(iTorrentMgr->Preferences()->ProxyServicePort());
		SaveSettingL(file, KSettingServicePort, servicePort);	

		// Partner port
		TBuf<16> partnerPort;
		partnerPort.Num(iBaseData->GetPartnerPort());
		SaveSettingL(file, KSettingPartnerPort, partnerPort);	

		// Torrents
		CSTTTorrentListItem* tempItem = iBaseData->GetTorrentListFirstItem();
		while (tempItem!=NULL)
		{
			SaveSettingL(file, KSettingTorrent, tempItem->GetTorrentFileName());
			tempItem=tempItem->GetNextSibling();
		}

		file.Write(KLit8EndLine);
		CleanupStack::PopAndDestroy(); // file
	}
	
	CleanupStack::PopAndDestroy(); // fs	
}

void CSymTorrentTrackerManager::SaveSettingL(RFile& aFile, const TDesC8& aSettingName, const TDesC& aSettingValue)
{
	aFile.Write(aSettingName);
	aFile.Write(KLit8Colon);
	
	
	HBufC8* encodedSettingValue = ConvertToUtf8L(aSettingValue);
	if (encodedSettingValue)
	{
		CleanupStack::PushL(encodedSettingValue);
		
		TBuf8<16> settingLength;
		settingLength.Num(encodedSettingValue->Length());
		aFile.Write(settingLength);
		
		aFile.Write(KLit8EqualSign);
		aFile.Write(*encodedSettingValue);
		CleanupStack::PopAndDestroy(); // encodedSettingValue
	}
	else
	{
		_LIT8(KLit8Unset, "0=");
		aFile.Write(KLit8Unset);
	}
	
	aFile.Write(KLit8EndLine);
}

CSTMTorrentHandler* CSymTorrentTrackerManager::TorrentHandler()
{
	return iTorrentHandler;
}

EXPORT_C void CSymTorrentTrackerManager::SetPieceSize(TInt aPieceSize)
{
	CSTMPackage* package = TorrentHandler()->GetTorrentPackage();
	package->SetPieceSize(aPieceSize);
}

EXPORT_C void CSymTorrentTrackerManager::NetworkConnectionStartedL(TBool /*aResult*/, RConnection& /*aConnection*/)
{	
}

EXPORT_C TConnectorSimpleStates CSymTorrentTrackerManager::CSymTorrentTrackerManager::GetConnectorState()
{
	if (iConnector!=NULL)
		return iConnector->GetConnectorState();
	else
		return EConOffline;
}
