/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

// INCLUDE FILES
#include "SymTorrentTrackerTorrentListContainer.h"
#include "STTTorrentListItem.h"

#include <CAknFileSelectionDialog.h>
#include <badesca.h>
#include <eikclb.h>
#include <eikclbd.h>
#include <akniconutils.h> 
#include <aknlists.h> 
#include <AknsUtils.h>
#include <akniconarray.h>
#include <aknnotewrappers.h> 

#include <SymTrackerState.mbg>

// Constants
_LIT(KBitmapFile, "\\resource\\apps\\SymTrackerState.mif");
_LIT(KTorrentListFormat,"%d\t%S");


// ================= MEMBER FUNCTIONS =======================

// Constructor
CSymTorrentTrackerTorrentListContainer::CSymTorrentTrackerTorrentListContainer(CSTTBaseData* aBaseData, CSymTorrentTrackerTorrentListView* aView)
: iBaseData(aBaseData), iView(aView)
{
}

// ---------------------------------------------------------
// CSymTorrentTrackerTorrentListContainer::ConstructL(const TRect& aRect)
// EPOC two phased constructor
// ---------------------------------------------------------
//
void CSymTorrentTrackerTorrentListContainer::ConstructL(const TRect& aRect)
{
    CreateWindowL();

    iTorrentListBox = new (ELeave) CAknSingleGraphicStyleListBox;
    //iTorrentListBox->SetMopParent(this);
	iTorrentListBox->ConstructL(this,EAknListBoxSelectionList);
	iTorrentListBox->SetContainerWindowL(*this);	

	//create scrollbar
	iTorrentListBox->CreateScrollBarFrameL();
	iTorrentListBox->ScrollBarFrame()->SetScrollBarVisibilityL(
	CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto);

    CArrayPtr<CGulIcon>* icons = new (ELeave) CAknIconArray(4);
	// array of the icons
	CleanupStack::PushL(icons);

		CGulIcon* offlineIcon = CGulIcon::NewL();
		CleanupStack::PushL(offlineIcon);
		CGulIcon* nopeerIcon = CGulIcon::NewL();
		CleanupStack::PushL(nopeerIcon);
		CGulIcon* fewpeersIcon = CGulIcon::NewL();
		CleanupStack::PushL(fewpeersIcon);
		CGulIcon* manypeersIcon = CGulIcon::NewL();
		CleanupStack::PushL(manypeersIcon);

		CFbsBitmap* offlineBmp = NULL;
		CFbsBitmap* offlineMask = NULL;
		AknIconUtils::CreateIconLC(offlineBmp, offlineMask, KBitmapFile,
			EMbmSymtrackerstateOffline, EMbmSymtrackerstateOffline_mask);
		offlineIcon->SetBitmap(offlineBmp);
		offlineIcon->SetMask(offlineMask);
		CleanupStack::Pop(2);
		icons->AppendL(offlineIcon);
		CleanupStack::Pop();

		CFbsBitmap* nopeerBmp = NULL;
		CFbsBitmap* nopeerMask = NULL;
		AknIconUtils::CreateIconLC(nopeerBmp, nopeerMask, KBitmapFile,
			EMbmSymtrackerstateNo_peer, EMbmSymtrackerstateNo_peer_mask);	
		nopeerIcon->SetBitmap(nopeerBmp);
		nopeerIcon->SetMask(nopeerMask);
		CleanupStack::Pop(2);
		icons->AppendL(nopeerIcon);
		CleanupStack::Pop();
	
		CFbsBitmap* fewpeersBmp = NULL;
		CFbsBitmap* fewpeersMask = NULL;
		AknIconUtils::CreateIconLC(fewpeersBmp, fewpeersMask, KBitmapFile,
			EMbmSymtrackerstateFew_peers, EMbmSymtrackerstateFew_peers_mask);		
		fewpeersIcon->SetBitmap(fewpeersBmp);
		fewpeersIcon->SetMask(fewpeersMask);
		CleanupStack::Pop(2);
		icons->AppendL(fewpeersIcon);
		CleanupStack::Pop();
	
		CFbsBitmap* manypeersBmp = NULL;
		CFbsBitmap* manypeersMask = NULL;
		AknIconUtils::CreateIconLC(manypeersBmp, manypeersMask, KBitmapFile,
			EMbmSymtrackerstateMany_peers, EMbmSymtrackerstateMany_peers_mask);		
		manypeersIcon->SetBitmap(manypeersBmp);
		manypeersIcon->SetMask(manypeersMask);
		CleanupStack::Pop(2);
		icons->AppendL(manypeersIcon);
		CleanupStack::Pop();

	CleanupStack::Pop(icons);

	iTorrentListBox->View()->SetListEmptyTextL(_L("(no torrents)"));

	iTorrentListBox->ItemDrawer()->ColumnData()->SetIconArray(icons);

    SetRect(aRect);
    ActivateL();
}

// Destructor
CSymTorrentTrackerTorrentListContainer::~CSymTorrentTrackerTorrentListContainer()
{
		delete iTorrentListBox;
}

// ---------------------------------------------------------
// CSymTorrentTrackerTorrentListContainer::TorrentListBox()
// Returns the listBox;
// ---------------------------------------------------------
//
CAknSingleGraphicStyleListBox* CSymTorrentTrackerTorrentListContainer::TorrentListBox()
{
	return iTorrentListBox;
}

// ---------------------------------------------------------
// CSymTorrentTrackerTorrentListContainer::SizeChanged()
// Called by framework when the view size is changed
// ---------------------------------------------------------
//
void CSymTorrentTrackerTorrentListContainer::SizeChanged()
{
    iTorrentListBox->SetRect(Rect());
}

// ---------------------------------------------------------
// CSymTorrentTrackerTorrentListContainer::CountComponentControls() const
// ---------------------------------------------------------
//
TInt CSymTorrentTrackerTorrentListContainer::CountComponentControls() const
{
    return 1; // return nbr of controls inside this container
}

// ---------------------------------------------------------
// CSymTorrentTrackerTorrentListContainer::ComponentControl(TInt aIndex) const
// ---------------------------------------------------------
//
CCoeControl* CSymTorrentTrackerTorrentListContainer::ComponentControl(TInt aIndex) const
{
	switch ( aIndex )
	{
		case 0:
			return iTorrentListBox;
		default:
			return NULL;
	}
}

// ---------------------------------------------------------
// CSymTorrentTrackerTorrentListContainer::Draw(const TRect& aRect) const
// ---------------------------------------------------------
//
void CSymTorrentTrackerTorrentListContainer::Draw(const TRect& /*aRect*/) const
{
    //CWindowGc& gc = SystemGc();
    // TODO: Add your drawing code here
    // example code...
    /*gc.SetPenStyle( CGraphicsContext::ENullPen );
    gc.SetBrushColor( KRgbGray );
    gc.SetBrushStyle( CGraphicsContext::ESolidBrush );
    gc.DrawRect( aRect );*/
}

// ---------------------------------------------------------
// CSymTorrentTrackerTorrentListContainer::HandleControlEventL(
//     CCoeControl* aControl,TCoeEvent aEventType)
// ---------------------------------------------------------
//
void CSymTorrentTrackerTorrentListContainer::HandleControlEventL(
    CCoeControl* /*aControl*/,TCoeEvent /*aEventType*/)
{
    // TODO: Add your control event handler code here
}

// ---------------------------------------------------------
// CSymTorrentTrackerTorrentListContainer::OfferKeyEventL(const TKeyEvent &aKeyEvent,
//	 TEventCode aType)
// ---------------------------------------------------------
//
TKeyResponse CSymTorrentTrackerTorrentListContainer::OfferKeyEventL(const TKeyEvent &aKeyEvent,TEventCode aType)
{
	 switch ( aKeyEvent.iCode )
	 {
		case EKeyUpArrow:
		{
			iTorrentListBox->OfferKeyEventL(aKeyEvent,aType);
			return EKeyWasConsumed;
			break;
		}
		case EKeyDownArrow:				
		{

			iTorrentListBox->OfferKeyEventL(aKeyEvent,aType);
			return EKeyWasConsumed;
			break;
		}
		case EKeyBackspace:
		{
			TInt currentitem=iTorrentListBox->CurrentItemIndex();
			
			if (currentitem>-1)
				if (iEikonEnv->QueryWinL(_L("Remove torrent?"), _L("")))
					RemoveTorrentL(currentitem);		
			return EKeyWasConsumed;		
			break;
		}
	    default:
		  return EKeyWasNotConsumed;		
	 }
}

// Insert a torrent to the list
void CSymTorrentTrackerTorrentListContainer::InsertTorrentL(const TDesC& aName, TTorrentStates aTorrentState)
{
	TBuf<200> insertString;		
	CDesCArray* itemArray = static_cast<CDesCArray*>
		(iTorrentListBox->Model()->ItemTextArray());
	
	insertString.Format(KTorrentListFormat,IconIndexFromTorrentState(aTorrentState),&aName);

	itemArray->InsertL(itemArray->Count(),insertString);
	iTorrentListBox->HandleItemAdditionL();
	iTorrentListBox->SetCurrentItemIndexAndDraw(itemArray->Count() - 1);
}

// Remove a torrent from the list
void CSymTorrentTrackerTorrentListContainer::RemoveTorrentL(TInt aIndex)
{
	TInt current=iTorrentListBox->CurrentItemIndex();

	CDesCArray* itemArray = static_cast<CDesCArray*>
		(iTorrentListBox->Model()->ItemTextArray());
	
	itemArray->Delete(aIndex);

	AknListBoxUtils::HandleItemRemovalAndPositionHighlightL(iTorrentListBox, aIndex, (aIndex == current));

	CSTTTorrentListItem* tempItem = iBaseData->GetTorrentListFirstItem();
	for (TInt i=0; i<aIndex; i++)
		tempItem=tempItem->GetNextSibling();
	iBaseData->DeleteListItem(tempItem->GetTorrentFileName());
}

// Refreshes one item of the torrentList
void CSymTorrentTrackerTorrentListContainer::RefreshTorrentListItem(CSTTTorrentListItem* aTorrentItem)
{
	//RefreshTorrentList();
	
	// not working correctly yet
	TInt index = 0;

	CSTTTorrentListItem* tempItem = iBaseData->GetTorrentListFirstItem();
	while (tempItem!=NULL && tempItem!=aTorrentItem)
	{
		tempItem=tempItem->GetNextSibling();
		index++;
	}

	RefreshTorrentListItem(index);
}

// Refreshes one item of the torrentList
void CSymTorrentTrackerTorrentListContainer::RefreshTorrentListItem(TInt aIndex)
{
	TInt current=iTorrentListBox->CurrentItemIndex();
	TBuf<200> insertString;		

	CSTTTorrentListItem* tempItem = iBaseData->GetTorrentListFirstItem();
	for (TInt i=0; i<aIndex; i++)
		if (tempItem)
			tempItem=tempItem->GetNextSibling();

	if (tempItem)
	{
		TPtr torrentFileName(tempItem->GetTorrentFileName());
		insertString.Format(KTorrentListFormat,IconIndexFromTorrentState(TorrentItemState(tempItem)),&torrentFileName);

		CDesCArray* itemArray = static_cast<CDesCArray*>
				(iTorrentListBox->Model()->ItemTextArray());
		itemArray->Delete(aIndex);
		AknListBoxUtils::HandleItemRemovalAndPositionHighlightL(iTorrentListBox, aIndex, (aIndex == current));
		itemArray->InsertL(aIndex, insertString);
		iTorrentListBox->HandleItemAdditionL();
	}
	else
		RefreshTorrentList();
}

// Clears the torrentList
void CSymTorrentTrackerTorrentListContainer::ClearTorrentList()
{
	CDesCArray* itemArray = static_cast<CDesCArray*>
			(iTorrentListBox->Model()->ItemTextArray());

	itemArray->Reset();

	DrawDeferred();
}

// Refreshs the torrentList
void CSymTorrentTrackerTorrentListContainer::RefreshTorrentList()
{
	ClearTorrentList();
	CSTTTorrentListItem* tempItem = iBaseData->GetTorrentListFirstItem();
	while (tempItem!=NULL)
	{
		InsertTorrentL(tempItem->GetTorrentFileName(),TorrentItemState(tempItem));
		tempItem=tempItem->GetNextSibling();
	}
}

TTorrentStates CSymTorrentTrackerTorrentListContainer::TorrentItemState(CSTTTorrentListItem* aTorrentItem)
{
	TTorrentStates torrentState;
	TInt torrentPeerNum;

	torrentPeerNum = aTorrentItem->PeerListLength();
	if (torrentPeerNum==0)
		torrentState = ETorNoPeer;
	else if (torrentPeerNum<=2)
		torrentState = ETorFewPeer;
	else 
		torrentState = ETorManyPeer;

	return torrentState;
}

TInt CSymTorrentTrackerTorrentListContainer::IconIndexFromTorrentState(TTorrentStates aTorrentState)
{
	if (iView->GetConnectorState()!=EConOffline)
	{
		return aTorrentState-1;
	}
	else
		return 0;
}
