/*****************************************************************************
 * Copyright (C) 2006 Pter Ekler, Zsolt Pszota
 * ---------------------------------------------------------------------------
 * This file is part of SymTracker
 *
 * SymTracker is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SymTracker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SymTracker; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************************/

// INCLUDE FILES
#include  <aknviewappui.h>
#include  <avkon.hrh>
#include <SymTracker.rsg>
#include  <eikbtgpc.h> 
#include  <eikclbd.h> 

#include  "SymTorrentTracker.hrh"
#include  "SymTorrentTrackerTorrentListView.h"
#include  "SymTorrentTrackerTorrentListContainer.h"
#include  "SymTorrentTrackerManager.h"
//#include  "SymTorrentAppUi.h"

// Constants
_LIT(KLitHideButtonText, "Hide");

// ================= MEMBER FUNCTIONS =======================

CSymTorrentTrackerTorrentListView::CSymTorrentTrackerTorrentListView(CSymTorrentTrackerManager* aTrackerManager)
 : iTrackerManager(aTrackerManager)
{	
}

// ---------------------------------------------------------
// CSymTorrentTrackerTorrentListView::ConstructL(const TRect& aRect)
// EPOC two-phased constructor
// ---------------------------------------------------------
//
void CSymTorrentTrackerTorrentListView::ConstructL()
{
    BaseConstructL( R_SYMTORRENTTRACKER_TORRENTLISTVIEW );
	iLastItemIndex=-1;
}

// ---------------------------------------------------------
// CSymTorrentTrackerTorrentListView::~CSymTorrentTrackerTorrentListView()
// destructor
// ---------------------------------------------------------
//
CSymTorrentTrackerTorrentListView::~CSymTorrentTrackerTorrentListView()
{
    if ( iContainer )
    {
        AppUi()->RemoveFromViewStack( *this, iContainer );
    }

    delete iContainer;
}

// ---------------------------------------------------------
// TUid CSymTorrentTrackerTorrentListView::Id()
// 
// ---------------------------------------------------------
//
TUid CSymTorrentTrackerTorrentListView::Id() const
{
    return TUid::Uid(ESymTorrentTrackerTorrentListView);
}

// ---------------------------------------------------------
// CSymTorrentTrackerTorrentListView::HandleCommandL(TInt aCommand)
// takes care of view command handling
// ---------------------------------------------------------
//
void CSymTorrentTrackerTorrentListView::HandleCommandL(TInt aCommand)
{   
    switch ( aCommand )
    {
		case EAknSoftkeyBack:
			iTrackerManager->setTitlePane(_L("SymTorrent"));
			//iTrackerManager->ViewChanger()->RemoveNaviPaneText();
			//iTrackerManager->ViewChanger()->ActivateViewL(ESymTorrentMainView);
			iTrackerManager->UiConn()->TrackerRemoveNaviPaneTextL();
			iTrackerManager->UiConn()->TrackerActiveViewL(ESymTorrentTrackerMainView);
			break;
		case ETrackerCmdSettings:
			iTrackerManager->UiConn()->TrackerActiveViewL(ESymTorrentTrackerSettingsView);
			//iTrackerManager->ViewChanger()->ActivateViewL(ESymTorrentSettingsView);
			break;
		case ETrackerCmdCreateTorrent:
			iTrackerManager->TrackerCmdCreateTorrent();		
			break;
		case ETrackerCmdStartTracker:
			AppUi()->HandleCommandL(ETrackerCmdStartTracker);	
			break;
		case ETrackerCmdStopTracker:
			iTrackerManager->TrackerCmdStopTracker();	
			break;
		case ETrackerCmdAddTorrent:
			iTrackerManager->TrackerCmdAddTorrent();
			break;
		case ETrackerCmdRemoveTorrent:
		{
			if (iEikonEnv->QueryWinL(_L("Remove torrent?"), _L("")))
				iContainer->RemoveTorrentL(iContainer->TorrentListBox()->CurrentItemIndex());		
			break;
		}
		default:
		{
			AppUi()->HandleCommandL( aCommand );
			break;
		}
	}
}

// ---------------------------------------------------------
// CSymTorrentTrackerTorrentListView::HandleClientRectChange()
// ---------------------------------------------------------
//
void CSymTorrentTrackerTorrentListView::HandleClientRectChange()
{
    if ( iContainer )
    {
        iContainer->SetRect( ClientRect() );
    }
}

// ---------------------------------------------------------
// CSymTorrentTrackerTorrentListView::DoActivateL(...)
// 
// --------------------------------------------------------
//
void CSymTorrentTrackerTorrentListView::DoActivateL(
   const TVwsViewId& /*aPrevViewId*/,TUid /*aCustomMessageId*/,
   const TDesC8& /*aCustomMessage*/)
{
	//CSymTorrentAppUi* appUi = static_cast<CSymTorrentAppUi*>(AppUi());
	//appUi->SetCurrentViewId(ESymTorrentTrackerTorrentListView);
	AppUi()->ActivateLocalViewL(TUid::Uid(ESymTorrentTrackerTorrentListView));
	
	TBool trackerRunning = (iTrackerManager->GetConnectorState() != EConOffline);
	
	UpdateTrackerStatusNaviPaneL();
	iTrackerManager->SetInTracker(ETrue);

    if (!iContainer)
    {
        iContainer = new (ELeave) CSymTorrentTrackerTorrentListContainer(iTrackerManager->GetBaseData(), this);
        iContainer->SetMopParent(this);
        iContainer->ConstructL( ClientRect() );
        AppUi()->AddToStackL( *this, iContainer );
		RefreshTorrentList();
		
		iTrackerManager->setTitlePane(_L("Tracker"));
    }
	if (iLastItemIndex != -1)
		iContainer->TorrentListBox()->SetCurrentItemIndex(iLastItemIndex);	
}

// ---------------------------------------------------------
// CSymTorrentTrackerTorrentListView::DoDeactivate()
// 
// ---------------------------------------------------------
//
void CSymTorrentTrackerTorrentListView::DoDeactivate()
{
    if ( iContainer )
    {
		iLastItemIndex=iContainer->TorrentListBox()->CurrentItemIndex();
        AppUi()->RemoveFromViewStack( *this, iContainer );		
    }
    
    delete iContainer;
    iContainer = NULL;
}

// From CAknView MenuObserver. Dynamic sets which menu is visible.
void CSymTorrentTrackerTorrentListView::DynInitMenuPaneL(
    TInt aResourceId,CEikMenuPane* aMenuPane)
{
	TBool startState = (iTrackerManager->GetConnectorState() != EConOffline);

    switch ( aResourceId )
    {
		case R_SYMTORRENTTRACKER_TORRENTLISTVIEW_MENU:
			aMenuPane->SetItemDimmed( ETrackerCmdRemoveTorrent, (iTrackerManager->GetBaseData()->GetTorrentListFirstItem()==NULL ));
			aMenuPane->SetItemDimmed( ETrackerCmdStartTracker, startState );
			aMenuPane->SetItemDimmed( ETrackerCmdStopTracker, !startState );
			//aMenuPane->SetItemDimmed( ETrackerCmdSettings, StartState );
			aMenuPane->SetItemDimmed( ETrackerCmdCreateTorrent, !startState );
			aMenuPane->SetItemDimmed( ETrackerCmdAddTorrent, !startState );
            break;
//		case R_SYMTRACKER_APP_MENU:
//			aMenuPane->SetItemDimmed( ESymTorrentCmdTracker, ETrue );
//			break;
		default:
            break;      
    }	
}

// From MSTTTorrentItemObserver

void CSymTorrentTrackerTorrentListView::TorrentPeersChanged(CSTTTorrentListItem* aTorrentItem)
{
	if (iContainer)
		iContainer->RefreshTorrentListItem(aTorrentItem);
}

// New functions

// Refreshs the torrentList
void CSymTorrentTrackerTorrentListView::RefreshTorrentList()
{
	if (iContainer)
	{
		iContainer->RefreshTorrentList();
	}
}

TConnectorSimpleStates CSymTorrentTrackerTorrentListView::GetConnectorState()
{
	return iTrackerManager->GetConnectorState();
}

void CSymTorrentTrackerTorrentListView::UpdateTrackerStatusNaviPaneL()
{
	//CSymTorrentAppUi* appUi = static_cast<CSymTorrentAppUi*>(AppUi());
	AppUi()->ActivateLocalViewL(TUid::Uid(ESymTorrentTrackerTorrentListView));
	
	TBool trackerRunning = (iTrackerManager->GetConnectorState() != EConOffline);
	
	if (trackerRunning)
		iTrackerManager->UiConn()->TrackerChangeNaviPaneTextL(_L("Tracker running"));
	else
		iTrackerManager->UiConn()->TrackerChangeNaviPaneTextL(_L("Tracker stopped"));
}


